    function SalesStatsWidgets(title="",id="",totalAmount=0) {
        var element = "#"+id;
        var size = 220;
        var data = [];
        var i=0;
        var colors = ["#29B6F6","#008fcf","#66BB6A","#EF5350","#006699","#ff0000","#FFBF00","#FF7F50","#DE3163","#9FE2BF","#40E0D0","#6495ED","#CCCCFF","#E74C3C","#9B59B6","#DAF7A6","#FFC300","#C70039","#00FF00","#0000FF"];
        $(element).each(function()
        { 
          $.each(this.attributes, function()
          {
              var val = this.value;
              var sp = val.split(":");
              if(sp[0]>0)
              {
                data.push({
                            status: sp[1], 
                            value:  sp[0],
                            color:  colors[i],
                            icon:  ""
                        });
                i++;
              }
          });
        });
        // Main variables
        var d3Container = d3.select(element),
            distance = 2, // reserve 2px space for mouseover arc moving
            radius = (size/2) - distance,
            sum = d3.sum(data, function(d) { return d.value; });

            if(totalAmount>0)
            {
                sum  = totalAmount;
            }

        // Tooltip
        // ------------------------------

        var tip = d3.tip()
            .attr('class', 'd3-tip')
            .offset([-10, 0])
            .direction('e')
            .html(function (d) {
                return "<ul class='list-unstyled mb-5'>" +
                    "<li>" + "<div class='text-size-base mb-5 mt-5'>" + d.data.icon + d.data.status + "</div>" + "</li>" +
                    "<li>" + "Total: &nbsp;" + "<span class='text-semibold pull-right'>" + d.value + "</span>" + "</li>" +
                    "<li>" + "Share: &nbsp;" + "<span class='text-semibold pull-right'>" + (100 / (sum / d.value)).toFixed(2) + "%" + "</span>" + "</li>" +
                "</ul>";
            });


        // Create chart
        // ------------------------------

        // Add svg element
        var container = d3Container.append("svg").call(tip);
        
        // Add SVG group
        var svg = container
            .attr("width", size)
            .attr("height", size)
            .append("g")
                .attr("transform", "translate(" + (size / 2) + "," + (size / 2) + ")");  


        // Construct chart layout
        // ------------------------------

        // Pie
        var pie = d3.layout.pie()
            .sort(null)
            .startAngle(Math.PI)
            .endAngle(3 * Math.PI)
            .value(function (d) { 
                return d.value;
            }); 

        // Arc
        var arc = d3.svg.arc()
            .outerRadius(radius)
            .innerRadius(radius / 1.35);


        //
        // Append chart elements
        //

        // Group chart elements
        var arcGroup = svg.selectAll(".d3-arc")
            .data(pie(data))
            .enter()
            .append("g") 
                .attr("class", "d3-arc")
                .style({
                    'stroke': '#fff',
                    'stroke-width': 2,
                    'cursor': 'pointer'
                });
        
        // Append path
        var arcPath = arcGroup
            .append("path")
            .style("fill", function (d) {
                return d.data.color;
            });


        //
        // Add interactions
        //

        // Mouse
        arcPath
            .on('mouseover', function(d, i) {

                // Transition on mouseover
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('elastic')
                    .attr('transform', function (d) {
                        d.midAngle = ((d.endAngle - d.startAngle) / 2) + d.startAngle;
                        var x = Math.sin(d.midAngle) * distance;
                        var y = -Math.cos(d.midAngle) * distance;
                        return 'translate(' + x + ',' + y + ')';
                    });

                $(element + ' [data-slice]').css({
                    'opacity': 0.3,
                    'transition': 'all ease-in-out 0.15s'
                });
                $(element + ' [data-slice=' + i + ']').css({'opacity': 1});
            })
            .on('mouseout', function(d, i) {

                // Mouseout transition
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('bounce')
                    .attr('transform', 'translate(0,0)');

                $(element + ' [data-slice]').css('opacity', 1);
            });

        // Animate chart on load
        arcPath
            .transition()
            .delay(function(d, i) {
                return i * 500;
            })
            .duration(500)
            .attrTween("d", function(d) {
                var interpolate = d3.interpolate(d.startAngle,d.endAngle);
                return function(t) {
                    d.endAngle = interpolate(t);
                    return arc(d);  
                }; 
            });


        //
        // Add text
        //

        // Total
        svg.append('text').attr('class', 'text-muted').attr({'class': 'half-donut-total','text-anchor': 'middle','dy': -13}).style({'font-size': '12px','fill': '#999'}).text(title);

        // Count
        svg
            .append('text')
            .attr('class', 'half-donut-count')
            .attr('text-anchor', 'middle')
            .attr('dy', 14)
            .style({
                'font-size': '21px',
                'font-weight': 500
            });

        // Animate count
        svg.select('.half-donut-count')
            .transition()
            .duration(1500)
            .ease('linear')
            .tween("text", function(d) {
                var i = d3.interpolate(this.textContent, sum);

                return function(t) {
                    this.textContent = d3.format(",d")(Math.round(i(t)));
                };
            });


        //
        // Add legend
        //

        // Append list
        var legend = d3.select(element)
            .append('ul')
            .attr('class', 'chart-widget-legend')
            .selectAll('li')
            .data(pie(data))
            .enter()
            .append('li')
            .attr('data-slice', function(d, i) {
                return i;
            })
            .attr('style', function(d, i) {
                return 'border-bottom: solid 2px ' + d.data.color;
            })
            .text(function(d, i) {
                return d.data.status + ': ';
            });

        // Append text
        legend.append('span')
            .text(function(d, i) {
                return d.data.value;
            });
    }
    // Chart setup
    function TicketsStatsWidgets() {
        var element = "#TicketsStatsWidgets";
        var size = 220;
        
        var TicketsData = $(element).attr("ws-total");
        TicketsData = TicketsData.split(':');
        var  totalTickets = TicketsData[0];
        var  totalTicketsTitle = TicketsData[1];
        
        var awaitingData = $(element).attr("ws-awaiting");
        awaitingData = awaitingData.split(':');
        var  awaitingTickets = awaitingData[0];
        var  awaitingTicketsTitle = awaitingData[1];
        
        var assignemeData = $(element).attr("ws-assigneme");
        assignemeData = assignemeData.split(':');
        var  assignemeTickets = assignemeData[0];
        var  assignemeTicketsTitle = assignemeData[1];
        
        var tawaitingData = $(element).attr("ws-today-awaiting");
        tawaitingData = tawaitingData.split(':');
        var  tawaitingTickets = tawaitingData[0];
        var  tawaitingTicketsTitle = tawaitingData[1];
        
        var closedData = $(element).attr("ws-closed");
        closedData = closedData.split(':');
        var  closedTickets = closedData[0];
        var  closedTicketsTitle = closedData[1];
        
        // Basic setup
        // ------------------------------

        // Add data set
        var data = [
            {
                "status": awaitingTicketsTitle,
                "icon": "<i class='status-mark border-blue-300 position-left'></i>",
                "value": awaitingTickets,
                "color": "#29B6F6"
            }, {
                "status": assignemeTicketsTitle,
                "icon": "<i class='status-mark border-success-300 position-left'></i>",
                "value": assignemeTickets,
                "color": "#008fcf"
            }, {
                "status": tawaitingTicketsTitle,
                "icon": "<i class='status-mark border-info-300 position-left'></i>",
                "value": tawaitingTickets,
                "color": "#66BB6A"
            }, {
                "status": closedTicketsTitle,
                "icon": "<i class='status-mark border-danger-300 position-left'></i>",
                "value": closedTickets,
                "color": "#EF5350"
            }
        ];

        // Main variables
        var d3Container = d3.select(element),
            distance = 2, // reserve 2px space for mouseover arc moving
            radius = (size/2) - distance,
            sum = d3.sum(data, function(d) { return d.value; });
            sum -=Number(tawaitingTickets)+Number(assignemeTickets);


        // Tooltip
        // ------------------------------

        var tip = d3.tip()
            .attr('class', 'd3-tip')
            .offset([-10, 0])
            .direction('e')
            .html(function (d) {
                return "<ul class='list-unstyled mb-5'>" +
                    "<li>" + "<div class='text-size-base mb-5 mt-5'>" + d.data.icon + d.data.status + "</div>" + "</li>" +
                    "<li>" + "Total: &nbsp;" + "<span class='text-semibold pull-right'>" + d.value + "</span>" + "</li>" +
                    "<li>" + "Share: &nbsp;" + "<span class='text-semibold pull-right'>" + (100 / (sum / d.value)).toFixed(2) + "%" + "</span>" + "</li>" +
                "</ul>";
            });


        // Create chart
        // ------------------------------

        // Add svg element
        var container = d3Container.append("svg").call(tip);
        
        // Add SVG group
        var svg = container
            .attr("width", size)
            .attr("height", size)
            .append("g")
                .attr("transform", "translate(" + (size / 2) + "," + (size / 2) + ")");  


        // Construct chart layout
        // ------------------------------

        // Pie
        var pie = d3.layout.pie()
            .sort(null)
            .startAngle(Math.PI)
            .endAngle(3 * Math.PI)
            .value(function (d) { 
                return d.value;
            }); 

        // Arc
        var arc = d3.svg.arc()
            .outerRadius(radius)
            .innerRadius(radius / 1.35);


        //
        // Append chart elements
        //

        // Group chart elements
        var arcGroup = svg.selectAll(".d3-arc")
            .data(pie(data))
            .enter()
            .append("g") 
                .attr("class", "d3-arc")
                .style({
                    'stroke': '#fff',
                    'stroke-width': 2,
                    'cursor': 'pointer'
                });
        
        // Append path
        var arcPath = arcGroup
            .append("path")
            .style("fill", function (d) {
                return d.data.color;
            });


        //
        // Add interactions
        //

        // Mouse
        arcPath
            .on('mouseover', function(d, i) {

                // Transition on mouseover
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('elastic')
                    .attr('transform', function (d) {
                        d.midAngle = ((d.endAngle - d.startAngle) / 2) + d.startAngle;
                        var x = Math.sin(d.midAngle) * distance;
                        var y = -Math.cos(d.midAngle) * distance;
                        return 'translate(' + x + ',' + y + ')';
                    });

                $(element + ' [data-slice]').css({
                    'opacity': 0.3,
                    'transition': 'all ease-in-out 0.15s'
                });
                $(element + ' [data-slice=' + i + ']').css({'opacity': 1});
            })
            .on('mouseout', function(d, i) {

                // Mouseout transition
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('bounce')
                    .attr('transform', 'translate(0,0)');

                $(element + ' [data-slice]').css('opacity', 1);
            });

        // Animate chart on load
        arcPath
            .transition()
            .delay(function(d, i) {
                return i * 500;
            })
            .duration(500)
            .attrTween("d", function(d) {
                var interpolate = d3.interpolate(d.startAngle,d.endAngle);
                return function(t) {
                    d.endAngle = interpolate(t);
                    return arc(d);  
                }; 
            });


        //
        // Add text
        //

        // Total
        svg.append('text').attr('class', 'text-muted').attr({'class': 'half-donut-total','text-anchor': 'middle','dy': -13}).style({'font-size': '12px','fill': '#999'}).text(totalTicketsTitle);

        // Count
        svg
            .append('text')
            .attr('class', 'half-donut-count')
            .attr('text-anchor', 'middle')
            .attr('dy', 14)
            .style({
                'font-size': '21px',
                'font-weight': 500
            });

        // Animate count
        svg.select('.half-donut-count')
            .transition()
            .duration(1500)
            .ease('linear')
            .tween("text", function(d) {
                var i = d3.interpolate(this.textContent, sum);

                return function(t) {
                    this.textContent = d3.format(",d")(Math.round(i(t)));
                };
            });


        //
        // Add legend
        //

        // Append list
        var legend = d3.select(element)
            .append('ul')
            .attr('class', 'chart-widget-legend')
            .selectAll('li')
            .data(pie(data))
            .enter()
            .append('li')
            .attr('data-slice', function(d, i) {
                return i;
            })
            .attr('style', function(d, i) {
                return 'border-bottom: solid 2px ' + d.data.color;
            })
            .text(function(d, i) {
                return d.data.status + ': ';
            });

        // Append text
        legend.append('span')
            .text(function(d, i) {
                return d.data.value;
            });
    }



 // Pie arc with legend
    // ------------------------------

    // Initialize chart
    
    function StafsStatsWidgets(){
        var element = "#StafsStatsWidgets";
        var ClientsData = $(element).attr("ws-total");
        ClientsData = ClientsData.split(':');
        var  totalClients = ClientsData[0];
        var  totalClientsTitle = ClientsData[1];

        var onlineData = $(element).attr("ws-online");
        onlineData = onlineData.split(':');
        var  onlineClients = onlineData[0];
        var  onlineClientsTitle = onlineData[1];

        var staffData = $(element).attr("ws-staff");
        staffData = staffData.split(':');
        var  staffClients = staffData[0];
        var  staffClientsTitle = staffData[1];
        // ------------------------------

        // Add data set
        var data = [
            {
                "status": onlineClientsTitle,
                "icon": "<i class='status-mark border-blue-300 position-left'></i>",
                "value": onlineClients,
                "color": "#29B6F6"
            }, {
                "status": staffClientsTitle,
                "icon": "<i class='status-mark border-success-300 position-left'></i>",
                "value": staffClients,
                "color": "#66BB6A"
            }
        ];
        ClientStaffStatsWidgets(element,data,totalClientsTitle);
    }
    function ClientStatsWidgets() {
        var element = "#ClientStatsWidgets";
        var ClientsData = $(element).attr("ws-total");
        ClientsData = ClientsData.split(':');
        var  totalClients = ClientsData[0];
        var  totalClientsTitle = ClientsData[1];

        var ActiveData = $(element).attr("ws-active");
        ActiveData = ActiveData.split(':');
        var  activeClients = ActiveData[0];
        var  activeClientsTitle = ActiveData[1];

        var InActiveData = $(element).attr("ws-inactive");
        InActiveData = InActiveData.split(':');
        var  inactiveClients = InActiveData[0];
        var  inactiveClientsTitle = InActiveData[1];

        var ClosedData = $(element).attr("ws-closed");
        ClosedData = ClosedData.split(':');
        var  closedClients = ClosedData[0];
        var  closedClientsTitle = ClosedData[1];

        var PendingData = $(element).attr("ws-pending");
        PendingData = PendingData.split(':');
        var  pendingClients = PendingData[0];
        var  pendingClientsTitle = PendingData[1];
        // Basic setup
        // ------------------------------

        // Add data set
        var data = [
            {
                "status": activeClientsTitle,
                "icon": "<i class='status-mark border-blue-300 position-left'></i>",
                "value": activeClients,
                "color": "#29B6F6"
            }, {
                "status": inactiveClientsTitle,
                "icon": "<i class='status-mark border-success-300 position-left'></i>",
                "value": inactiveClients,
                "color": "#66BB6A"
            }, {
                "status": closedClientsTitle,
                "icon": "<i class='status-mark border-danger-300 position-left'></i>",
                "value": closedClients,
                "color": "#EF5350"
            }, {
                "status": pendingClientsTitle,
                "icon": "<i class='status-mark border-danger-300 position-left'></i>",
                "value": pendingClients,
                "color": "#EF5350"
            }
        ];
        ClientStaffStatsWidgets(element,data,totalClientsTitle);
    }
    // Chart setup
    function ClientStaffStatsWidgets(element,data,totalClientsTitle) {
        var size = 170;
        

        // Main variables
        var d3Container = d3.select(element),
            distance = 2, // reserve 2px space for mouseover arc moving
            radius = (size/2) - distance,
            sum = d3.sum(data, function(d) { return d.value; });
        // Tooltip
        // ------------------------------
        var tip = d3.tip().attr('class', 'd3-tip').offset([-10, 0]).direction('e').html(function (d) {return "<ul class='list-unstyled mb-5'><li><div class='text-size-base mb-5 mt-5'>" + d.data.icon + d.data.status + "</div></li><li>Total: &nbsp;" + "<span class='text-semibold pull-right'>" + d.value + "</span>" + "</li><li>" + "Share: &nbsp;" + "<span class='text-semibold pull-right'>" + (100 / (sum / d.value)).toFixed(2) + "%" + "</span>" + "</li></ul>";});
        // Create chart
        // ------------------------------
        // Add svg element
        var container = d3Container.append("svg").call(tip);
        // Add SVG group
        var svg = container.attr("width", size).attr("height", size / 2).append("g").attr("transform", "translate(" + (size / 2) + "," + (size / 2) + ")");  
        // Construct chart layout
        // ------------------------------
        // Pie
        var pie = d3.layout.pie().sort(null).startAngle(-Math.PI / 2).endAngle(Math.PI / 2).value(function (d) { return d.value;}); 
        // Arc
        var arc = d3.svg.arc().outerRadius(radius).innerRadius(radius / 1.3);



        //
        // Append chart elements
        //

        // Group chart elements
        var arcGroup = svg.selectAll(".d3-arc").data(pie(data)).enter().append("g").attr("class", "d3-arc").style({'stroke': '#fff','stroke-width': 2,'cursor': 'pointer'});
        
        // Append path
        var arcPath = arcGroup.append("path").style("fill", function (d) {return d.data.color;});


        //
        // Interactions
        //

        // Mouse
        arcPath
            .on('mouseover', function(d, i) {

                // Transition on mouseover
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('elastic')
                    .attr('transform', function (d) {
                        d.midAngle = ((d.endAngle - d.startAngle) / 2) + d.startAngle;
                        var x = Math.sin(d.midAngle) * distance;
                        var y = -Math.cos(d.midAngle) * distance;
                        return 'translate(' + x + ',' + y + ')';
                    });

                $(element + ' [data-slice]').css({
                    'opacity': 0.3,
                    'transition': 'all ease-in-out 0.15s'
                });
                $(element + ' [data-slice=' + i + ']').css({'opacity': 1});
            })
            .on('mouseout', function(d, i) {

                // Mouseout transition
                d3.select(this)
                .transition()
                    .duration(500)
                    .ease('bounce')
                    .attr('transform', 'translate(0,0)');

                $(element + ' [data-slice]').css('opacity', 1);
            });

        // Animate chart on load
        arcPath
            .transition()
                .delay(function(d, i) {
                    return i * 500;
                })
                .duration(500)
                .attrTween("d", function(d) {
                    var interpolate = d3.interpolate(d.startAngle,d.endAngle);
                    return function(t) {
                        d.endAngle = interpolate(t);
                        return arc(d);  
                    }; 
                });


        //
        // Append total text
        //

        svg.append('text')
            .attr('class', 'text-muted')
            .attr({
                'class': 'half-donut-total',
                'text-anchor': 'middle',
                'dy': -33
            })
            .style({
                'font-size': '12px',
                'fill': '#999'
            })
            .text(totalClientsTitle);


        //
        // Append count
        //

        // Text
        svg
            .append('text')
            .attr('class', 'half-conut-count')
            .attr('text-anchor', 'middle')
            .attr('dy', -5)
            .style({
                'font-size': '21px',
                'font-weight': 500
            });

        // Animation
        svg.select('.half-conut-count')
            .transition()
            .duration(1500)
            .ease('linear')
            .tween("text", function(d) {
                var i = d3.interpolate(this.textContent, sum);

                return function(t) {
                    this.textContent = d3.format(",d")(Math.round(i(t)));
                };
            });


        //
        // Legend
        //

        // Add legend list
        var legend = d3.select(element)
            .append('ul')
            .attr('class', 'chart-widget-legend')
            .selectAll('li')
            .data(pie(data))
            .enter()
            .append('li')
            .attr('data-slice', function(d, i) {
                return i;
            })
            .attr('style', function(d, i) {
                return 'border-bottom: solid 2px ' + d.data.color;
            })
            .text(function(d, i) {
                return d.data.status + ': ';
            });

        // Legend text
        legend.append('span')
            .text(function(d, i) {
                return d.data.value;
            });
    }