(function($) {
  $.fn.wsSelect = function() {
    // Loop through each select element
    this.each(function(index) {
      var $select = $(this);
      // Check if custom select has already been initialized
      if ($select.data('wsSelectInitialized')) return; // Skip initialization if already initialized
      // Set flag to indicate initialization
      $select.data('wsSelectInitialized', true);
      
      // Generate unique custom class names
      var customSelectClass = 'ws-select-' + (index + 1);
      var selectBoxClass = $select.attr('class') + '-ws-' + (index + 1); // Append '-ws-index' to existing classes
      
      // Create custom select elements
      var $customSelect = $('<div class="ws-select-main ' + customSelectClass + '"></div>');
      var $dropdown = $('<div class="ws-select-dropdown"></div>');
      var $selectedText = $('<div class="selected-text"><div class="selected-value"></div><div class="highlighted-value"></div></div>');
      var $searchContainer = $('<div class="ws-select-search"></div>'); // Wrap search input in a div
      var $searchInput = $('<input type="text" class="ws-select-search-field" placeholder="Search...">'); // Change class name
      var $optionsContainer = $('<ul class="ws-select-options"></ul>');

      // Append elements to the custom select container
      $customSelect.append($selectedText);
      $customSelect.append($dropdown);
      $dropdown.append($searchContainer); // Append search container
      $searchContainer.append($searchInput); // Append search input to search container
      $dropdown.append($optionsContainer);

      // Populate options
      if ($select.find('optgroup').length > 0) {
        // If optgroup exists, structure options accordingly
        $select.find('optgroup').each(function() {
          var $optgroup = $(this);
          var optgroupLabel = $optgroup.attr('label');
          var $optgroupLi = $('<li class="ws-optgroup" data-optgroup-label="' + optgroupLabel + '"><span>' + optgroupLabel + '</span><ul></ul></li>');
          $optgroup.find('option').each(function() {
            var $option = $(this);
            var value = $option.val();
            var text = $option.text();
            var classes = $option.attr('class'); // Get custom classes from option
            var $optionLi = $('<li data-value="' + value + '">' + text + '</li>');
            if (classes) {
              $optionLi.addClass(classes); // Apply custom classes to li element
            }
            $optgroupLi.find('ul').append($optionLi);
            // Check if the option is initially selected
            if ($option.is(':selected')) {
              $optionLi.addClass('highlighted');
              $optionLi.addClass('selected');
              var selectedText = text.trim() !== '' ? text : 'Choose....'; // Check if text is empty
              $selectedText.find('.selected-value').text(selectedText);
            }
          });
          $optionsContainer.append($optgroupLi);
        });
      } else {
        // If optgroup does not exist, treat options as direct children of select box
        $select.find('option').each(function() {
          var $option = $(this);
          var value = $option.val();
          var text = $option.text();
          var classes = $option.attr('class'); // Get custom classes from option
          var $optionLi = $('<li data-value="' + value + '">' + text + '</li>');
          if (classes) {
            $optionLi.addClass(classes); // Apply custom classes to li element
          }
          $optionsContainer.append($optionLi);
          // Check if the option is initially selected
          if ($option.is(':selected')) {
            $optionLi.addClass('highlighted');
            $optionLi.addClass('selected');
            var selectedText = text.trim() !== '' ? text : 'Choose....'; // Check if text is empty
            $selectedText.find('.selected-value').text(selectedText);
          }
        });
      }

// Check if any option is initially selected
var $initialSelectedOption = $select.find('option:selected');
if ($initialSelectedOption.length <= 0) {
  // No option is selected, set selected-value to "Choose"
  var defaultText = '';
  var $firstOption = $select.find('option:first');
  if ($firstOption.length > 0) {
    // If the first option exists, set selected-value to its text
    defaultText = $firstOption.text();
  } else {
    // If no options exist, set selected-value to "Choose"
    defaultText = 'Choose';
  }
  $selectedText.find('.selected-value').text(defaultText);
} else {
  // An option is initially selected, set selected-value to its text
  $selectedText.find('.selected-value').text($initialSelectedOption.text());
}



      // Hide original select
      $select.hide();

      // Insert custom select after the original select
      $select.after($customSelect);

      // Event handling for custom select
      $selectedText.on('click', function(e) {
        e.stopPropagation(); // Prevent event bubbling
        $dropdown.toggleClass("opened"); // Toggle dropdown
        $searchInput.focus(); // Focus search input
        var selectBoxHeight = $customSelect.height(); // Get height of the select box
        $dropdown.css('top', selectBoxHeight); // Set top property of dropdown
        $customSelect.toggleClass("opened"); // Toggle 'opened' class to custom select div
      });

    $optionsContainer.on('click', 'li', function() {
      var $li = $(this);
      
      // Check if clicked element is an option or an optgroup
      if (!$li.hasClass('ws-optgroup')) {
        var value = $li.data('value');
        var text = $li.text();
        
        $selectedText.find('.selected-value').text(text); // Update selected value
        $searchInput.val(text);
        $select.val(value).trigger('change'); // Update select value and trigger change event
        
        $optionsContainer.find('li').removeClass('highlighted'); // Remove 'highlighted' class from all li elements
        $li.addClass('highlighted'); // Add 'highlighted' class to the clicked li
        $optionsContainer.find('li.selected').removeClass('selected'); // Remove 'selected' class from all li elements
        $li.addClass('selected'); // Add 'selected' class to the clicked li
        
        $dropdown.removeClass("opened"); // Hide dropdown
        $customSelect.removeClass("opened"); // Remove 'opened' class from custom select div
      }
    });


      // Search functionality
      $searchInput.on('input', function() {
        var searchValue = $(this).val().toLowerCase();
        $optionsContainer.children('li').each(function() {
          var optionText = $(this).text().toLowerCase();
          $(this).toggle(optionText.includes(searchValue));
        });
      });

      // Close the dropdowns except the one clicked or clicked within a dropdown
      $(document).on('click', function(e) {
        var $clickedDropdown = $(e.target).closest('.ws-select-main');
        if ($clickedDropdown.length) {
          // Clicked within a dropdown or on a dropdown itself
          $('.ws-select-main').not($clickedDropdown).each(function() {
            var $dropdown = $(this).find('.ws-select-dropdown');
            var $customSelect = $(this);
            $dropdown.removeClass("opened"); // Hide dropdown
            $customSelect.removeClass("opened"); // Remove 'opened' class from custom select div
          });
        } else {
          // Clicked outside any dropdown
          $('.ws-select-main').each(function() {
            var $dropdown = $(this).find('.ws-select-dropdown');
            var $customSelect = $(this);
            $dropdown.removeClass("opened"); // Hide dropdown
            $customSelect.removeClass("opened"); // Remove 'opened' class from custom select div
          });
        }
      });
      // Add custom classes to select box and custom select div
      $select.addClass(selectBoxClass);
      $customSelect.addClass(customSelectClass);
    });
    return this;
  };
})(jQuery);
