class WsTables {
    constructor(containerId, options) {
        this.container = document.getElementById(containerId);
        this.options = options;
        this.currentPage = 1;
        this.data = [];
        this.totalRecords = 0;
        this.rowsPerPage = 10; // Default, will be updated from server response
        this.sortField = options.defaultSortField || ''; // Default sort field
        this.sortOrder = 'asc'; // Default sort order
        this.cssClasses = options.cssClasses || {};
        this.emptyMsg = options.emptyMsg || '';
        this.customParams = options.customParams || {}; // Custom parameters
        this.enableCheckbox = options.enableCheckbox || false; // Checkbox feature
        this.checkboxOptions = options.checkboxOptions || {}; // Checkbox options
        this.bodyCheckboxWidth = this.checkboxOptions.bodyCheckboxWidth || '15px';
        this.init();
    }

    init() {
        this.fetchData(this.currentPage);
    }

    // Function to add ordinal suffix to day
    addOrdinalSuffix(day)
    {
        if (day > 3 && day < 21) return day + 'th';
        switch (day % 10) {
            case 1: return day + 'st';
            case 2: return day + 'nd';
            case 3: return day + 'rd';
            default: return day + 'th';
        }
    }

    formatDate(dateString, format) {
        const date = new Date(dateString);
        const day = date.getDate().toString().padStart(2, '0');
        const month = (date.getMonth() + 1).toString().padStart(2, '0');
        const year = date.getFullYear().toString();

        const lastSpaceIndex = format.lastIndexOf(' ');
        const dateFormat = format.slice(0, lastSpaceIndex).trim(); // Date format part
        const timeFormat = format.slice(lastSpaceIndex).trim();    // Time format part        

        let formattedDate = '';
        let formattedTime = '';

        // Check for time format in the provided format string
        const hasTimeFormat = /[Hhms]/.test(format);
        // Format month names
        const monthNames = [
            'January', 'February', 'March', 'April', 'May', 'June', 
            'July', 'August', 'September', 'October', 'November', 'December'
        ];
        const shortMonthNames = [
            'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 
            'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
        ];
        const monthName = monthNames[date.getMonth()];
        const shortMonthName = shortMonthNames[date.getMonth()];

        // Format day with ordinal suffix
        const formattedDay = this.addOrdinalSuffix(day);

        // Determine day of week names
        const dayOfWeekNames = [
            'Sunday', 'Monday', 'Tuesday', 'Wednesday', 
            'Thursday', 'Friday', 'Saturday'
        ];
        const shortDayOfWeekNames = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
        const dayOfWeekName = dayOfWeekNames[date.getDay()];
        const shortDayOfWeekName = shortDayOfWeekNames[date.getDay()];        
        // Format date based on the date-only format or date-time format
        switch (dateFormat) {
            case 'jS F Y':
                formattedDate = `${formattedDay} ${monthName} ${year}`;
                break;
            case 'jS M Y':
                formattedDate = `${formattedDay} ${shortMonthName} ${year}`;
                break;
            case 'l, F jS, Y':
                formattedDate = `${dayOfWeekName}, ${monthName} ${formattedDay}, ${year}`;
                break;
            case 'D, F jS, Y':
                formattedDate = `${shortDayOfWeekName}, ${monthName} ${formattedDay}, ${year}`;
                break;
            case 'l, M jS, Y':
                formattedDate = `${dayOfWeekName}, ${shortMonthName} ${formattedDay}, ${year}`;
                break;
            case 'D, M jS, Y':
                formattedDate = `${shortDayOfWeekName}, ${shortMonthName} ${formattedDay}, ${year}`;            
                break;
            case 'd M, Y':
                formattedDate = `${day} ${shortMonthName}, ${year}`;
                break;
            case 'D/M/Y':
                formattedDate = `${day}/${month}/${year}`;
                break;
            case 'd/m/Y':
                formattedDate = `${day}/${month}/${year}`;
                break;
            case 'D.M.Y':
                formattedDate = `${day}.${month}.${year}`;
                break;
            case 'd.m.Y':
                formattedDate = `${day}.${month}.${year}`;
                break;
            case 'D-M-Y':
                formattedDate = `${day}-${month}-${year}`;
                break;
            case 'd-m-Y':
                formattedDate = `${day}-${month}-${year}`;
                break;
            case 'M/D/Y':
                formattedDate = `${month}/${day}/${year}`;
                break;
            case 'm/d/Y':
                formattedDate = `${month}/${day}/${year}`;
                break;
            case 'Y/M/D':
                formattedDate = `${year}/${month}/${day}`;
                break;
            case 'Y/m/d':
                formattedDate = `${year}/${month}/${day}`;
                break;
            case 'Y-M-D':
                formattedDate = `${year}-${month}-${day}`;
                break;
            case 'Y-m-d':
                formattedDate = `${year}-${month}-${day}`;
                break;
            default:
                formattedDate = dateString; // Default to returning the original string if format is not recognized
                break;
        }

        // Format time if time components are present in the format string
        if (hasTimeFormat) {
            const hours = date.getHours().toString().padStart(2, '0');
            const minutes = date.getMinutes().toString().padStart(2, '0');
            const seconds = date.getSeconds().toString().padStart(2, '0');
            formattedTime = `${hours}:${minutes}:${seconds}`;
        }

        // Combine date and time if time components are present
        if (hasTimeFormat) {
            return formattedDate + ' ' + formattedTime;
        } else {
            return formattedDate;
        }
    }

      loader(ivbloader,type=2,LoaderText=' Request inprocess, Please Wait')
        {
            var element = $(ivbloader);
            if(element.length == 0)
            {
                var start = ivbloader.charAt(0);
                var text = ivbloader.substring(1);
                if(start == "#")
                {
                    $('body').append('<div id="'+text+'"></div>');
                }else{
                    $('body').append('<div class="'+text+'"></div>');
                }
            }
            var isLoaderExist = $(".wivbloader").length;
            if(isLoaderExist == 0)
            {
                //var LoaderIcon ='<i class="icon-spinner9 spinner"></i>';
                var LoaderIcon ='<img src="/backend/assets/images/load.gif" style="width:62px;height:62px;"  title="loader" alt="loader">';
                var html = '<span class="wivbloader">';
                html += '<div class="blockUI blockOverlay" style="z-index: 10001; border: none; margin: 0px; padding: 0px; width: 100%; height: 100%; top: 0px; left: 0px; background-color: rgb(255, 255, 255,.4);position: fixed;"></div>';
                html += '<div class="blockUI blockMsg blockElement" style="position: fixed;padding: 0px;margin: 0px;width: 50%;top: 50%;left: 25%;text-align: center;color: #006699;background: #fff;border: 3px solid #ccc;border-radius: 10px;z-index: 10020;font-weight: bold;">'+LoaderIcon+' '+LoaderText+'</div>';
                html += '</span>';
                if(type == 1) $(ivbloader).append(html);
            }
             if(type == 2) setTimeout(function() {$('.wivbloader').remove();}, 500); 
        }
    async fetchData(page)
    {
        this.loader('.ivContentbloader',1); 
        try {
            const formData = new FormData();
            formData.append('page', page);
            formData.append('sortField', this.sortField);
            formData.append('sortOrder', this.sortOrder);
            // Append custom parameters
            for (const key in this.customParams) {
                formData.append(key, this.customParams[key]);
            }
            const response = await fetch(this.options.dataUrl, {
                method: 'POST',
                body: formData
            });
            const data = await response.json();
            this.data = data.records;
            this.totalRecords = data.totalRecords;
            this.rowsPerPage = data.rowsPerPage; // Update rowsPerPage from server response
            this.renderTable();
            if (this.totalRecords > 0) this.renderPagination();            
        } catch (error) {
            this.container.innerHTML = '<div class="gracefulexit">Error fetching data: ' + error + '</div>';
            this.loader('.ivContentbloader',2); 
        }
    }

    renderTable() {
        // Clear previous table
        this.container.innerHTML = '';
        if (this.totalRecords === 0) {
            this.container.innerHTML = '<div class="gracefulexit">' + this.emptyMsg + '</div>';
            this.loader('.ivContentbloader',2); 
        } else {
            const table = document.createElement('table');
            this.addCssClasses(table, 'ws-table');
            this.addCssClasses(table, this.cssClasses.table);
            const thead = this.createTableHeader();
            const tbody = this.createTableBody();
            table.appendChild(thead);
            table.appendChild(tbody);
            this.container.appendChild(table);
            this.loader('.ivContentbloader',2); 
        }
    }

    createTableHeader() {
        const thead = document.createElement('thead');
        this.addCssClasses(thead, this.cssClasses.thead);

        const row = document.createElement('tr');
        this.addCssClasses(row, 'customTableTh');
        this.addCssClasses(row, this.cssClasses.headerRow);
        // Add checkbox column if enabled
        if (this.enableCheckbox)
        {
            const th = document.createElement('th');
            th.style.width = this.bodyCheckboxWidth; // Example: Set custom width for the checkbox column
            const checkbox = document.createElement('input');
            checkbox.type = 'checkbox';
            checkbox.className = 'ws-cbox-all';
            checkbox.addEventListener('click', (event) => {
                this.toggleAllCheckboxes(event.target.checked);
            });
            th.appendChild(checkbox);
            row.appendChild(th);
        }

        this.options.columns.forEach(col => {
            const th = document.createElement('th');
            th.textContent = col.label;
            this.addCssClasses(th, this.cssClasses.headerCell);

            // Add sorting functionality
            if (col.sortable) {
                th.classList.add('sortable');
                th.addEventListener('click', () => {
                    this.updateSortField(col.field);
                });
                this.updateSortIcons(th, col.field);
            }
            if(col.width)
            {
                th.style.width = col.width;
            }

            row.appendChild(th);
        });

        thead.appendChild(row);
        return thead;
    }

    createTableBody() {
        const tbody = document.createElement('tbody');
        this.addCssClasses(tbody, this.cssClasses.tbody);

        this.data.forEach(item => {
            const row = document.createElement('tr');
            this.addCssClasses(row, this.cssClasses.bodyRow);
            // Add checkbox column if enabled
            if (this.enableCheckbox) {
                const td = document.createElement('td');
                td.style.width = this.bodyCheckboxWidth; // Example: Set custom width for the checkbox column
                const checkbox = document.createElement('input');
                checkbox.type = 'checkbox';
                checkbox.className = this.checkboxOptions.bodyCheckboxClass || 'ws-cbox';
                checkbox.name = this.checkboxOptions.bodyCheckboxName || 'ws-cbox-name';
                checkbox.value = item.id; // Assuming each item has a unique ID
                td.appendChild(checkbox);
                row.appendChild(td);
            }
            this.options.columns.forEach(col => {
                const td = document.createElement('td');
                if (col.action) {
                    td.innerHTML = col.action(item[col.field]);
                }else if (col.format)
                {
                    td.innerHTML=this.formatDate(item[col.field],col.format);
                } else {
                    td.innerHTML = item[col.field];
                }
                this.addCssClasses(td, this.cssClasses.bodyCell);
                if(col.classes)
                {
                    this.addCssClasses(td, col.classes);
                }
                row.appendChild(td);
            });
            tbody.appendChild(row);
        });

        return tbody;
    }
    toggleAllCheckboxes(checked) {
        const checkboxes = this.container.querySelectorAll('.ws-cbox');
        checkboxes.forEach(checkbox => {
            checkbox.checked = checked;
        });
    }
    renderPagination() {
        // Clear previous pagination
        const existingPagination = document.querySelector('.ws-pagination-main');
        if (existingPagination) {
            existingPagination.remove();
        }

        // Main pagination div
        const paginationMain = document.createElement('div');
        paginationMain.classList.add('ws-pagination-main');
        this.container.appendChild(paginationMain);

        // Pagination info div
        const paginationInfo = document.createElement('div');
        paginationInfo.classList.add('pagination-info');
        paginationMain.appendChild(paginationInfo);

        // Pagination ul
        const paginationUl = document.createElement('ul');
        this.addCssClasses(paginationUl, `${this.cssClasses.pagination} pagination-flat`);
        paginationUl.style.marginTop = '10px';
        paginationUl.style.listStyle = 'none';
        paginationMain.appendChild(paginationUl);

        const totalPages = Math.ceil(this.totalRecords / this.rowsPerPage);
        const displayPages = 3; // Number of pages to display before and after the current page

        // Previous button
        const prevLi = document.createElement('li');
        if (this.currentPage > 1) {
            prevLi.innerHTML = `<a>&laquo; Previous</a>`;
            prevLi.addEventListener('click', () => {
                this.currentPage--;
                this.fetchData(this.currentPage);
                this.updatePaginationUI();
            });
        } else {
            prevLi.innerHTML = '<span>&laquo; Previous</span>';
            prevLi.classList.add('disabled');
        }
        paginationUl.appendChild(prevLi);

        // First page
        if (this.currentPage > displayPages + 1) {
            const firstLi = document.createElement('li');
            firstLi.innerHTML = `<a>1</a>`;
            firstLi.addEventListener('click', () => {
                this.currentPage = 1;
                this.fetchData(1);
                this.updatePaginationUI();
            });
            paginationUl.appendChild(firstLi);

            // Ellipsis before the current range
            const ellipsisLi = document.createElement('li');
            ellipsisLi.innerHTML = `<span>...</span>`;
            paginationUl.appendChild(ellipsisLi);
        }

        // Pagination numbers around the current page
        for (let i = Math.max(1, this.currentPage - displayPages); i <= Math.min(totalPages, this.currentPage + displayPages); i++) {
            const pageLi = document.createElement('li');
            if (i === this.currentPage) {
                pageLi.classList.add('active');
                pageLi.innerHTML = `<span>${i}</span>`;
            } else {
                pageLi.innerHTML = `<a>${i}</a>`;
                pageLi.addEventListener('click', () => {
                    this.currentPage = i;
                    this.fetchData(i);
                    this.updatePaginationUI();
                });
            }
            paginationUl.appendChild(pageLi);
        }

        // Last page
        if (this.currentPage < totalPages - displayPages) {
            // Ellipsis after the current range
            const ellipsisLi = document.createElement('li');
            ellipsisLi.innerHTML = `<span>...</span>`;
            paginationUl.appendChild(ellipsisLi);

            const lastLi = document.createElement('li');
            lastLi.innerHTML = `<a>${totalPages}</a>`;
            lastLi.addEventListener('click', () => {
                this.currentPage = totalPages;
                this.fetchData(totalPages);
                this.updatePaginationUI();
            });
            paginationUl.appendChild(lastLi);
        }

        // Next button
        const nextLi = document.createElement('li');
        if (this.currentPage < totalPages) {
            nextLi.innerHTML = `<a>Next &raquo;</a>`;
            nextLi.addEventListener('click', () => {
                this.currentPage++;
                this.fetchData(this.currentPage);
                this.updatePaginationUI();
            });
        } else {
            nextLi.innerHTML = '<span>Next &raquo;</span>';
            nextLi.classList.add('disabled');
        }
        paginationUl.appendChild(nextLi);

        // Update pagination info
        paginationInfo.innerHTML = `Showing ${Math.min((this.currentPage - 1) * this.rowsPerPage + 1, this.totalRecords)} to ${Math.min(this.currentPage * this.rowsPerPage, this.totalRecords)} of ${this.totalRecords} entries and Page ${this.currentPage} of ${totalPages}`;
    }

    updatePaginationUI() {
        this.renderPagination();
    }

    updateSortField(field) {
        if (this.sortField === field) {
            this.sortOrder = this.sortOrder === 'asc' ? 'desc' : 'asc';
        } else {
            this.sortField = field;
            this.sortOrder = 'asc';
        }
        this.fetchData(this.currentPage);
        this.renderTable();
    }

    updateSortIcons(targetTh, field) {
        const sortableThs = document.querySelectorAll('.sortable');
        sortableThs.forEach(th => {
            if (th !== targetTh) {
                th.classList.remove('asc', 'desc');
            }
        });

        if (this.sortField === field) {
            targetTh.classList.add(this.sortOrder);
        }
    }

    addCssClasses(element, classes) {
        if (classes) {
            classes.split(' ').forEach(cls => {
                if (cls.trim()) {
                    element.classList.add(cls);
                }
            });
        }
    }
}
//Usage Example
/*
document.addEventListener('DOMContentLoaded', () => {
    new WsTables('table-container', {
        dataUrl: '/acp/client_emails',
        columns: [
            { label: 'Subject', field: 'subject', sortable: true },
            { label: 'Dated', field: 'datetime', sortable: true },
            {
                label: 'Action',
                field: 'id',
                action: (id) => `<a onclick="ws.mail_preview(${id});"><i class="icon-envelop2"></i></a>`,
                sortable: false,
                width:'20px',
                classes:''
            }
        ],
        enableCheckbox: true, // Enable or disable the checkbox feature
        checkboxOptions: {
            bodyCheckboxName: 'clients[]',
            bodyCheckboxClass: 'ws-cbox-select',
            bodyCheckboxWidth: '15px'
        },
        cssClasses: {
            table: 'table table-striped table-bordered',
            thead: '',
            tbody: '',
            pagination: 'pagination'
        },
        defaultSortField: 'datetime', // Specify the default sort field here
        emptyMsg: 'No System Email Message Log.',
        customParams: {
        user_id: '<?=$user_id;?>'
        }
    });
});*/