<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\Google;
use GuzzleHttp\Client as GuzzleClient;
use Carbon\Carbon;
use PHPMailer\PHPMailer\PHPMailer;

function gmail_config()
{
    $SITE_URL = GetConfigs('SITE_URL');
    $SITE_URL = rtrim($SITE_URL,"/");
    $token = GetConfigs("GMAIL-MAILER-TOKEN");
    $configs = [ // Configuration fields 
            'email' => [
                        "Name" => "Google Email/Username",
                        "Type" => "email",
                        "Description" => "Enter Email/Username here.",
                        "DivColumns" => 6,
                        "Tooltip" => "Enter Google Email or Username below in Email field",
                       ],
            "client_id"=>[
                    "Name" => "Client ID",
                    "Type" => "text",
                    "Description" => "Enter Client ID here.",
                    "DivColumns" => 6,
                    "Tooltip" => "Enter Google Auth Client ID",
                   ],
            'secret_key'=>[
                    "Name" => "Secret Key",
                    "Type" => "text",
                    "Description" => "Enter Secret Key here.",
                    "DivColumns" => 6,
                    "Tooltip" => "Enter Google Auth Secret Key",
                   ],
            'token'=>[
                    "Name" => "Token",
                    "Type" => "textarea",
                    "Description" => "Generate Token here",
                    "Rows" => 5,
                    "Default" => $token,
                    "Tooltip" => "Generated Authorization token display in textarea using Google",
                    "ExtraProperties"=>"disabled",
                   ],
            'callback'=>[
                    "Name" => "CallbackURL",
                    "Type" => "text",
                    "Description" => "",
                    "DivColumns" => 12,
                    "Default" => $SITE_URL."/admin/mail-auth/gmail",
                    "ExtraProperties"=>" readonly",
                    "Tooltip" => "Callback url for auth provider Google",
                   ],
            ];

    $configarray = [
    "name" => "Gmail",
    "description" => "Gmail API",
    "version" => "1",//Your Module Version Number
    "author" => "WHASOLS",
    "author_url" => "https://whasols.com/",
    "language" => "english",
    "generate_token" => true,
    "configs" => $configs,
        ];
    return $configarray;
}
function gmail_callback($params=[])
{
    $code = $_REQUEST["code"];
    $state = $_REQUEST["state"];
    $oldState = GetConfigs("GMAIL-MAILER-AUTH-STATE");
    if(empty($oldState)) return ["status"=>"error","message"=>"Auth Requested State empty, Please generate again Token"];
    if(empty($state)) return ["status"=>"error","message"=>"Response Auth State empty, Please generate again Token"];
    if(empty($code)) return ["status"=>"error","message"=>"Auth Code empty, Please generate again Token"];
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    $callback = $params["callback"];
    if(empty($secret_key)) return ["status"=>"error","message"=>"Google Secret Key empty"];
    if(empty($client_id)) return ["status"=>"error","message"=>"Google Client ID empty"];
    $options = [];
    $params = [
        'clientId' => $client_id,
        'clientSecret' => $secret_key,
        'redirectUri' => $callback,
        'accessType' => 'offline'
    ];
    
    try
    {
        $provider = new Google($params);
        $token = $provider->getAccessToken(
            'authorization_code',
            [
                'code' => $code
            ]
        );
//        DoUpdateConfigs("GMAIL-MAILER-CODE",$code);
        DoUpdateConfigs("GMAIL-MAILER-TOKEN",$token);
        // Assuming $accessTokenObject is your AccessToken object
        $accessToken = $token->getToken();  // Access Token
        $expires = $token->getExpires();    // Expiration Timestamp
        $refreshToken = $token->getRefreshToken();  // Refresh Token
        $data = ["expires"=>$expires,"accessToken"=>$accessToken,"refreshToken"=>$refreshToken];
        DoUpdateConfigs("GMAIL-MAILER-TOKEN-DATA",_je($data));
        return ["status"=>"success","message"=>"Google Auth Token Validated Successfully"];
    }
    catch (Exception $e)
    {
        return ["status"=>"error","message"=>"Google Auth Token Validation Error: "._je($e)];
    }
}
function gmail_generate_access($params=[])
{
    //Instantiation and passing `true` enables exceptions
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    $callback = $params["callback"];
    if(empty($secret_key)) return ["status"=>"error","message"=>"Google Secret Key empty"];
    if(empty($client_id)) return ["status"=>"error","message"=>"Google Client ID empty"];
    $options = [];
    $params = [
        'clientId' => $client_id,
        'clientSecret' => $secret_key,
        'redirectUri' => $callback,
        'scopes'       => ['https://www.googleapis.com/auth/gmail.send'],
        'accessType' => 'offline',
        'prompt'       => 'consent'   // Force the consent screen to be shown again
    ];
    $provider = new Google($params); 
    $authUrl = $provider->getAuthorizationUrl();
    if(empty($authUrl)) return ["status"=>"error","message"=>"Google Authorization URL generation failed"];
    DoUpdateConfigs("GMAIL-MAILER-AUTH-STATE",$provider->getState());
    return ["status"=>"success","message"=>"Google Authorization URL generated successfully","url"=>$authUrl];
}
function gmail_isExpireToken($provider)
{
    $TokenData = GetConfigs("GMAIL-MAILER-TOKEN-DATA");
    $TokenData = _jd($TokenData,true);
    $expires = $TokenData["expires"];
    $accessToken = $TokenData["accessToken"];
    $refreshToken = $TokenData["refreshToken"];
    if(empty($accessToken)) return ["status"=>"error","message"=>"Please generate Google Access Token"];
    if(empty($refreshToken)) return ["status"=>"error","message"=>"Google Refresh Token not found, so please generate or revoke Google Tokens"];

    $currentTimestamp = time();
    $tokenData = ["accessToken"=>$accessToken,"expires"=>$expires];
    if ($expires < $currentTimestamp)
    {
        // Access token has expired
        // Perform token refresh logic here
        try {
            $newAccessToken = $provider->getAccessToken('refresh_token', [
                'refresh_token' => $refreshToken,
            ]);
            // Update your stored token data with the new access token and expiration timestamp
            $tokenData = ['refreshToken'=>$refreshToken,'accessToken'=>$newAccessToken->getToken(),'expires'=>$newAccessToken->getExpires()];

        } catch (IdentityProviderException $e)
        {
            return ["status"=>"error","message"=>$e->getMessage(),"token"=>""];
        }
    }
    if($accessToken != $tokenData["accessToken"])
    {
        DoUpdateConfigs("GMAIL-MAILER-TOKEN",$tokenData["accessToken"]);
        DoUpdateConfigs("GMAIL-MAILER-TOKEN-DATA",_je($tokenData));
    }
    return ["status"=>"success","token"=>$tokenData];
}
function gmail_post($params=[],$mailData=[])
{
    $mail = new PHPMailer();
    $smtp_user = $params["email"];
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    $provider = new Google(
        [
            'clientId' => $client_id,
            'clientSecret' => $secret_key,
        ]);
    $isExpiredToken = gmail_isExpireToken($provider); 
    if($isExpiredToken["status"] == "error") return $isExpiredToken;
    $refresh_token = $token = GetConfigs("GMAIL-MAILER-TOKEN"); 
    $From = $mailData["From"];
    $AddressTo = $mailData["AddressTo"];
    $Subject = $mailData["Subject"];
    $Body = $mailData["Body"];
    $toEmail = (is_array($AddressTo)) ? $AddressTo[0]["Email"]:$AddressTo;
    $fromEmail = (is_array($From)) ? $From["Email"]:$From;
    //$fromEmail = "hostarina@gmail.com";
    $ReplyTo= _s("ReplyTo",$mailData,[]);
    $Cc= _s("Cc",$mailData,[]); //Cc Array = Emails of CC.
    $Bcc= _s("Bcc",$mailData,[]); //BccArray = Emails of Bcc.

    $boundary = uniqid(rand(), true);
    
    $rawMessageString = "From: $fromEmail\r\n";
    $rawMessageString .= "To: $toEmail\r\n";
    if(is_array($Cc) && !empty($Cc))
    {
        $Cc = implode(",",$Cc);
        $rawMessageString .= "Cc: $Cc\r\n";
    }
    if(is_array($Bcc) && !empty($Bcc))
    {
        $Bcc = implode(",",$Bcc);
        $rawMessageString .= "Bcc: $Bcc\r\n";
    }
    if(is_array($ReplyTo) && !empty($ReplyTo))
    {
        $ReplyToEmail= $ReplyTo["Email"];
        $ReplyToName= $ReplyTo["Name"];
        $rawMessageString .= "Reply-To: $ReplyToEmail\r\n";
    }
    $BodyText = $mail->html2text($Body);
    $rawMessageString .= "Subject: $Subject\r\n\r\n";
    $rawMessageString .= "MIME-Version: 1.0\r\n";
    $rawMessageString .= "Content-Type: multipart/alternative; boundary=\"{$boundary}\"\r\n\r\n";
    $rawMessageString .= "--{$boundary}\r\n";
    $rawMessageString .= "Content-Type: text/plain; charset=UTF-8\r\n\r\n";
    $rawMessageString .= "{$BodyText}\r\n";
    $rawMessageString .= "--{$boundary}\r\n";
    $rawMessageString .= "Content-Type: text/html; charset=UTF-8\r\n\r\n";
    $rawMessageString .= "{$Body}\r\n";
    $rawMessageString .= "--{$boundary}--";     

/*    
    $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
    if(!empty($Attachments))
    {
        foreach ($Attachments as $file)
        {
            $file = $add["File"];
            $name = $add["Name"];
            if(!empty($file))
            {
                $mail->addAttachment($file,$name);//Name is optional
            }
        }
    }
*/
    $rawMessage = base64_encode($rawMessageString);
    //$rawMessage = str_replace(['+', '/', '='], ['-', '_', ''], $rawMessage);
    try{
        $guzzleClient = new GuzzleClient([
            'headers' => [
                'Authorization' => 'Bearer ' .$token
            ]
        ]);
        // Send the email using the Gmail API
        $response = $guzzleClient->post('https://www.googleapis.com/gmail/v1/users/me/messages/send', [
            'json' => [
                'raw' =>$rawMessage
            ]
        ]);    
        // Check if the email was sent successfully
        if ($response->getStatusCode() === 200) return ["status"=>"success","message"=>"Email sent successfully"];
          return ["status"=>"error","message"=>"Email Sending Failed"];
    } catch (\GuzzleHttp\Exception\ClientException $e)
    {
            $response = $e->getResponse();
            $response = $response->getBody()->getContents();
            $response = json_decode($response,true);
          return ["status"=>"error","message"=>"Email Sending Failed: ".$response["error"]["message"]];
    }
}
function gmail_test($params=[])
{
    $FromName = GetConfigs("MAIL_FROM_NAME");
    if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
    if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];    
    $email = GetConfigs("SITE_EMAIL");
    if(empty($email)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];
    //Recipients
    $Subject = "Test Gmail API";
    $Body = "Testing Gmail API Connection email";
    $mailData = [];
    $mailData["From"] = ["Email"=>$FromEmail,"Name"=>$FromName];
    $mailData["Subject"] = $Subject;
    $mailData["Body"] = $Body;
    $mailData["ClientId"] = 0;
    $mailData["AddressTo"][] = ["Email"=>$email,"Name"=>$FromName];
    return gmail_post($params,$mailData);
} 
function gmail_send($params=[], $mailData=[])
{
    return gmail_post($params,$mailData);
}