<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
function Mailgun_config()
{
    $configs = [ // Configuration fields 
                'api_key' => [
                        "Name" => "Private API Key",
                        "Type" => "password",
                        "Description" => "Enter Private API Key here.",
                        "DivColumns" => 12,
                        "Default" => "", 
                        "Tooltip" => 'Follow this link to get a Private API Key from Mailgun: <a href="https://app.mailgun.com/app/account/security/api_keys" target="_blank">Get a Private API Key.</a>',
                         ],
                'domain_name' => [
                        "Name" => "Domain Name",
                        "Type" => "text",
                        "Description" => "Enter Domain Name here.",
                        "DivColumns" => 12,
                        "Tooltip" => 'Follow this link to get a Domain Name from Mailgun: <a href="https://app.mailgun.com/app/domains" target="_blank">Get a Domain Name.</a>',
                       ],
            "region" => [ //Dropdown Select box
                    "Name" => "Region", 
                    "Type" =>  "dropdown",
                    "Options" => ['us'=>'US','eu'=>'EU'], 
                    "Description" => "Select Region", 
                    "Default" => "us",
                    "DivColumns" => 12,
                    "Tooltip"=>'Define which endpoint you want to use for sending messages.<br>If you are operating under EU laws, you may be required to use EU region. <a href="https://www.mailgun.com/about/regions/" target="_blank">More information</a> on Mailgun.com.'
                ],
            ];
        $configs['debug'] = [
                "Name" => "Debugging",
                "Type" => "radio",
                "Options" => ["Yes"=>"Enable debugging output","No"=>"Disable"],
                "Description" => "Select email debugging option",
                "DivColumns" => 12,
                "Default"=>"Yes",
                "Tooltip" => "If Debugging option selected as Enable then Debugging logs will be stored in database as activity logs and you can check at logs page",
               ];
    $configarray = [
    "name" => "Mailgun",
    "description" => "Mailgun",
    "version" => "1",//Your Module Version Number
    "author" => "WHASOLS",
    "author_url" => "https://whasols.com/",
    "language" => "english",
    "generate_token" => false,
    "configs" => $configs,
        ];
    return $configarray;
}
function Mailgun_post($params=[],$postFields=[])
{
    $api_key = _s("api_key",$params,"");
    $domain_name = _s("domain_name",$params,"");
    $region = _s("region",$params,"us");
    $url = ($region == "us") ? "https://api.mailgun.net/v3/":"https://api.eu.mailgun.net/v3/";
    $url .=$domain_name."/messages";

    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => $url,
      CURLOPT_USERPWD=>'api:' . $api_key,
      CURLOPT_POSTFIELDS => $postFields,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => 'POST',
    ));
    $response = curl_exec($curl);
    $error = curl_error($curl);
    curl_close($curl);
    if(!empty($error)) return ["status"=>"error","message"=>$error];
    if(!is_object($response)) return ["status"=>"error","message"=>"Mailgun API Error: ".$response];
    $response = json_decode($response,true);
    if(is_array($response))
    {
        $id = _s("id",$response,"");
        if(!empty($id)) return ["status"=>"success","message"=>"Email sent successfully, ".$response["message"]];
        return ["status"=>"error","message"=>"Email sending failed, ".$response["message"]];
    }
    return ["status"=>"error","message"=>"Email sending failed, ".$response];
}
function Mailgun_test($params=[],$mailData=[])
{
    $Subject= "Testing Email";
    $date = date('Y-m-d H:i:s');
    $Body = "Email testing using php mailer, dated:".$date;
    $ClientId = 0;
    $from_name = GetConfigs("MAIL_FROM_NAME");
    if(empty($from_name)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
    if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];
    $EmailsTos = GetConfigs("SITE_EMAIL");
    if(empty($EmailsTos)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];
    $postFields = [];
    $postFields["from"] = $FromEmail;
    $postFields["to"] = $EmailsTos;
    $postFields["subject"] = $Subject;
    $postFields["html"] = $Body;
    $response = Mailgun_post($params,$postFields);
    return $response;
}
function Mailgun_send($params=[], $mailData=[])
{
    $Subject= $mailData["Subject"];
    $Body= $mailData["Body"];

    $ClientId = $mailData["ClientId"];
    $From= $mailData["From"];
    $FromEmail= $From["Email"];
    $FromName= $From["Name"];
 
    $ReplyTo= $mailData["ReplyTo"];
    $ReplyToEmail= $ReplyTo["Email"];
    $ReplyToName= $ReplyTo["Name"];

    $AddressTo= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
    $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
    $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
    //$Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
      
    $postFields = [];
    $postFields["from"] = $FromEmail;
    $postFields["to"] = $AddressTo;
    $postFields["subject"] = $Subject;
    //$postFields["cc"] = $Cc;
    //$postFields["bcc"] = $Bcc;
    $postFields["html"] = $Body;
    $response = Mailgun_post($params,$postFields);
    return $response;
}