<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use Carbon\Carbon;
use SendGrid\Mail\Mail;
function SendGrid_config()
{
	$configarray = [
	"name" => "SendGrid",
	"description" => "Send Grid Emailer",
	"version" => "1",//Your Module Version Number
	"author" => "WHASOLS",
	"author_url" => "https://whasols.com/",
	"language" => "english",
	"configs" => [ // Configuration fields
	       "key" => ["Name" =>"Api Key", "Type" => "password", "Size" => "50"]
	        ],
	    ];
	return $configarray;
}
function SendGrid_test($params=[])
{
    // Define path/existence of Composer autoloader
    $composerAutoloadFile = __DIR__ . '/vendor/autoload.php';
    $composerAutoloadFileExists = (is_file($composerAutoloadFile));
    // Can't locate SendGrid\Mail\Mail class?
    if (!class_exists(Mail::class))
    {
        // Suggest to load Composer autoloader of project
        if (!$composerAutoloadFileExists) return ["status"=>"error","message"=>"Composer autoloader not found. Execute 'composer install' in the project folder first."];
        // Load Composer autoloader
        require_once $composerAutoloadFile;
        // If desired class still not existing
        // Suggest to review the Composer autoloader settings
        if (!class_exists(Mail::class)) return ["status"=>"error","message"=>"Error finding SendGrid classes. Please review your autoloading configuration."];
        $apiKey = $params["key"];
        if(empty($apiKey)) return ["status"=>"error","message"=>"Please Enter Send Grid API Key"];
        
        $Subject= "Testing Email";
        $date = date('Y-m-d H:i:s');
        $Body = "Email testing using php mailer, dated:".$date;
        $ClientId = 0;
        $FromName = GetConfigs("MAIL_FROM_NAME");
        if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
        $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
        if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];
        $tos = GetConfigs("SITE_EMAIL");
        if(empty($tos)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];
        $tos= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
        $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
        $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
        $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
        
        
        $email = new Mail();
        $email->setFrom($FromEmail,$FromName);
        $email->addTos($tos);
        $email->setSubject($Subject);
        $email->addContent("text/plain", $Body);
        $email->addContent("text/html", $Body);
        
        
        $sendgrid = new \SendGrid($apiKey);
        try {
            $response = $sendgrid->send($email);
            $statusCode = $response->statusCode();
            if (in_array($statusCode, [200, 202])) return ["status"=>"success","message"=>"Email sent successfully"];
            if(is_array($response) || is_object($response)) $response = json_decode($response, true);
            if(is_array($params) || is_object($params)) $params = json_decode($params, true);
            logModuleCall("SendGrid", "Send Email", $params, $response); 
            return ["status"=>"error","message"=>"Email Sending Failed, Please Check Module Logs"];
        } catch (Exception $e)
        {
            return ["status"=>"error","message"=>'Caught exception: '. $e->getMessage()];
        }
    }
    return ["status"=>"error","message"=>"Send Grid Mail Class not found"];
}
function SendGrid_send($params=[], $mailData=[])
{
    // Define path/existence of Composer autoloader
    $composerAutoloadFile = __DIR__ . '/vendor/autoload.php';
    $composerAutoloadFileExists = (is_file($composerAutoloadFile));
    // Can't locate SendGrid\Mail\Mail class?
    if (!class_exists(Mail::class))
    {
        // Suggest to load Composer autoloader of project
        if (!$composerAutoloadFileExists) return ["status"=>"error","message"=>"Composer autoloader not found. Execute 'composer install' in the project folder first."];
        // Load Composer autoloader
        require_once $composerAutoloadFile;
        // If desired class still not existing
        // Suggest to review the Composer autoloader settings
        if (!class_exists(Mail::class)) return ["status"=>"error","message"=>"Error finding SendGrid classes. Please review your autoloading configuration."];
        
        $Subject= $mailData["Subject"];
        $Body= $mailData["Body"];
    
        $ClientId = $mailData["ClientId"];
        $From= $mailData["From"];
        $FromEmail= $From["Email"];
        $FromName= $From["Name"];
    
        $ReplyTo= $mailData["ReplyTo"];
        $ReplyToEmail= $ReplyTo["Email"];
        $ReplyToName= $ReplyTo["Name"];
    
        $tos= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
        $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
        $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
        $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
        
        
        $email = new Mail();
        $email->setFrom($FromEmail,$FromName);
        if(count($tos) == 1)
        {
            $email->addTo($tos[0]["Email"],$tos[0]["Name"]);
        }else{
            $email->addTos($tos);
        }
        if(!empty($Cc))
        {
            $ccEmails=[];
            $CcList = explode(",",$Cc);
            foreach($CcList as $ce)
            {
                $cePart = explode("@",$ce);
                $ccEmails[] = [$ce,$cePart[0]];
            }
        $email->addCcs($ccEmails);
        }
        if(!empty($Bcc))
        {
            $bccEmails=[];
            $BccList = explode(",",$Bcc);
            foreach($BccList as $be)
            {
                $bePart = explode("@",$be);
                $bccEmails[] = [$be,$bePart[0]];
            }
        $email->addBccs($ccEmails);
        }
        $email->setSubject($Subject);
        $email->addContent("text/plain", $Body);
        $email->addContent(
            "text/html", $Body
        );
        
        $attachmentsList = [];
        if(!empty($Attachments))
        {
           foreach($Attachments as $attach)
           {
               $fileName = $attach["Name"];
               $file = $attach["File"];
               $ext = array_pop(explode('.', $file));
                $file_encoded = base64_encode(file_get_contents($file));
                $attachType = "attachment";//attachment,inline
                $appType = "";
                if(in_array($ext,["png,jpg,jpeg,gif"])) $appType="image/".$ext;
                if(in_array($ext,["pdf,doc,txt,docx","xls","xlsx"])) $appType="application/".$ext;
                $attachmentsList[] = [$file_encoded,$appType,$fileName,$attachType];
           }
           $email->addAttachments($attachmentsList);
        }
        $apiKey = $params["key"];
        $sendgrid = new \SendGrid($apiKey);
        try {
            $response = $sendgrid->send($email);
            $statusCode = $response->statusCode();
            if (in_array($statusCode, [200, 202])) return ["status"=>"success","message"=>"Email sent successfully"];
            if(is_array($response) || is_object($response)) $response = json_decode($response, true);
            if(is_array($params) || is_object($params)) $params = json_decode($params, true);
            logModuleCall("SendGrid", "Send Email", $params, $response); 
            return ["status"=>"error","message"=>"Email Sending Failed, Please Check Module Logs"];
        } catch (Exception $e)
        {
            return ["status"=>"error","message"=>'Caught exception: '. $e->getMessage()];
        }
    }
}