<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use Carbon\Carbon;
function ZeptoMail_config()
{
	$configarray = [
	"name" => "ZeptoMail",
	"description" => "ZeptoMail Emailer",
	"version" => "1",//Your Module Version Number
	"author" => "WHASOLS",
	"author_url" => "https://whasols.com/",
	"language" => "english",
	"configs" => [
            "domain_name" => [
                "Name" => "Hosted region",
                "Type" => "radio",
                "Options" => [
                    "com" => "zeptomail.zoho.com",
                    "eu" => "zeptomail.zoho.eu",
                    "in" => "zeptomail.zoho.in",
                    "com.cn" => "zeptomail.zoho.com.cn",
                    "com.au" => "zeptomail.zoho.com.au",
                ],
                "Description" => "The region where your Zoho account data reside.",	
                "DivColumns" => 12,
            ],
            "sendmailtoken" => [
                "Name" => "Send Mail Token",
                "Type" => "password",
                "Description" =>"Send Mail token generated in the relevant ZeptoMail Mail Agent.",
                "DivColumns" => 12,
            ],
            "mail_format" => [
                "Name" => "Mail Format",
                "Type" => "radio",
                "Options" => [
                    "option1" => "Plaintext",
                    "option2" => "HTML",
                ],
                "Description" =>"The default format of emails sent from the plugin.",
                "Default" => "option2",
                "DivColumns" => 12,
            ],
	        ],
	    ];
	return $configarray;
}

function ZeptoMail_test($params=[])
{
    $Subject= "Testing Email";
    $date = date('Y-m-d H:i:s');
    $Body = "Email testing using php mailer, dated:".$date;
    $ClientId = 0;
    $from_name = GetConfigs("MAIL_FROM_NAME");
    if(empty($from_name)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    $from_email_address = GetConfigs("MAIL_FROM_EMAIL");
    if(empty($from_email_address)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];
    $EmailsTos = GetConfigs("SITE_EMAIL");
    if(empty($EmailsTos)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];

    $mail_format = $params["mail_format"];

    $postFields = [
        "from" => [
            "address" =>$from_email_address,
            "name" =>$from_name,
        ],
        "subject" =>$Subject,
    ];
    $postFields["to"][] = [
                "email_address" => [
                    "address" => $EmailsTos,
                    "name" => $from_name,
                ],
            ];
    if ($mail_format === "option1")
    {
        $postFields["textbody"] = $Body;
    } elseif ($mail_format === "option2")
    {
        $postFields["htmlbody"] =$Body;
    }

    $postFields["cc"] = [];
    $postFields["bcc"] = [];

    return ZeptoMail_call($params,$postFields);
}
function ZeptoMail_send($params=[], $mailData=[])
{
    $Subject= $mailData["Subject"];
    $Body= $mailData["Body"];

    $ClientId = $mailData["ClientId"];
    $From= $mailData["From"];
    $from_email_address= $From["Email"];
    $from_name= $From["Name"];

    $ReplyTo= $mailData["ReplyTo"];
    $ReplyToEmail= $ReplyTo["Email"];
    $ReplyToName= $ReplyTo["Name"];

    $tos= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
    $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
    $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
    $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
    
    $sendmailtoken = $params["sendmailtoken"];
    $mail_format = $params["mail_format"];

    $postFields = [
        "from" => [
            "address" =>$from_email_address,
            "name" =>$from_name,
        ],
        "subject" =>$Subject,
    ];

    $inlineImages = [];
    if ($mail_format === "option1")
    {
        $postFields["textbody"] = $Body;
    } elseif ($mail_format === "option2")
    {
        $postFields["htmlbody"] =$Body;
        preg_match_all(
            "/<body\b[^>]*>(.*?)<\/body>/is",
            $postFields["htmlbody"],
            $bodyMatches
        );
        if (!empty($bodyMatches[1]))
        {
            $bodyContent = $bodyMatches[1][0];
            preg_match_all(
                '/<img[^>]+src="([^">]+)"/',
                $bodyContent,
                $matches
            );
            $inlineImages = $matches[1];
        }
        else
        {
            $inlineImages = [];
        }

        $inlineAttachments = [];
        $cids = [];
        foreach ($inlineImages as $imageUrl) {
            $cid = uniqid("image");
            $cids[$imageUrl] = $cid;
            // Replace the image source URL with CID in the message body
            $postFields["htmlbody"] = preg_replace(
                '/(<img[^>]+src=)("' .
                    preg_quote($imageUrl, "/") .
                    '")/',
                '$1"cid:' . $cid . '"',
                $postFields["htmlbody"],
                1
            );
            $imageData = file_get_contents($imageUrl);
            $extension = pathinfo($imageUrl, PATHINFO_EXTENSION);

            $inlineAttachments[] = [
                "name" => basename($imageUrl),
                "cid" => $cids[$imageUrl],
                "mime_type" => "image/jpeg",
                "content" => base64_encode($imageData),
            ];
        }
        $postFields["inline_images"] = $inlineAttachments;
    }
    
    foreach ($tos as $to) {
        $postFields["to"][] = [
            "email_address" => [
                "address" => $to["Email"],
                "name" => $to["Name"],
            ],
        ];
    }
    $ccEmails=[];
    if(!empty($Cc))
    {
        $CcList = explode(",",$Cc);
        foreach($CcList as $ce)
        {
            $cePart = explode("@",$ce);
            $postFields["cc"][] = [
                "email_address" => [
                    "address" =>$ce,
                    "name" =>$cePart[0],
                ],
            ];
        }
    }
    $bccEmails=[];
    if(!empty($Bcc))
    {
        $BccList = explode(",",$Bcc);
        foreach($BccList as $be)
        {
            $bePart = explode("@",$be);
            $postFields["bcc"][] = [
                "email_address" => [
                    "address" =>$be,
                    "name" =>$bePart[0],
                ],
            ];
        }
    }

    $attachmentsList = [];
    if(!empty($Attachments))
    {    
        foreach ($Attachments as $attachment) {
            $content = file_get_contents($attachment["File"]);
            $filename = $attachment["Name"];
            $attachmentsList[] = [
                "name" => $filename,
                "mime_type" => "application/octet-stream",
                "content" => base64_encode($content),
            ];
        }
    }
    $postFields["attachments"] = json_encode($attachmentsList);
    return ZeptoMail_call($params,$postFields);
}
function ZeptoMail_call($params,$postFields)
{
    $domain_name = _s('domain_name',$params,"");
    $sendmailtoken = _s('sendmailtoken',$params,"");
    $mail_format = _s('mail_format',$params,"");
    if(empty($domain_name)) return ["status"=>"error","message"=>"Please Select Hosted region"];
    if(empty($sendmailtoken)) return ["status"=>"error","message"=>"Please Enter Send Mail Token"];
    if(empty($mail_format)) return ["status"=>"error","message"=>"Please Select Mail Format"];
    try {
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL =>"https://api.zeptomail.".$domain_name."/v1.1/email",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode($postFields),
            CURLOPT_HTTPHEADER => [
                "accept: application/json",
                "authorization: " . $sendmailtoken,
                "cache-control: no-cache",
                "content-type: application/json",
                "origin: WHASOLS"
            ],
        ]);
        $response = curl_exec($curl);
        curl_close($curl);
        $decodedData = json_decode($response, true);
        $message = "";
        if (isset($decodedData["data"]) && !empty($decodedData["data"]))
        {
            return ["status"=>"success","message"=>"Email sent successfully"];
        }
        else
        {
            if (isset($decodedData["error"]) && !empty($decodedData["error"]))
            {
                if (isset($decodedData["error"]["details"][0]["message"]) && $decodedData["error"]["details"][0]["message"] === "Invalid API Token found") {
                    $message = "Configuration failed. Enter a valid Send Mail token and try again.";
                }
                if (!empty($decodedData["error"]["details"][0]["target"]) && $decodedData["error"]["details"][0]["target"] === "from"){
                    $message = "Configuration failed. Enter a valid From address and try again.";
                }
                if ($message === ""){
                    $message = "Mail sending failed. Please conatct us through presales@zeptomail.com";
                }
                logModuleCall("ZeptoMail", "Send Email", $params, $response); 
                return ["status"=>"error","message"=>"Email Sending Failed, due to ".$message];
            }
        }
        return ["status"=>"error","message"=>"Credentials validation failed, due to ".$message];
    } catch (Exception $e)
    {
        return ["status"=>"error","message"=>'Caught exception: '. $e->getMessage()];
    }
}