<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\OAuth;
use PHPMailer\PHPMailer\Exception;
//Alias the League Google OAuth2 provider class
use League\OAuth2\Client\Provider\Google;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use Carbon\Carbon;

function gmail_config()
{
    $SITE_URL = GetConfigs('SITE_URL');
    $SITE_URL = rtrim($SITE_URL,"/");
    $token = GetConfigs("GMAIL-MAILER-TOKEN");
    $configs = [ // Configuration fields 
            "type" => [ //Dropdown Select box
                    "Name" => "Mail Type", 
                    "Type" =>  "dropdown",
                    "Options" => ['TEXT'=>'Text','HTML'=>'HTML'], 
                    "Description" => "Select Mail Type", 
                    "Default" => "HTML",
                    "DivColumns" => 6,
                ],
            "encryption" => [ //Dropdown Select box
                    "Name" => "Mail Encoding", 
                    "Type" =>  "dropdown", 
                    "Options" => ['UTF-8'=>'UTF-8','8bit'=>'8bit','7bit'=>'7bit','Binary'=>'Binary','Base64'=>'Base64'], 
                    "Description" => "Select Mail Encoding Type", 
                    "Default" => "UTF-8",
                    "DivColumns" => 4,
                ],
            "ssl" => [ //Dropdown Select box
                    "Name" => "SMTP SSL Type", 
                    "Type" =>  "dropdown", 
                    "Options" => ['SSL'=>'SSL','TLS'=>'TLS'], 
                    "Description" => "Dropdown Description here", 
                    "Default" => "SSL", 
                    "DivColumns" => 4,
                ],
            'port' => [
                        "Name" => "SMTP Port #",
                        "Type" => "number",
                        "Min" => 0,
                        "Max" => 99999,
                        "Description" => "SMTP Port #",
                        "DivColumns" => 4,
                        "Default" => 465,
                        "Tooltip" => "Enter SMTP Port as 465 or 587 or any valid port number, Default Port for SSL is 465 and for TLS is 587",
                       ],
            'hostname' => [
                        "Name" => "Google Hostname",
                        "Type" => "text",
                        "Description" => "Enter Hostname here.",
                        "DivColumns" => 6,
                        "Default" =>"smtp.gmail.com", 
                        "Tooltip" => "Enter Google hostname below as smtp.gmail.com or any valid hostname",
                         ],
            'email' => [
                        "Name" => "Google Email/Username",
                        "Type" => "email",
                        "Description" => "Enter Email/Username here.",
                        "DivColumns" => 6,
                        "Tooltip" => "Enter Google Email or Username below in Email field",
                       ],
            "client_id"=>[
                    "Name" => "Client ID",
                    "Type" => "text",
                    "Description" => "Enter Client ID here.",
                    "DivColumns" => 6,
                    "Tooltip" => "Enter Google Auth Client ID",
                   ],
            'secret_key'=>[
                    "Name" => "Secret Key",
                    "Type" => "text",
                    "Description" => "Enter Secret Key here.",
                    "DivColumns" => 6,
                    "Tooltip" => "Enter Google Auth Secret Key",
                   ],
            'token'=>[
                    "Name" => "Token",
                    "Type" => "textarea",
                    "Description" => "Generate Token here",
                    "Rows" => 5,
                    "Default" => $token,
                    "Tooltip" => "Generated Authorization token display in textarea using Google",
                    "ExtraProperties"=>"disabled",
                   ],
            'callback'=>[
                    "Name" => "CallbackURL",
                    "Type" => "text",
                    "Description" => "",
                    "DivColumns" => 12,
                    "Default" => $SITE_URL."/admin/mail-auth/gmail",
                    "ExtraProperties"=>" readonly",
                    "Tooltip" => "Callback url for auth provider Google",
                   ],
            'debug'=>[
                "Name" => "Debugging",
                "Type" => "radio",
                "Options" => ["Yes"=>"Enable debugging output","No"=>"Disable"],
                "Description" => "Select email debugging option",
                "DivColumns" => 12,
                "Default"=>"Yes",
                "Tooltip" => "If Debugging option selected as Enable then Debugging logs will be stored in database as activity logs and you can check at logs page",
               ],
            'content'=>[
                "Name" => "Extra Content",
                "Type" => "system",
                "Description" => "",
                "DivColumns" => 12,
                "Content" => '<div class="alert alert-info ws-smtp-info">Some Time during SMTP Connection 465 Port not working using third-party email providers as (cpanel,cwp,cyberpanel and others), so try 587 port.<p>Before Save Changes, please test the credentials using <b>Test Configuration</b> button</p></div>',
               ]
            ];


    $configarray = [
    "name" => "Gmail",
    "description" => "Gmail XOAUTH2",
    "version" => "1",//Your Module Version Number
    "author" => "WHASOLS",
    "author_url" => "https://whasols.com/",
    "language" => "english",
    "generate_token" => true,
    "configs" => $configs,
        ];
    return $configarray;
}
function gmail_callback($params=[])
{
    $code = $_REQUEST["code"];
    $state = $_REQUEST["state"];
    $oldState = GetConfigs("GMAIL-MAILER-AUTH-STATE");
    if(empty($oldState)) return ["status"=>"error","message"=>"Auth Requested State empty, Please generate again Token"];
    if(empty($state)) return ["status"=>"error","message"=>"Response Auth State empty, Please generate again Token"];
    if(empty($code)) return ["status"=>"error","message"=>"Auth Code empty, Please generate again Token"];
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    $callback = $params["callback"];
    
    if(empty($secret_key)) return ["status"=>"error","message"=>"Google Secret Key empty"];
    if(empty($client_id)) return ["status"=>"error","message"=>"Google Client ID empty"];
    $options = [];
    $params = [
        'clientId' => $client_id,
        'clientSecret' => $secret_key,
        'redirectUri' => $callback,
        'accessType' => 'offline'
    ];
    
    try
    {
        $provider = new Google($params);
        $token = $provider->getAccessToken(
            'authorization_code',
            [
                'code' => $code
            ]
        );
//        DoUpdateConfigs("GMAIL-MAILER-CODE",$code);
        DoUpdateConfigs("GMAIL-MAILER-TOKEN",$token);
        // Assuming $accessTokenObject is your AccessToken object
        $accessToken = $token->getToken();  // Access Token
        $expires = $token->getExpires();    // Expiration Timestamp
        $refreshToken = $token->getRefreshToken();  // Refresh Token
        DoUpdateConfigs("GMAIL-MAILER-TOKEN-DATA",_je(["expires"=>$expires,"accessToken"=>$accessToken,"refreshToken"=>$refreshToken]));
        return ["status"=>"success","message"=>"Google Auth Token Validated Successfully"];
    }
    catch (IdentityProviderException $e)
    {
        return ["status"=>"error","message"=>"Google Auth Token Validation Error: ".$e->getMessage()];
    }
}
function gmail_generate_access($params=[])
{
    //Instantiation and passing `true` enables exceptions
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    $callback = $params["callback"];
    if(empty($secret_key)) return ["status"=>"error","message"=>"Google Secret Key empty"];
    if(empty($client_id)) return ["status"=>"error","message"=>"Google Client ID empty"];
    $options = [];
    $params = [
        'clientId' => $client_id,
        'clientSecret' => $secret_key,
        'redirectUri' => $callback,
        'accessType' => 'offline',
        'prompt'       => 'consent'   // Force the consent screen to be shown again        
    ];
    $provider = new Google($params);
    $options = [
        'scope' => [
            'https://mail.google.com/'
        ]
    ];
    $authUrl = $provider->getAuthorizationUrl($options);
    if(empty($authUrl)) return ["status"=>"error","message"=>"Google Authorization URL generation failed"];
    DoUpdateConfigs("GMAIL-MAILER-AUTH-STATE",$provider->getState());
    return ["status"=>"success","message"=>"Google Authorization URL generated successfully","url"=>$authUrl];
}
function gmail_isExpireToken($provider)
{
    $TokenData = GetConfigs("GMAIL-MAILER-TOKEN-DATA");
    $TokenData = _jd($TokenData,true);
    $expires = $TokenData["expires"];
    $accessToken = $TokenData["accessToken"];
    $refreshToken = $TokenData["refreshToken"];
    if(empty($accessToken)) return ["status"=>"error","message"=>"Please generate Google Access Token"];
    if(empty($refreshToken)) return ["status"=>"error","message"=>"Google Refresh Token not found, so please generate or revoke Google Tokens"];

    $currentTimestamp = time();
    $tokenData = ["accessToken"=>$accessToken,"expires"=>$expires];
    if ($expires < $currentTimestamp)
    {
        // Access token has expired
        // Perform token refresh logic here
        try {
            $newAccessToken = $provider->getAccessToken('refresh_token', [
                'refresh_token' => $refreshToken,
            ]);
            // Update your stored token data with the new access token and expiration timestamp
            $tokenData = ['refreshToken'=>$refreshToken,'accessToken'=>$newAccessToken->getToken(),'expires'=>$newAccessToken->getExpires()];

        } catch (IdentityProviderException $e)
        {
            return ["status"=>"error","message"=>$e->getMessage(),"token"=>""];
        }
    }
    if($accessToken != $tokenData["accessToken"])
    {
        DoUpdateConfigs("GMAIL-MAILER-TOKEN",$tokenData["accessToken"]);
        DoUpdateConfigs("GMAIL-MAILER-TOKEN-DATA",_je($tokenData));
    }
    return ["status"=>"success","token"=>$tokenData];
}
function gmail_instance($params=[])
{
    $smtp_port = $params["port"];
    $smtp_host = $params["hostname"];
    $smtp_user = $params["email"];
    $mailtype = $params["type"];
    $ssl = $params["ssl"];
    $debug= $params["debug"];
    $callback = $params["callback"];
    $mail = new PHPMailer(true);
    // Server settings
    $mail->isSMTP();
    $mail->Timeout = 10; // set the timeout (seconds)
    if(empty($smtp_host)) return ["status"=>"error","message"=>"Google Hostname empty"];
    if(empty($smtp_user)) return ["status"=>"error","message"=>"Google Username/Email empty"];
    if(empty($smtp_port)) return ["status"=>"error","message"=>"SMTP Port empty"];
    $mail->Host       = $smtp_host;
    $mail->SMTPAuth   = true;
    if($ssl == "TLS")
    {
        $mail->SMTPSecure = "ssl";
    }
    elseif($ssl == "SSL")
    {
        $mail->SMTPSecure = "ssl";
    }else{
       $mail->SMTPSecure = "ssl";         //Enable TLS encryption; `PHPMailer::ENCRYPTION_SMTPS` encouraged 
    } 
    $mail->Port       = $smtp_port;
    $client_id = $params["client_id"];
    $secret_key = $params["secret_key"];
    if(empty($secret_key)) return ["status"=>"error","message"=>"Google Secret Key empty"];
    if(empty($client_id)) return ["status"=>"error","message"=>"Google Client ID empty"];
    //Create a new OAuth2 provider instance
    $provider = new Google(
        [
            'clientId' => $client_id,
            'clientSecret' => $secret_key,
        ]);
    $isExpiredToken = gmail_isExpireToken($provider); 
    if($isExpiredToken["status"] == "error") return $isExpiredToken;
    $token = GetConfigs("GMAIL-MAILER-TOKEN"); 
    //Pass the OAuth provider instance to PHPMailer
    $mail->setOAuth(
        new OAuth(
            [
                'provider' => $provider,
                'clientId' => $client_id,
                'clientSecret' => $secret_key,
                'refreshToken'=>$token,
                'userName' => $smtp_user,
            ]
            )
        );
    //Set AuthType to use XOAUTH
    $mail->AuthType = 'XOAUTH2';        
    //$mail->SMTPSecure = false; // Set to false to disable SSL/TLS
    //$mail->SMTPAutoTLS = false; // Disable automatic TLS
    if($debug == "Yes")
    {   $mail->SMTPDebug = 4;
        $mail->Debugoutput = function ($string, $level)
        {
            if (0 < $level)
            {
                logActivity(0,"Google Debug: " . $string, array());
            }
        };
    }
    return $mail;
}
function gmail_test($params=[])
{

    $mail = gmail_instance($params);
    if(is_array($mail)) return $mail;    

    $FromName = GetConfigs("MAIL_FROM_NAME");
    if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
    if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];    
    $email = GetConfigs("SITE_EMAIL");
    if(empty($email)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];
    //Recipients
    $mail->setFrom($FromEmail, $FromName);
    $mail->addAddress($email);//Name is optional
    $Subject = "Test Google";
    $Body = "Testing Google XOAUTH Connection email";
    //Content
    $mail->isHTML(true); 
    $mail->Subject = $Subject;
    $mail->Body = $Body;
    $mail->msgHTML($Body);
    //Your API Code here
    try
    {    
        if($mail->send())
        {
        	//DoUpdateConfigs("GMAIL-MAILER-TOKEN",$token);
        	return ["status"=>"success","message"=>"Email sent successfully".$mail->ErrorInfo];
        }
        return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
    }catch (Exception $e)
    {
      return ["status"=>"error","message"=>"Email Sending Failed: ".$e->getMessage()];
    }
}
function gmail_send($params=[], $mailData=[])
{

    $mail = gmail_instance($params);
    if(is_array($mail)) return $mail;    
    try {
        $Subject= $mailData["Subject"];
        $Body= $mailData["Body"];    
        $ClientId = $mailData["ClientId"];
        $From= $mailData["From"];
        $FromEmail= $From["Email"];
        $FromName= $From["Name"];
    
        $ReplyTo= $mailData["ReplyTo"];
        $ReplyToEmail= $ReplyTo["Email"];
        $ReplyToName= $ReplyTo["Name"];
        
    
        $AddressTo= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
        $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
        $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
        $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information

        //Recipients
        $mail->setFrom($FromEmail, $FromName);
        if(!empty($ReplyToEmail))
        {
            $mail->addReplyTo($ReplyToEmail, $ReplyToName);
        }
        if(!empty($AddressTo))
        {
            foreach ($AddressTo as $add)
            {
                $email = $add["Email"];
                $name = $add["Name"];
                if(empty($name))
                {
                  $mail->addAddress($email);//Name is optional
                }else{
                  $mail->addAddress($email,$name);//Add a recipient
                }
            }
        }
    
        if(!empty($Cc)) $mail->addCC($Cc);
        if(!empty($Bcc)) $mail->addBCC($Bcc);
        if(!empty($Attachments))
        {
            foreach ($Attachments as $file)
            {
                $file = $add["File"];
                $name = $add["Name"];
                if(!empty($file))
                {
                    $mail->addAttachment($file,$name);//Name is optional
                }
            }
        }
          //Content
          $isHtml = ($mailtype == "HTML") ? true:false;
          $mail->isHTML($isHtml); 
          $mail->Subject = $Subject;
          $mail->Body = $Body;
          if($isHtml)
          {
              $mail->msgHTML($Body);
          }else{
              $mail->AltBody = $Body;
          }
            //Your API Code here
            if($mail->send()) return ["status"=>"success","message"=>"Email sent successfully".$mail->ErrorInfo];
          return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
        }catch (Exception $e)
        {
          return ["status"=>"error","message"=>"Email Sending Failed: ".$e->getMessage()];
        }      
}