<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
function mailjet_config()
{
	return [
	"name" => "MailJet",
	"description" => "Default Emailer",
	"version" => "1",//Your Module Version Number
	"author" => "WHASOLS",
	"author_url" => "https://whasols.com/",
	"language" => "english",
	"generate_token" => false,
	"configs" => [
        'api_key' => [
            "Name" => "API Key",
            "Type" => "text",
            "Description" => "API Key Provided in your account.",
             ],
        'secret_key' => [
            "Name" => "Secret Key",
            "Type" => "text",
            "Description" => "Secret Key of given API Key.",
        ],
        'test_recipient' => [
            "Name" => "Test Recipient Email",
            "Type" => "text",
            "Description" => "Email to use for sending test email.",
        ]        
	        ],
	    ];
}
function mailjet_response($response)
{
    if(!is_array($response)) $response = json_decode($response,true);
    $status = _s("status",$response,"");
    if($status == "error") return ["status"=>"error","message"=>"Email Sending Failed: ".$response["message"]];
    $errors = [];
    foreach ($response["Messages"] as $msg)
    {
        $status = $msg["Status"];
        if($status == "error") $errors[] = $msg["Errors"][0]["ErrorMessage"];
    }
    if(empty($errors)) return ["status"=>"success","message"=>"Email sent successfully"];
    return ["status"=>"error","message"=>"Email Sending Failed: ".implode(", ",$errors)];
}
function mailjet_test($params=[]){
    $FromName = GetConfigs("MAIL_FROM_NAME");
    if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
    if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from email empty in Mail Tab"];
    $test_recipient_email = $params['test_recipient'];
    if(empty($test_recipient_email)) return ["status"=>"error","message"=>"Testing Email Not Defined,ADD Test Recipient email at configuration to receive test email."];

    $url = "https://api.mailjet.com/v3.1/send";
    $apiKey = $params['api_key'];
    if(empty($apiKey)) return ["status"=>"error","message"=>"API KEY Empty at Configuration"];
    $apiSecret = $params['secret_key'];
    if(empty($apiSecret)) return ["status"=>"error","message"=>"API Secret Key Empty at Configuration"];
  
    $date = date('Y-m-d H:i:s');
    $Body = "Success!<br>Email testing using mailJet, dated:".$date;

    $data = [
        'Messages' => [
            [
                'From' => [
                    'Email' => $FromEmail,
                    'Name' =>  $FromName
                ],
                'To' => [
                    [
                        'Email' => $test_recipient_email,
                        'Name' => "Tester"
                    ]
                ],
                'Subject' => 'Testing MailJet API ',
                'HTMLPart' => $Body
            ]
        ]
    ];
    $options=[];
    $options["CURLOPT_USERPWD"] = $apiKey . ':' . $apiSecret;
    $options["CURLOPT_HTTPHEADER"] = ['Content-Type: application/json'];
    $response = curlCall($url,$data,$options);
    return mailjet_response($response);
}
function mailjet_send($params=[], $mailData=[])
{    
    $apiKey = $params['api_key'];
    if(empty($apiKey)) return ["status"=>"error","message"=>"API KEY Empty at Configuration"];
    
    $apiSecret = $params['secret_key'];
    if(empty($apiSecret)) return ["status"=>"error","message"=>"API Secret Key Empty at Configuration"];

    $FromName = $From["Name"];
    if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
    
    $FromEmail = $From["Email"];
    if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];
    $url = "https://api.mailjet.com/v3.1/send";
    $Subject= $mailData["Subject"];
    $Body= $mailData["Body"];

    $ClientId = $mailData["ClientId"];
    $From= $mailData["From"];

    $ReplyTo= $mailData["ReplyTo"];
    $ReplyToEmail= $ReplyTo["Email"];
    $ReplyToName= $ReplyTo["Name"];

    $tos= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
    $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
    $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
    $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
    
    $data = [
        'Messages' => [
            [
                'From' => [
                    'Email' => $FromEmail,
                    'Name' =>  $FromName
                ],
                'To' => $tos,
                'Subject' => $Subject,
                'HTMLPart' => $Body
            ]
        ]
    ];
    $options=[];
    $options["CURLOPT_USERPWD"] = $apiKey . ':' . $apiSecret;
    $options["CURLOPT_HTTPHEADER"] = ['Content-Type: application/json'];
    $response = curlCall($url,$data,$options);
    return mailjet_response($response);
}