<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use Carbon\Carbon;
function sendmail_disabled()
{
    $php_ini_path = php_ini_loaded_file();
    if ($php_ini_path)
    {
        // Read the contents of the php.ini file
        $ini_contents = file_get_contents($php_ini_path);
        // Check if the disable_functions directive contains the mail function
        if (preg_match('/^disable_functions\s*=\s*(.*)$/m', $ini_contents, $matches))
        {
            $disabled_functions = array_map('trim', explode(',', $matches[1]));
            if (in_array('mail', $disabled_functions)) return ["status"=>"error","message"=>"The mail function is disabled in your hosting server php.ini file."];
        }
    }
    return [];
}
function sendmail_config()
{
	$configarray = [
	"name" => "Mail()",
	"description" => "Default Emailer",
	"version" => "1",//Your Module Version Number
	"author" => "WHASOLS",
	"author_url" => "https://whasols.com/",
	"language" => "english",
	"generate_token" => false,
	"configs" => [
            "encryption" => [ //Dropdown Select box
                    "Name" => "Mail Encoding", 
                    "Type" =>  "dropdown", 
                    "Options" => ['UTF-8'=>'UTF-8','8bit'=>'8bit','7bit'=>'7bit','Binary'=>'Binary','Base64'=>'Base64'], 
                    "Description" => "Select Mail Encoding Type", 
                    "Default" => "UTF-8",
                    "DivColumns" => 4,
                ],
            "debug" => [
                "Name" => "Debugging",
                "Type" => "radio",
                "Options" => ["Yes"=>"Enable debugging output","No"=>"Disable"],
                "Description" => "Select email debugging option",
                "DivColumns" => 12,
                "Default" => "Yes",
                "Tooltip" => "If Debugging option selected as Enable then Debugging logs will be stored in database as activity logs and you can check at logs page",
               ]
	        ],
	    ];
    $isDisabled = sendmail_disabled();
    if(!empty($isDisabled))
    {
    $configarray["configs"]['content'] = [
            "Name" => "Extra Content",
            "Type" => "system",
            "Description" => "",
            "DivColumns" => 12,
            "Content" => '<div class="alert alert-info ws-smtp-info">'.$isDisabled["message"].'</div>',
           ];
    }
	return $configarray;
}
function sendmail_test($params=[])
{
    $isDisabled = sendmail_disabled();
    if(!empty($isDisabled)) return $isDisabled;
    try {
        $encryption = $params["encryption"];
        $debug= $params["debug"];
        //Create a new PHPMailer instance
        $mail = new PHPMailer(true);
        $FromName = GetConfigs("MAIL_FROM_NAME");
        if(empty($FromName)) return ["status"=>"error","message"=>"Mail from Name empty in Mail Tab"];
        $FromEmail = GetConfigs("MAIL_FROM_EMAIL");
        if(empty($FromEmail)) return ["status"=>"error","message"=>"Mail from Email empty in Mail Tab"];
        $mail->setFrom($FromEmail, $FromName);
        $mail->addReplyTo($FromEmail, $FromName);
        $email = GetConfigs("SITE_EMAIL");
        if(empty($email)) return ["status"=>"error","message"=>"Email Address empty in General Tab"];
        $mail->addAddress($email,$FromName);//Name is optional
        $mail->Subject = "Testing Email";
        $date = date('Y-m-d H:i:s');
        $Body = "Email testing using mail(), dated:".$date;
        $mail->msgHTML($Body);
        //Replace the plain text body with one created manually
        $mail->AltBody = $Body;
        if($mail->send()) return ["status"=>"success","message"=>"Email sent successfully"];
        return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
    }
    catch (Exception $e)
    {
        return ["status"=>"error","message"=>"Mailer Error: ".$e->getMessage()];
    }     
}

function sendmail_send($params=[], $mailData=[])
{
    $isDisabled = sendmail_disabled();
    if(!empty($isDisabled)) return $isDisabled;
try {
    //Instantiation and passing `true` enables exceptions
    $mail = new PHPMailer(true);
    $mail->Timeout = 10; // set the timeout (seconds)
    $Subject= $mailData["Subject"];
    $Body= $mailData["Body"];

    $ClientId = $mailData["ClientId"];
    $From= $mailData["From"];
    $FromEmail= $From["Email"];
    $FromName= $From["Name"];
    $ReplyTo= $mailData["ReplyTo"];
    $ReplyToEmail= $ReplyTo["Email"];
    $ReplyToName= $ReplyTo["Name"];
    $AddressTo= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
    $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
    $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
    $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information

    /************************** Mailer Configs ************************/
    $encryption = $params["encryption"];
    $debug= $params["debug"];
    if($debug == "Yes")
    {   $mail->SMTPDebug = 4;
        $mail->Debugoutput = function ($string, $level)
        {
            if (0 < $level)
            {
                logActivity(0,"SMTP Debug: " . htmlentities($string), array());
            }
        };
    }
    //Recipients
    $mail->setFrom($FromEmail, $FromName);
    if(!empty($ReplyToEmail))
    {
        $mail->addReplyTo($ReplyToEmail, $ReplyToName);
    }
    if(!empty($AddressTo))
    {
        foreach ($AddressTo as $add)
        {
            $email = $add["Email"];
            $name = $add["Name"];
            if(empty($name))
            {
              $mail->addAddress($email);//Name is optional
            }else{
              $mail->addAddress($email,$name);//Add a recipient
            }
        }
    }

    if(!empty($Cc)) $mail->addCC($Cc);
    if(!empty($Bcc)) $mail->addBCC($Bcc);
    if(!empty($Attachments))
    {
        foreach ($Attachments as $file)
        {
            $file = $add["File"];
            $name = $add["Name"];
            if(!empty($file))
            {
                $mail->addAttachment($file,$name);//Name is optional
            }
        }
    }
      //Content
      $mail->isHTML(true); 
      $mail->Subject = $Subject;
      $mail->msgHTML($Body);
      $mail->AltBody = $Body;
    if($mail->send()) return ["status"=>"success","message"=>"Email sent successfully"];
      return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
    }catch (Exception $e)
    {
      return ["status"=>"error","message"=>"Email Sending Failed: ".$e->getMessage()];
    }      
}