<?php
if ( ! defined('WHASOLS')) exit('No direct script access allowed');
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;
use Carbon\Carbon;
$smtp_errors = [];
function smtp_config()
{
    $SITE_URL = GetConfigs('SITE_URL');
    $SITE_URL = rtrim($SITE_URL,"/");
    $myhost = strtolower(trim($SITE_URL));
    $count = substr_count($myhost, '.');
    if($count === 2)
    {
        if(strlen(explode('.', $myhost)[1]) > 3) $myhost = explode('.', $myhost, 2)[1];
    }
    else
    if($count > 2)
    {
        if(function_exists('get_domain'))
        {
            $myhost = get_domain(explode('.', $myhost, 2)[1]);
        }
    }
    $smtpServer = str_replace(["https://","http://"],["",""],$myhost);
    $configs = [ // Configuration fields 
            "type" => [ //Dropdown Select box
                    "Name" => "Mail Type", 
                    "Type" =>  "dropdown",
                    "Options" => ['TEXT'=>'Text','HTML'=>'HTML'], 
                    "Description" => "Select Mail Type", 
                    "Default" => "HTML",
                    "DivColumns" => 6,
                ],
            "encryption" => [ //Dropdown Select box
                    "Name" => "Mail Encoding", 
                    "Type" =>  "dropdown", 
                    "Options" => ['UTF-8'=>'UTF-8','8bit'=>'8bit','7bit'=>'7bit','Binary'=>'Binary','Base64'=>'Base64'], 
                    "Description" => "Select Mail Encoding Type", 
                    "Default" => "UTF-8",
                    "DivColumns" => 4,
                ],
                "ssl" => [ //Dropdown Select box
                    "Name" => "SMTP SSL Type", 
                    "Type" =>  "dropdown", 
                    "Options" => ['NONE'=>'None','SSL'=>'SSL','TLS'=>'TLS'], 
                    "Description" => "Dropdown Description here", 
                    "Default" => "NONE", 
                    "DivColumns" => 4,
                ],
                'port' => [
                        "Name" => "SMTP Port #",
                        "Type" => "number",
                        "Min" => 0,
                        "Max" => 99999,
                        "Description" => "SMTP Port #",
                        "DivColumns" => 4,
                        "Default" => 587,
                        "Tooltip" => "Enter SMTP Port as 465 or 587 or any valid port number, Default Port for SSL is 465 and for TLS is 587",
                       ],
                'hostname' => [
                        "Name" => "SMTP Hostname",
                        "Type" => "text",
                        "Description" => "Enter Hostname here.",
                        "DivColumns" => 6,
                        "Default" => $smtpServer, 
                        "Tooltip" => "Enter SMTP hostname below as mail.yourdomain.com or any valid hostname",
                         ],
                'email' => [
                        "Name" => "SMTP Email/Username",
                        "Type" => "email",
                        "Description" => "Enter Email/Username here.",
                        "DivColumns" => 6,
                        "Tooltip" => "Enter SMTP Email or Username below in Email field",
                       ],
                'password' => [
                        "Name" => "SMTP Password",
                        "Type" => "password",
                        "Description" => "Enter Password here.",
                        "DivColumns" => 6,
                        "Tooltip" => "Enter SMTP Email password below in password field",
                       ],
            ];
        $configs['debug'] = [
                "Name" => "Debugging",
                "Type" => "radio",
                "Options" => ["Yes"=>"Enable debugging output","No"=>"Disable"],
                "Description" => "Select email debugging option",
                "DivColumns" => 12,
                "Default"=>"Yes",
                "Tooltip" => "If Debugging option selected as Enable then Debugging logs will be stored in database as activity logs and you can check at logs page",
               ];
        $configs['content'] = [
                "Name" => "Extra Content",
                "Type" => "system",
                "Description" => "",
                "DivColumns" => 12,
                "Content" => '<div class="alert alert-info ws-smtp-info">Some Time during SMTP Connection 465 Port not working using third-party email providers as (cpanel,cwp,cyberpanel and others), so try 587 port.<p>Before testing email, please save the email credentials using <b>Save Changes</b> button then Click on <b>Send Test Email</b> button</p></div>',
               ];
    $configarray = [
    "name" => "SMTP",
    "description" => "SMTP",
    "version" => "1",//Your Module Version Number
    "author" => "WHASOLS",
    "author_url" => "https://whasols.com/",
    "language" => "english",
    "generate_token" => false,
    "configs" => $configs,
        ];
    return $configarray;
}
function smtp_test($params=[])
{
    $smtp_port = $params["port"];
    $smtp_host = $params["hostname"];
    $smtp_user = $params["email"];
    $smtp_pass = $params["password"];
    $mailtype = $params["type"];
    $ssl = $params["ssl"];
    $debug= $params["debug"];
    try {
    $mail = new PHPMailer(true);
    $mail->SMTPAutoTLS = false;
    $mail->Timeout = 10; // set the timeout (seconds)
    if(empty($smtp_host)) return ["status"=>"error","message"=>"SMTP Hostname empty"];
    if(empty($smtp_user)) return ["status"=>"error","message"=>"SMTP Username/Email empty"];
    if(empty($smtp_pass)) return ["status"=>"error","message"=>"SMTP Password empty"];
    if(empty($smtp_port)) return ["status"=>"error","message"=>"SMTP Port empty"];
    if($ssl == "TLS")
    {
        $mail->SMTPSecure = "tls";
    }
    elseif($ssl == "SSL")
    {
        $mail->SMTPSecure = "ssl";
    }else{
       $mail->SMTPSecure = "tls";         //Enable TLS encryption; `PHPMailer::ENCRYPTION_SMTPS` encouraged 
    }        
    //$mail->SMTPDebug = 2; // 2 for detailed debugging
    $mail->SMTPAuth = true;
    //$mail->SMTPDebug = 4;
    $mail->Username = $smtp_user;
    $mail->Password = $smtp_pass;
    $mail->Host = $smtp_host;
    $mail->Port = $smtp_port;
    $validCredentials = false;
    $validCredentials = $mail->SmtpConnect();
    return ["status"=>"success","message"=>"SMTP connected successfully"];
    }
    catch (Exception $e)
    {
        return ["status"=>"error","message"=>"Mailer Error: ".$e->getMessage()];
    }
}
function smtp_send($params=[], $mailData=[])
{
    try {
        $Subject= $mailData["Subject"];
        $Body= $mailData["Body"];
    
        $ClientId = $mailData["ClientId"];
        $From= $mailData["From"];
        $FromEmail= $From["Email"];
        $FromName= $From["Name"];
    
        $ReplyTo= $mailData["ReplyTo"];
        $ReplyToEmail= $ReplyTo["Email"];
        $ReplyToName= $ReplyTo["Name"];
        
    
        $AddressTo= $mailData["AddressTo"]; //Array = Email sent to "address to", AddressTo is an Array which have Email and Name information.
        $Cc= $mailData["Cc"]; //Cc Array = Emails of CC.
        $Bcc= $mailData["Bcc"]; //BccArray = Emails of Bcc.
        $Attachments= $mailData["Attachments"]; //Attachments Array= Which have file and name information
          
          
        /************************** Mailer Configs ************************/
        $smtp_port = $params["port"];
        $smtp_host = $params["hostname"];
        $smtp_user = $params["email"];
        $smtp_pass = $params["password"];
        $mailtype = $params["type"];
        $ssl = $params["ssl"];
        $debug= $params["debug"];
        $encryption= strtolower(_s("encryption",$params,"8bit"));
        if(empty($smtp_host)) return ["status"=>"error","message"=>"SMTP Hostname empty"];
        if(empty($smtp_user)) return ["status"=>"error","message"=>"SMTP Username/Email empty"];
        if(empty($smtp_pass)) return ["status"=>"error","message"=>"SMTP Password empty"];
        if(empty($smtp_port)) return ["status"=>"error","message"=>"SMTP Port empty"];

        
        //Create a new PHPMailer instance
        $mail = new PHPMailer();
        $mail->SMTPAutoTLS = false;
        $mail->Timeout = 10; // set the timeout (seconds)
        $mail->CharSet = GetConfigs("SITE_CHARSET");
        $mail->XMailer = GetConfigs("SITE_NAME");

        $mail->Sender = $FromEmail;
        $filePath = __FILE__;  // Full path of the current PHP script
        $remoteAddr = $_SERVER['SERVER_ADDR'];  // Client IP address
    
        $mail->addCustomHeader('X-PHP-Script', basename($filePath) . ' for ' . $remoteAddr);
        $mail->addCustomHeader('X-PHP-Filename', $filePath . ' REMOTE_ADDR: ' . $remoteAddr);
        $mail->addCustomHeader('X-Priority', '1');       


        //Tell PHPMailer to use SMTP
        $mail->isSMTP();
        if($ssl == "TLS")
        {
            $mail->SMTPSecure = "tls";
        }
        elseif($ssl == "SSL")
        {
            $mail->SMTPSecure = "ssl";
        }else{
           $mail->SMTPSecure = "tls";         //Enable TLS encryption; `PHPMailer::ENCRYPTION_SMTPS` encouraged 
        }        
        //Enable SMTP debugging
        //SMTP::DEBUG_OFF = off (for production use)
        //SMTP::DEBUG_CLIENT = client messages
        //SMTP::DEBUG_SERVER = client and server messages
        $logMessage = '';
        if($debug == "Yes")
        {   
            $mail->SMTPDebug = 4;
            $mail->Debugoutput = function ($string, $level)
            {
                if (0 < $level)
                {
                    logActivity(0,"SMTP Debug: " . htmlentities($string), array());
                }
            };
        }
        //Set the hostname of the mail server
        $mail->Host = $smtp_host;
        //Set the SMTP port number - likely to be 25, 465 or 587
        $mail->Port = (int)$smtp_port > 0 ? $smtp_port : 587;
        //Whether to use SMTP authentication
        $mail->SMTPAuth = true;
        //Username to use for SMTP authentication
        $mail->Username = $smtp_user;
        //Password to use for SMTP authentication
        $mail->Password = $smtp_pass;
        $mail->SMTPOptions = array(
            'ssl' => array(
                'verify_peer' => false,
                'verify_peer_name' => false,
                'allow_self_signed' => true
            )
        );

        //Recipients
        $mail->setFrom($FromEmail, $FromName);
        if(!empty($ReplyToEmail))
        {
            $mail->addReplyTo($ReplyToEmail, $ReplyToName);
        }
        if(!empty($AddressTo))
        {
            foreach ($AddressTo as $add)
            {
                $email = $add["Email"];
                $name = $add["Name"];
                if(empty($name))
                {
                  $mail->addAddress($email);//Name is optional
                }else{
                  $mail->addAddress($email,$name);//Add a recipient
                }
            }
        }
    
        if(!empty($Cc)) $mail->addCC($Cc);
        if(!empty($Bcc)) $mail->addBCC($Bcc);
        //Content
        $mail->Subject = $Subject;
        $mail->Encoding = $encryption;
        $isHtml = ($mailtype == "HTML") ? true:false;
        $mail->isHTML($isHtml); 
        if(!empty($Body))
        {
            $mail->Body =$Body;
            $mail->AltBody = "";
        }else{
            $mail->AltBody = $Body;
        }
        //$BodyText = $mail->html2text($Body);
        if(!empty($Attachments))
        {
            foreach ($Attachments as $file)
            {
                $file = $add["File"];
                $name = $add["Name"];
                if(!empty($file))
                {
                    $mail->addAttachment($file,$name);//Name is optional
                }
            }
        }
        //Your API Code here
        if($mail->send()) return ["status"=>"success","message"=>"Email sent successfully"];
        return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
    }catch (phpmailerException $e)
    {
      return ["status"=>"error","message"=>"Email Sending Failed: ".$mail->ErrorInfo];
    }      
}