<?php
function fraudlabs_MetaData()
{
    return array("DisplayName" => "FraudLabs Pro", "SupportsRechecks" => true, "APIVersion" => "1.2");
}
function fraudlabs_getConfigArray()
{
    return array("Enable" => array("FriendlyName" => "Enable FraudLabs Pro", "Type" => "yesno", "Description" => "Tick to enable FraudLabs Pro Fraud Checking for Orders"), "licenseKey" => array("FriendlyName" => "FraudLabs Pro License Key", "Type" => "text", "Size" => "30", "Description" => "Don't have an account? " . "<a href=\"https://whasols.com/go/6/fraudlabs-create-account\" class=\"autoLinked\" target=\"_blank\">" . "Click here to sign up &raquo;</a>"), "riskScore" => array("FriendlyName" => "FraudLabs Pro Fraud Risk Score", "Type" => "text", "Size" => "2", "Default" => 20, "Description" => "Higher than this value and the order will be blocked (1 -> 100)"), "rejectFreeEmail" => array("FriendlyName" => "Reject Free Email Service", "Type" => "yesno", "Description" => "Block orders from free email addresses such as Hotmail & Yahoo!"), "rejectCountryMismatch" => array("FriendlyName" => "Reject Country Mismatch", "Type" => "yesno", "Description" => "Block orders where order address is different from IP Location"), "rejectAnonymousNetwork" => array("FriendlyName" => "Reject Anonymous Networks", "Type" => "yesno", "Description" => "Block orders where the user is ordering through an anonymous network"), "rejectHighRiskCountry" => array("FriendlyName" => "Reject High Risk Country", "Type" => "yesno", "Description" => "Block orders from high risk countries"));
}
function fraudlabs_call($data)
{
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => 'https://api.fraudlabspro.com/v1/order/screen?'.http_build_query($data),
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => 'POST',
    ));
    $response = curl_exec($curl);
    curl_close($curl);
    logModuleCall('fraudlabs',"doFraudCheck", $data, $response,[]);
    $response = json_decode($response,true);
    return $response;
}
function fraudlabs_hash($string)
{
    $hash = "fraudlabspro_" . $string;
    for ($i = 0; $i < 65536; $i++) {
        $hash = sha1("fraudlabspro_" . $hash);
    }
    return $hash;
}
function fraudlabs_validateRules(array $params,array $response)
{
    $maxRiskScore = (int) $params["riskScore"];
    if (0 < $maxRiskScore && $maxRiskScore < (int) $response["fraudlabspro_score"])return \Lang::trans("fraudlabs.highFraudRiskScore");
    if ($response["fraudlabspro_status"] != "APPROVE") return \Lang::trans("fraudlabs.manualReview");

    if ($params["rejectCountryMismatch"] == "yes" && $response["is_country_match"] == "N")return \Lang::trans("fraudlabs.countryMismatch");
    if ($params["rejectAnonymousNetwork"] == "yes" && $response["is_proxy_ip_address"] == "Y") return \Lang::trans("fraudlabs.anonymousProxy");
    if ($params["rejectHighRiskCountry"] == "yes"  && $response["is_high_risk_country"] == "Y") return \Lang::trans("fraudlabs.highRiskCountry");
    return "";
}
function fraudlabs_doFraudCheck(array $params, $checkOnly = false)
{
    $emailDomain = explode("@", $params["clientsdetails"]["email"], 2);
    $emailDomain = isset($emailDomain[1]) ? $emailDomain[1] : "";
    $billing = array();
    $billing["first_name"] = $params["clientsdetails"]["firstname"];
    $billing["last_name"] = $params["clientsdetails"]["lastname"];
    if ($params["clientsdetails"]["address1"]) {
        $billing["bill_addr"] = $params["clientsdetails"]["address1"];
    }
    if ($params["clientsdetails"]["city"]) {
        $billing["bill_city"] = $params["clientsdetails"]["city"];
    }
    if ($params["clientsdetails"]["state"]) {
        $billing["bill_state"] = $params["clientsdetails"]["state"];
    }
    if ($params["clientsdetails"]["postcode"]) {
        $billing["bill_zip_code"] = $params["clientsdetails"]["postcode"];
    }
    $billing["bill_country"] = $params["clientsdetails"]["country"];
    $billing["user_phone"] = $params["clientsdetails"]["phonenumber"];
    $currencyCode = $params["clientsdetails"]["currencycode"];
    $request = array_merge(array("key"=>$params["licenseKey"],"ip" =>$params["ip"], "format" => "json", "email_domain" => $emailDomain, "email" => $params["clientsdetails"]["email"], "email_hash" => fraudlabs_hash($params["clientsdetails"]["email"]), "user_order_id" => substr($params["order"]["order_number"], 0, 15), "amount" => $params["order"]["amount"], "currency" => $currencyCode), $billing);

    $response = fraudlabs_call($request);
    if(in_array($response["fraudlabspro_error_code"],[101, 102, 103, 104, 203, 204, 210, 211])) return ["status"=>"error","message"=>$response["fraudlabspro_message"],"data"=>$response];
    $IsValid = fraudlabs_validateRules($params, $response);
    if (!$checkOnly && !empty($IsValid)) return ["status"=>"error","message"=>$IsValid,"data"=>$response,"fraud"=>true];
    return ["status"=>"success","message"=>"Fraud Checks Processed successfully","data"=>$response];
}

function fraudlabs_doReportConfigArray(array $params)
{
    return [
        "tag" => ["FriendlyName" => "Action", "Type" => "dropdown", "Default" =>"", "Description" => "<br><br><br>Perform APPROVE, REJECT, or REJECT_BLACKLIST action to transaction.<ul>
        <li>APPROVE: Approve a transaction</li>
        <li>REJECT: Reject a transaction</li>
        <li>REJECT_BLACKLIST: Reject and blacklist the transaction</li>
        </ul><br>
When a transaction was blacklisted, the information will be recorded into the system for future blacklist validation check.","Options"=>["APPROVE"=>"Approve a transaction","REJECT"=>"Reject a transaction","REJECT_BLACKLIST"=>"Reject and blacklist the transaction"]],
        "notes" => ["FriendlyName" => "Note", "Type" => "textarea", "Default" =>"", "Description" => "Enter Note for Report Request"]
    ];
}
function fraudlabs_doReport(array $params)
{
    $licenseKey = $params["licenseKey"];
    $tag = $params["tag"];
    $notes = $params["notes"];
    if(empty($licenseKey)) return ["status"=>"error","message"=>"FraudLabPro API Key empty"];
    if(empty($tag)) return ["status"=>"error","message"=>"Please Select Status"];
    if(empty($notes)) return ["status"=>"error","message"=>"Please Enter Note"];
    $data = [];
    $data["key"] = $licenseKey;
    $data["id"] = $params["data"]["fraudlabspro_id"];
    $data["action"] = $tag;
    $data["note"] = $notes;    
    $data["format"] = "json";    
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => 'https://api.fraudlabspro.com/v2/order/feedback',
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => json_encode($data),
    ));
    $response = curl_exec($curl);
    curl_close($curl);   
    $response = json_decode($response,true);
    logModuleCall('fraudlabs',__FUNCTION__, $data, $response,[]);
    if(array_key_exists("error",$response)) return ["status"=>"error","message"=>"FraudLabPro Error: ".$response["error"]["error_message"]." (".$response["error"]["error_code"].")"];
    return ["status"=>"success","message"=>"FraudLabPro Reported successfully"];
    
}
function fraudlabs_processResultsForDisplay(array $params)
{
    $fraudResults = $params["data"];
    $fieldMapping = array("ip" => array("ip_country", "is_country_match", "ip_region", "ip_city", "ip_continent", "ip_latitude", "ip_longitude", "ip_timezone", "ip_domain", "ip_isp_name", "ip_usage_type", "is_proxy_ip_address", "is_ip_blacklist"), "billing" => array("is_high_risk_country", "is_phone_blacklist", "is_export_controlled_country", "user_order_id"), "email" => array("is_free_email", "is_disposable_email", "is_new_domain_name", "is_domain_exists", "is_email_blacklist"), "general" => array("fraudlabspro_id", "fraudlabspro_score", "fraudlabspro_status", "fraudlabspro_distribution","fraudlabspro_version", "fraudlabspro_credits"));
    $booleanFields = array("is_country_match", "is_high_risk_country", "is_free_email", "is_disposable_email", "is_new_domain_name", "is_domain_exists", "is_proxy_ip_address", "is_ip_blacklist", "is_email_blacklist", "is_phone_blacklist", "is_export_controlled_country");
    $panels = array();
    foreach ($fieldMapping as $panelTitle => $panelElements)
    {
        $panelValues = array();
        foreach ($panelElements as $element)
        {
            if($element == "fraudlabspro_id")
            {
                $panelValues[$element] = '<a href="https://www.fraudlabspro.com/merchant/transaction-details/'.$fraudResults[$element].'" target="_blank">'.$fraudResults[$element].'</a>';
            }else if($element == "fraudlabspro_score")
            {
                $panelValues[$element] = $fraudResults[$element].' <img src="https://cdn.fraudlabspro.com/assets/img/scores/bar-'.$fraudResults[$element].'.png" style="width: 170px;height: 18px;">';
            }else if($element == "fraudlabspro_status")
            {
                $badgebg = $fraudResults[$element] == "APPROVE" ? "bg-success":"bg-danger";
                $panelValues[$element] = '<span class="badge '.$badgebg.'">'.$fraudResults[$element].'</span>';
            }else{
                $elementValue = $fraudResults[$element];
                if(in_array($element,$booleanFields)) $elementValue = $elementValue == true ? 'Yes':'No';
                if(!empty($elementValue)) $panelValues[$element] = $elementValue;
            }
            
        }
        if(!empty($panelValues)) $panels[$panelTitle] = $panelValues;
    }
    $errorCode = $fraudResults["fraudlabspro_error_code"];
    if ($errorCode) {
        $panelValues["general"]["error"] = $fraudResults["fraudlabspro_message"];
    }else{
        $panelValues["general"]["error"] = "";
    }
    $errorMessage = "";
    if (!empty($panelValues["general"]["error"]))
    {
        $errorMessage = $fraudResults["fraudlabspro_message"] . " (" . $fraudResults["fraudlabspro_error_code"] . ")";
    }
    return ["ErrorMsg" => $errorMessage,"Panels" => $panels];
}