<?php
use Illuminate\Database\Capsule\Manager as Capsule;
function ipqualityscore_MetaData()
{
    return array("DisplayName" => "IpQualityScore", "SupportsRechecks" => true, "APIVersion" => "1.0");
}
function ipqualityscore_getConfigArray()
{
    return array("Enable" => array("FriendlyName" => "Enable IpQualityScore", "Type" => "yesno", "Description" => "Tick to enable IpQualityScore Fraud Checking for Orders"), "licenseKey" => array("FriendlyName" => "IpQualityScore License Key", "Type" => "text", "Size" => "30", "Description" => "Don't have an account? <a href=\"https://whasols.com/go/4/maxmind\" class=\"autoLinked\" target=\"_blank\">Click here to sign up &raquo;</a>"), "riskScore" => array("FriendlyName" => "Fraud Risk Score", "Type" => "text", "Size" => "2", "Default" => 20, "Description" => "Higher than this value and the order will be blocked (0.01 -> 99)"),
        "fastCheck" => array("FriendlyName" => "Fast Check", "Type" => "yesno", "Description" => "When this parameter is enabled our API will not perform certain forensic checks that take longer to process. Enabling this feature greatly increases the API speed without much impact on accuracy. This option is intended for services that require decision making in a time sensitive manner and can be used for any strictness level."),
        "strictness" => array("FriendlyName" => "Strictness", "Type" => "radio", "Description" => "How in depth (strict) do you want this query to be? Higher values take longer to process and may provide a higher false-positive rate. We recommend starting at \"0\", the lowest strictness setting, and increasing to \"1\" or \"2\" depending on your levels of fraud.","Options"=>[1,2,3,4,5]),
        "allow_public_access_points" => array("FriendlyName" => "Allow Public Access Points", "Type" => "yesno", "Description" => "Bypasses certain checks for IP addresses from education and research institutions, schools, and some corporate connections to better accommodate audiences that frequently use public connections."),
        "transaction_strictness" => array("FriendlyName" => "Transaction Strictness", "Type" => "radio", "Description" => "Adjusts the weights for penalties applied due to irregularities and fraudulent patterns detected on order and transaction details that can be optionally provided on each API request. This feature is only beneficial if you are passing order and transaction details. A table is available further down the page with supported transaction variables.","Options"=>[1,2]),
        "rejectCrawlerVpnTor" => array("FriendlyName" => "Reject Crawler, VPN, Tors", "Type" => "yesno", "Description" => "Block orders if User Agent (Crawler, TOR or VPN)"));
}
function ipqualityscore_get($response=[],$param = "")
{
    $value = $response;
    $keys = explode('.', $param); // Split the parameter by '.'
    // Traverse each key to reach the nested value
    foreach ($keys as $key) {
        if (isset($value[$key]))
        {
            $value = $value[$key];
        } else {
            return "";
        }
    }
    return $value;
} 
function ipqualityscore_activate(array $params = array())
{
}
function ipqualityscore_call($data,$params)
{
    $licenseKey = $params["licenseKey"];
    $formattedParameters = http_build_query($data);
    $url = sprintf(
        'https://www.ipqualityscore.com/api/json/ip/%s/%s?%s',
        $licenseKey,
        $params["ip"],
        $formattedParameters
    );
    $curl = curl_init();
    curl_setopt_array($curl,[CURLOPT_URL => $url,CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,CURLOPT_FOLLOWLOCATION => true]);
    $response = curl_exec($curl);
    curl_close($curl);
    logModuleCall('ipqualityscore',__FUNCTION__, $data, $response,[]);
    if(empty($response)) return ["status"=>"error","message"=>"IpQualityScore API Response empty"];
    $response = json_decode($response,true);
    return $response;
}
function ipqualityscore_validateRules(array $params,array $response)
{
    if(!$response["success"]) return $response["message"];
    $maxRiskScore = (int) $params["riskScore"];
    if (0 < $maxRiskScore && $maxRiskScore < $response["fraud_score"]) return \Lang::trans("ipqualityscore_highfraudriskscore");
    if($params["rejectCrawlerVpnTor"] == "yes")
    {
        if($response["proxy"]) return \Lang::trans("ipqualityscore_proxy");
        if($response["tor"]) return \Lang::trans("ipqualityscore_tor");
        if($response["active_tor"]) return \Lang::trans("ipqualityscore_tor");
        if($response["vpn"]) return \Lang::trans("ipqualityscore_vpn");
        if($response["active_vpn"]) return \Lang::trans("ipqualityscore_vpn");
    }
    return "";
}
function ipqualityscore_doFraudCheck(array $params, $checkOnly = false)
{
    $request = array(
        'key' => $_SERVER["HTTP_HOST"],
        'user_agent' => $params["userAgent"],
        'user_language' =>$params["acceptLanguage"],
        'strictness' =>$params["strictness"]>0 ? $params["strictness"]:1,
        'allow_public_access_points' =>$params["allow_public_access_points"] == "yes" ? true:false,
        'lighter_penalties' =>false
    );
    $request['billing_first_name'] = $params["clientsdetails"]["firstname"];
    $request['billing_last_name'] = $params["clientsdetails"]["lastname"];
    $request['billing_company'] = $params["clientsdetails"]["companyname"];
    $request['billing_country'] = $params["clientsdetails"]["country"];
    $request['billing_address_1'] = $params["clientsdetails"]["address1"];
    $request['billing_city'] = $params["clientsdetails"]["city"];
    $request['billing_state'] = $params["clientsdetails"]['state'];
    $request['billing_zipcode'] = $params["clientsdetails"]['postcode'];
    $request['billing_phone'] = $params["clientsdetails"]['phonenumber'];
    $request['billing_email'] = $params["clientsdetails"]['email'];

    $response = ipqualityscore_call($request,$params);
    if(array_key_exists("status",$response))
    {
        if($response["status"] == "error") return $response;
    }
    $IsValid = ipqualityscore_validateRules($params, $response);
    if (!$checkOnly && !empty($IsValid)) return ["status"=>"error","message"=>$IsValid,"data"=>$response,"fraud"=>true];
    return ["status"=>"success","message"=>"Fraud Checks Processed successfully","data"=>$response];
}
function ipqualityscore_doReport(array $params)
{
    $licenseKey = $params["licenseKey"];
    $data = _s("data",$params,[]);
    $request_id = _s("request_id",$data,"");
    if(empty($licenseKey)) return ["status"=>"error","message"=>"API Key empty"];
    if(empty($request_id)) return ["status"=>"error","message"=>"Fraud Check Request ID empty"];
    $formattedParameters = http_build_query($data);
    $url = "https://www.ipqualityscore.com/api/json/report/".$licenseKey."?request_id=".$request_id;
    $curl = curl_init();
    curl_setopt_array($curl,[CURLOPT_URL => $url,CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,CURLOPT_FOLLOWLOCATION => true]);
    $response = curl_exec($curl);
    curl_close($curl);
    logModuleCall('ipqualityscore',__FUNCTION__, $data, $response,[]);
    if(empty($response)) return ["status"=>"error","message"=>"IpQualityScore API Response empty"];
    $response = json_decode($response,true);
    if(_s('success',$response,"") === true) return ["status"=>"success","message"=>"IpQualityScore Reported successfully"];
    return ["status"=>"error","message"=>_s('message',$response,"")];
}
function ipqualityscore_processResultsForDisplay(array $params)
{
    $fraudResults = $params["data"];
    $fieldMapping = array("ip" => ["country_code","region","city","ISP","ASN","operating_system","browser","organization","is_crawler","timezone","mobile","host","proxy","vpn","tor","active_vpn","active_tor","device_brand","device_model","bot_status","connection_type","zip_code","latitude","longitude"],
        "email.desc" => ["transaction_details.address_email_identity_match","transaction_details.email_name_identity_match","transaction_details.leaked_billing_email","transaction_details.valid_billing_email","transaction_details.phone_email_identity_match","transaction_details.risk_score"],
        "billing" => ["transaction_details.risky_billing_phone","transaction_details.billing_phone_country","transaction_details.billing_phone_country_code","transaction_details.billing_phone_carrier","transaction_details.billing_phone_line_type","transaction_details.fraudulent_behavior","transaction_details.risky_username","transaction_details.valid_billing_phone","transaction_details.leaked_user_data","transaction_details.phone_name_identity_match","transaction_details.phone_address_identity_match","transaction_details.name_address_identity_match"],
        "credit_card.desc" => ["transaction_details.is_prepaid_card","transaction_details.bin_country","transaction_details.bin_type","transaction_details.bin_bank_name"],
        "general" => ["request_id","fraud_score","recent_abuse","abuse_velocity",]);

    $booleanFields = ["is_crawler","mobile","proxy","vpn","tor","active_vpn","active_tor","recent_abuse","bot_status","transaction_details.valid_billing_address","transaction_details.valid_shipping_address","transaction_details.valid_billing_email","transaction_details.valid_shipping_email","transaction_details.risky_billing_phone","transaction_details.risky_shipping_phone","transaction_details.fraudulent_behavior","transaction_details.valid_billing_phone","transaction_details.valid_shipping_phone","transaction_details.leaked_billing_email","transaction_details.leaked_shipping_email","transaction_details.leaked_user_data","transaction_details.is_prepaid_card"];

    $panels = array();
    foreach ($fieldMapping as $panelTitle => $panelElements)
    {
        $panelValues = array();
        foreach ($panelElements as $element)
        {
            $elementValue = ipqualityscore_get($fraudResults,$element);
            if($panelTitle == "general" && $element == "request_id")
            {
                $panelValues[$element] = '<a href="https://www.ipqualityscore.com/user/view-request/'.$elementValue.'" target="_blank">'.$elementValue.'</a>';
            }else if($panelTitle == "general" && $element == "fraud_score")
            {
                if($elementValue>0)
                {
                $panelValues[$element] = $elementValue.' <img src="https://cdn.fraudlabspro.com/assets/img/scores/bar-'.$elementValue.'.png" style="width: 170px;height: 18px;">';
                }else{
                $panelValues[$element] = $elementValue;
                }
            }else{
                if(in_array($element,$booleanFields)) $elementValue = $elementValue == true ? 'Yes':'No';
                if(!empty($elementValue)) $panelValues[$element] = $elementValue;
            }
        }
        if(!empty($panelValues)) $panels[$panelTitle] = $panelValues;
    }
    $error = ipqualityscore_get($fraudResults,"success");
    if(empty($error))
    {
        $errorCode = ipqualityscore_get($fraudResults,"message");
        $panelValues["general"]["error"] = $errorCode;
    }else{
        $panelValues["general"]["error"] = "";
    }
    $errorMessage = "";
    if (!empty($panelValues["general"]["error"])) $errorMessage = $panelValues["general"]["error"];
    return ["ErrorMsg" => $errorMessage,"Panels" => $panels];
}
?>