<?php
use Illuminate\Database\Capsule\Manager as Capsule;
function maxmind_MetaData()
{
    return array("DisplayName" => "MaxMind", "SupportsRechecks" => true, "APIVersion" => "1.2");
}
function maxmind_getConfigArray()
{
    return array("Enable" => array("FriendlyName" => "Enable MaxMind", "Type" => "yesno", "Description" => "Tick to enable MaxMind Fraud Checking for Orders"), "userId" => array("FriendlyName" => "MaxMind User ID", "Type" => "text", "Size" => "30", "Description" => "Don't have an account? <a href=\"https://whasols.com/go/4/maxmind\" class=\"autoLinked\" target=\"_blank\">Click here to sign up &raquo;</a>"), "licenseKey" => array("FriendlyName" => "MaxMind License Key", "Type" => "text", "Size" => "30"), "serviceType" => array("FriendlyName" => "Service Type", "Default" => "Insights", "Type" => "dropdown", "Options" => implode(",", array("Score", "Insights", "Factors")), "Description" => "Determines the level of checks that are performed. Default is <strong>Score</strong>. <a href=\"https://whasols.com/go/3/maxmind-compare\" class=\"autoLinked\" target=\"_blank\">Learn more</a>"), "riskScore" => array("FriendlyName" => "MaxMind Fraud Risk Score", "Type" => "text", "Size" => "2", "Default" => 20, "Description" => "Higher than this value and the order will be blocked (0.01 -> 99)"), "ignoreAddressValidation" => array("FriendlyName" => "Do Not Validate Address Information", "Type" => "yesno", "Description" => "Tick to ignore warnings related to address information validation failing."), "rejectFreeEmail" => array("FriendlyName" => "Reject Free Email Service", "Type" => "yesno", "Description" => "Block orders from free email addresses such as Hotmail & Yahoo!<sup>*</sup>"), "rejectCountryMismatch" => array("FriendlyName" => "Reject Country Mismatch", "Type" => "yesno", "Description" => "Block orders where order address is different from IP Location<sup>*</sup>"), "rejectAnonymousNetwork" => array("FriendlyName" => "Reject Anonymous Networks", "Type" => "yesno", "Description" => "Block orders where the user is ordering through an anonymous network<sup>*</sup>"), "rejectHighRiskCountry" => array("FriendlyName" => "Reject High Risk Country", "Type" => "yesno", "Description" => "Block orders from high risk countries<sup>*</sup>"), "customRules" => array("FriendlyName" => "Custom Rules", "Type" => "System", "Description" => "Additional rules can be created within your MaxMind account to apply automated fraud check filtering based on rules and criteria you define.<br>For more information about custom rules, visit the <a href=\"https://whasols.com/go/5/maxmind-custom-rules\" class=\"autoLinked\" target=\"_blank\">MaxMind website</a>"), "<div class=\"pull-right\">*</div>" => array("Type" => "System", "Description" => "Only Available for Insights & Factors"));
}
function maxmind_createTable($drop = false)
{
    $table = "mod_maxmind_payment";
    $fillable = array("processor", "whasols_module");
    $getDefaultTableData = array(array("processor" => "authorizenet", "whasols_module" => "acceptjs"), array("processor" => "authorizenet", "whasols_module" => "authorize"), array("processor" => "authorizenet", "whasols_module" => "authorizecim"), array("processor" => "authorizenet", "whasols_module" => "authorizeecheck"), array("processor" => "authorizenet", "whasols_module" => "planetauthorize"), array("processor" => "bluepay", "whasols_module" => "bluepay"), array("processor" => "bluepay", "whasols_module" => "bluepayecheck"), array("processor" => "bluepay", "whasols_module" => "bluepayremote"), array("processor" => "ccavenue", "whasols_module" => "ccavenue"), array("processor" => "ccavenue", "whasols_module" => "ccavenuev2"), array("processor" => "eway", "whasols_module" => "ewaytokens"), array("processor" => "mollie", "whasols_module" => "mollieideal"), array("processor" => "moneris_solutions", "whasols_module" => "moneris"), array("processor" => "moneris_solutions", "whasols_module" => "monerisvault"), array("processor" => "skrill", "whasols_module" => "moneybookers"), array("processor" => "skrill", "whasols_module" => "skrill"), array("processor" => "optimal_payments", "whasols_module" => "optimalpayments"), array("processor" => "paypal", "whasols_module" => "payflowpro"), array("processor" => "paypal", "whasols_module" => "paypal"), array("processor" => "paypal", "whasols_module" => "paypalexpress"), array("processor" => "paypal", "whasols_module" => "paypalpaymentspro"), array("processor" => "paypal", "whasols_module" => "paypalpaymentsproref"), array("processor" => "payza", "whasols_module" => "payza"), array("processor" => "psigate", "whasols_module" => "psigate"), array("processor" => "securetrading", "whasols_module" => "securetrading"), array("processor" => "stripe", "whasols_module" => "stripe"), array("processor" => "sagepay", "whasols_module" => "sagepayrepeats"), array("processor" => "sagepay", "whasols_module" => "sagepaytokens"), array("processor" => "sagepay", "whasols_module" => "protx"), array("processor" => "sagepay", "whasols_module" => "protxvspform"), array("processor" => "usa_epay", "whasols_module" => "usaepay"), array("processor" => "worldpay", "whasols_module" => "worldpay"), array("processor" => "worldpay", "whasols_module" => "worldpayfuturepay"), array("processor" => "worldpay", "whasols_module" => "worldpayinvisible"), array("processor" => "worldpay", "whasols_module" => "worldpayinvisiblexml"));
    $schemaBuilder = Capsule::schema();
    if ($drop) {
        $schemaBuilder->dropIfExists($table);
    }
    if (!$schemaBuilder->hasTable($table)) {
        $schemaBuilder->create($table, function (\Illuminate\Database\Schema\Blueprint $table) {
            $table->increments("id");
            $table->char("processor", 128)->default("");
            $table->char("whasols_module", 128)->default("");
            $table->timestamp("created_at")->default("0000-00-00 00:00:00");
            $table->timestamp("updated_at")->default("0000-00-00 00:00:00");
            $table->unique("whasols_module");
            $table->index(array("processor", "whasols_module"), "index_modules");
        });
        Capsule::table($table)->insert($getDefaultTableData);
    }
}
function maxmind_getPaymentModule($payment_method=""){
    $Query = get_one("mod_maxmind_payment",["whasols_module"=>$payment_method],"processor");
    if(!empty($Query)) return $Query->processor;
    return "stripe";
}
function maxmind_activate(array $params = array())
{
    maxmind_createTable();
}
function maxmind_call($data,$params)
{
    $serviceType = strtolower($params["serviceType"]);
    if (!in_array($serviceType,["score", "insights", "factors"])) return ["status"=>"error","message"=>"Invalid service type: " . $serviceType];
    $accountId = $params["userId"];
    $licenseKey = $params["licenseKey"];
    //$serviceType = "insights";
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => 'https://minfraud.maxmind.com/minfraud/v2.0/'.$serviceType,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => json_encode($data),
      CURLOPT_USERPWD => "$accountId:$licenseKey",
    ));
    $response = curl_exec($curl);
    curl_close($curl);    
    logModuleCall('maxmind',"doFraudCheck", $data, $response,[]);
    if(empty($response)) return ["status"=>"error","message"=>"MaxMind API Response empty"];
    $response = json_decode($response,true);
    return $response;
}
function maxmind_get($response=[],$param = "")
{
    $value = $response;
    $keys = explode('.', $param); // Split the parameter by '.'
    // Traverse each key to reach the nested value
    foreach ($keys as $key) {
        if (isset($value[$key]))
        {
            $value = $value[$key];
        } else {
            return "";
        }
    }
    return $value;
}  
function maxmind_validateRules(array $params,array $response)
{
    $maxRiskScore = (int) $params["riskScore"];
    if (0 < $maxRiskScore && $maxRiskScore < $response["risk_score"]) return \Lang::trans("maxmind_highfraudriskscore");
    if (maxmind_get($response,"disposition") && maxmind_get($response,"disposition.action") != "accept") return \Lang::trans("maxmind.manualReview");
    if ($params["rejectFreeEmail"] == "yes" && maxmind_get($response,"email.is_free")) return \Lang::trans("maxmind_rejectemail");
    if ($params["rejectCountryMismatch"] == "yes" && !maxmind_get($response,"billing_address.is_in_ip_country")) return \Lang::trans("maxmind_countrymismatch");
    if ($params["rejectAnonymousNetwork"] == "yes" && maxmind_get($response,"ip_address.is_anonymous")) return \Lang::trans("maxmind_anonproxy");
    if ($params["rejectHighRiskCountry"] == "yes" && maxmind_get($response,"ip_address.country.is_high_risk")) return \Lang::trans("maxmind_highriskcountry");
    if ($params["ignoreAddressValidation"] != "yes")
    {
        $warnings = maxmind_get($response,"warnings");
        if (is_array($warnings))
        {
            $warningCodes = array_column($warnings, 'code');
            if (!empty($params["clientsdetails"]["city"]) && in_array("BILLING_CITY_NOT_FOUND",$warningCodes)) return \Lang::trans("maxmind_addressinvalid");
            if (!empty($params["clientsdetails"]["postcode"]) && in_array("BILLING_POSTAL_NOT_FOUND",$warningCodes)) return \Lang::trans("maxmind_addressinvalid");
            if (!empty($params["clientsdetails"]["country"]) && in_array("BILLING_COUNTRY_NOT_FOUND",$warningCodes)) return \Lang::trans("maxmind_addressinvalid");
            if (empty($params["clientsdetails"]["country"]) && in_array("BILLING_COUNTRY_MISSING",$warningCodes)) return \Lang::trans("maxmind_addressinvalid");
            if (in_array("IP_ADDRESS_NOT_FOUND",$warningCodes)) return \Lang::trans("maxmind_invalidip");
        }
    }
    return "";
}
function maxmind_doFraudCheck(array $params, $checkOnly = false)
{
    $emailDomain = explode("@", $params["clientsdetails"]["email"], 2);
    $emailDomain = isset($emailDomain[1]) ? $emailDomain[1] : "";
    $billing = array();
    $billing["first_name"] = $params["clientsdetails"]["firstname"];
    $billing["last_name"] = $params["clientsdetails"]["lastname"];
    if ($params["clientsdetails"]["companyname"]) {
        $billing["company"] = $params["clientsdetails"]["companyname"];
    }
    if ($params["clientsdetails"]["address1"]) {
        $billing["address"] = $params["clientsdetails"]["address1"];
    }
    if ($params["clientsdetails"]["city"]) {
        $billing["city"] = $params["clientsdetails"]["city"];
    }
    if ($params["clientsdetails"]["state"]) {
        $billing["region"] = $params["clientsdetails"]["state"];
    }
    if ($params["clientsdetails"]["postcode"]) {
        $billing["postal"] = $params["clientsdetails"]["postcode"];
    }
    $billing["country"] = $params["clientsdetails"]["country"];
    $phoneCountryCode = $params["clientsdetails"]["phonecc"];
    $phoneNumber = $params["clientsdetails"]["phonenumber"];
    if ($phoneNumber) {
        $billing["phone_number"] = $phoneNumber;
        if ($phoneCountryCode) {
            $billing["phone_country_code"] = $phoneCountryCode;
        }
    }
    $currencyCode = $params["clientsdetails"]["currencycode"];
    $request = array("device" => array("ip_address" => $params["ip"]), "event" => array("transaction_id" => $params["order"]["order_number"], "type" => "purchase"), "account" => array("user_id" => $params["clientsdetails"]["userid"], "username_md5" => md5($params["clientsdetails"]["userid"])), "email" => array("address" => $params["clientsdetails"]["email"], "domain" => $emailDomain), "billing" => $billing, "payment" => array("processor" => maxmind_getPaymentModule($params["order"]["payment_method"])), "order" => array("amount" => $params["order"]["amount"], "currency" => $currencyCode, "discount_code" => $params["order"]["promo_code"]));
    $cardNumber = isset($params["creditcard"]["cardnum"]) ? $params["creditcard"]["cardnum"]:'';
    $last_4_digits = isset($params["creditcard"]["last_four"]) ? $params["creditcard"]["last_four"]:'';
    if ($cardNumber) {
        $cardDetails = array("issuer_id_number" => substr($cardNumber, 0, 6), "last_4_digits" =>$last_4_digits, "token" => $params["clientsdetails"]["userid"] . generateFriendlyPassword(16));
        $request["credit_card"] = $cardDetails;
    }
    if (array_key_exists("sessionId", $params) && $params["sessionId"]) {
        $request["device"]["session_id"] = $params["sessionId"];
    }
    if (array_key_exists("userAgent", $params) && $params["userAgent"]) {
        $request["device"]["user_agent"] = $params["userAgent"];
    }
    if (array_key_exists("acceptLanguage", $params) && $params["acceptLanguage"]) {
        $request["device"]["accept_language"] = $params["acceptLanguage"];
    }
    $response = maxmind_call($request,$params);
    if(array_key_exists("status",$response))
    {
        if($response["status"] == "error") return $response;
    }
    $errorCode = maxmind_get($response,"code");
    $error = maxmind_get($response,"error");
    if(!empty($error))
    {
        logActivity("MaxMind Fraud Check - Error Occurred: " . $errorCode . " - " . $error);
        return ["status"=>"error","message"=>$error,"data"=>$response];
    }
    $IsValid = maxmind_validateRules($params, $response);
    if (!$checkOnly && !empty($IsValid)) return ["status"=>"error","message"=>$IsValid,"data"=>$response,"fraud"=>true];
    return ["status"=>"success","message"=>"Fraud Checks Processed successfully","data"=>$response];
}
function maxmind_doReportConfigArray(array $params)
{
    return [
        "tag" => ["FriendlyName" => "Status", "Type" => "dropdown", "Default" =>"", "Description" => "Select Maxmind Status to mark this Fraudcheck as not_fraud, suspected_fraud, spam_or_abuse, or chargeback","Options"=>["not_fraud"=>"Not Fraud","suspected_fraud"=>"Suspected as Fraud","spam_or_abuse"=>"Spam or Abuse","chargeback"=>"Chargeback"]],
        "notes" => ["FriendlyName" => "Note", "Type" => "textarea", "Default" =>"", "Description" => "Enter Note for Report Request"],
        "transaction_id" => ["FriendlyName" => "Payment Transaction ID", "Type" => "text", "Default" =>"", "Description" => "Enter Transaction ID (Optional)"]
    ];
}
function maxmind_doReport(array $params)
{
    $accountId = $params["userId"];
    $licenseKey = $params["licenseKey"];
    $tag = $params["tag"];
    $notes = $params["notes"];
    $transaction_id = $params["transaction_id"];
    if(empty($licenseKey)) return ["status"=>"error","message"=>"Maxmind API Key empty"];
    if(empty($accountId)) return ["status"=>"error","message"=>"Maxmind Account ID empty"];
    if(empty($tag)) return ["status"=>"error","message"=>"Please Select Status"];
    if(empty($notes)) return ["status"=>"error","message"=>"Please Enter Note"];
    $data = [];
    $data["ip_address"] = $params["ip"];
    if(!empty($transaction_id))
    {
        $data["transaction_id"] = $transaction_id;
    }else{
    $data["transaction_id"] = $params["order"]["invoice_id"]>0 ? $params["order"]["invoice_id"]:$params["order"]["order_number"];        
    }
    $data["tag"] = $tag;
    $data["notes"] = $notes;
    
    $data["minfraud_id"] = maxmind_get($params["data"],"id");
    //$data["maxmind_id"] = $data["minfraud_id"];
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => 'https://minfraud.maxmind.com/minfraud/v2.0/transactions/report',
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT => 10,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => json_encode($data),
      CURLOPT_USERPWD => "$accountId:$licenseKey",
    ));
    $response = curl_exec($curl);
    curl_close($curl);    
    logModuleCall('maxmind',__FUNCTION__, $data, $response,[]);
    if(empty($response)) return ["status"=>"success","message"=>"Maxmind Reported successfully"];
    $response = json_decode($response,true);
    return ["status"=>"error","message"=>_s('error',$response,"")];
}
function maxmind_processResultsForDisplay(array $params)
{
    $fraudResults = $params["data"];
    $accountId = $params["userId"];
    $fieldMapping = array("ip" => array("ip_address.city.names.en", "ip_address.city.confidence", "ip_address.subdivisions.0.iso_code", "ip_address.subdivisions.0.names.en", "ip_address.subdivisions.0.confidence", "ip_address.continent.code", "ip_address.continent.names.en", "ip_address.country.confidence", "ip_address.country.iso_code", "ip_address.country.names.en", "ip_address.location.accuracy_radius", "ip_address.location.latitude", "ip_address.location.longitude", "ip_address.location.time_zone", "ip_address.traits.user_type", "ip_address.traits.autonomous_system_number", "ip_address.traits.autonomous_system_organization", "ip_address.traits.isp", "ip_address.traits.is_anonymous", "ip_address.traits.is_anonymous_vpn"), "subscores.desc" => array("subscores.billing_address", "subscores.billing_address_distance_to_ip_location", "subscores.browser", "subscores.country", "subscores.country_mismatch", "subscores.email_address", "subscores.email_domain", "subscores.order_amount", "subscores.phone_number", "subscores.time_of_day"), "billing" => array("billing_address.latitude", "billing_address.longitude", "billing_address.distance_to_ip_location", "billing_address.is_in_ip_country"), "credit_card.desc" => array("credit_card.issuer.name", "credit_card.issuer.phone_number", "credit_card.brand", "credit_card.country", "credit_card.is_issued_in_billing_address_country", "credit_card.is_prepaid", "credit_card.is_virtual", "credit_card.type"), "email.desc" => array("email.first_seen", "email.is_free", "email.is_high_risk"), "general" => array("id","risk_score", "funds_remaining", "queries_remaining"));

    $booleanFields = array("email.is_free", "email.is_high_risk", "billing_address.is_in_ip_country", "ip_address.traits.is_anonymous", "ip_address.traits.is_anonymous_vpn", "credit_card.is_issued_in_billing_address_country", "credit_card.is_prepaid", "credit_card.is_virtual");

    $panels = array();
    foreach ($fieldMapping as $panelTitle => $panelElements)
    {
        $panelValues = array();
        foreach ($panelElements as $element)
        {
            $elementValue = maxmind_get($fraudResults,$element);
            if($panelTitle == "general" && $element == "id")
            {
                $panelValues[$element] = '<a href="https://www.maxmind.com/en/accounts/'.$accountId.'/minfraud-interactive/transactions/'.$elementValue.'" target="_blank">'.$elementValue.'</a>';
            }else if($panelTitle == "general" && $element == "risk_score")
            {
                $panelValues[$element] = $elementValue.' <img src="https://cdn.fraudlabspro.com/assets/img/scores/bar-'.$elementValue.'.png" style="width: 170px;height: 18px;">';
            }else{
                if(in_array($element,$booleanFields)) $elementValue = $elementValue == true ? 'Yes':'No';
                if(!empty($elementValue)) $panelValues[$element] = $elementValue;
            }
        }
        if(!empty($panelValues)) $panels[$panelTitle] = $panelValues;
    }
    $error = maxmind_get($fraudResults,"error");
    if(!empty($error))
    {
        $errorCode = maxmind_get($fraudResults,"code");
        $panelValues["general"]["error"] = $errorCode.":".$error;
    }else{
        $panelValues["general"]["error"] = "";
    }
    $errorMessage = "";
    if (!empty($panelValues["general"]["error"])) $errorMessage = $panelValues["general"]["error"];
    return ["ErrorMsg" => $errorMessage,"Panels" => $panels];
}
?>