<?php
/**
 * WHMSOLS EasyPaisa Module
 * This is the EasyPaisa (EasyPaisa) Payment Module
 * @author Khawar Shahzad (hostarina.com)
 * @license MIT
 */




/**
 * EasyPaisa metaData
 *
 * Values returned here are used to determine module related capabilities and
 * settings.
 *
 * @see http://docs.whmsols.com/Gateway_Module_Meta_Data_Parameters
 *
 * @return array
 */
function easypaisa_info()
{
    return array(
        'DisplayName' => 'EasyPay',
        'APIVersion' => '1.1', // Use API Version 1.1
        'DisableLocalCredtCardInput' => true,
        'TokenisedStorage' => false,
    );
}

/**
 * Define gateway configuration options.
 *
 * The fields you define here determine the configuration options that are
 * presented to administrator users when activating and configuring your
 * payment gateway module for use.
 *
 * Supported field types include:
 * * text
 * * password
 * * yesno
 * * dropdown
 * * radio
 * * textarea
 *
 * Examples of each field type and their possible configuration parameters are
 * provided in the sample function below.
 *
 * @return array
 */
function easypaisa_config()
{
    return array(
        // the friendly display name for a payment gateway should be
        // defined here for backwards compatibility
        'Name' => array(
            'Type' => 'System',
            'Value' => 'easypaisa',
        ),
        // a text field type allows for single line text input
        'Title' => array(
            'Name' => 'Display Name',
            'Type' => 'text',
            'Size' => '50',
            'Default' => 'Easy Paisa',
            'Description' => '',
        ),
        // a text field type allows for single line text input
        'STORE_ID' => array(
            'Name' => 'Store ID',
            'Type' => 'text',
            'Size' => '4',
            'Default' => '',
            'Description' => 'Enter your Store ID here',
        ),
        'HASH_KEY' => array(
            'Name' => 'Secret Hash Key',
            'Type' => 'text',
            'Size' => '32',
            'Default' => '',
            'Description' => 'Enter your Easy Paisa Merchant Secret Hash Key ',
        ),
        'PAYMENT_METHOD' => array(
            'Name' => 'Payment Method',
            'Type' => 'text',
            'Default' => '',
            'Description' => 'Empty for all payment methods',
        ),
        'LIVE' => array(
            'Name' => 'Test Mode',
            'Type' => 'yesno',
            'Default' => 'no',
            'Description' => 'Tick to enable test mode (Optional)',
        )
    );
}





/**
 * Generation of Payment link.
 *
 * @return string
 */

function easypaisa_link($arg)
{


    // Gateway Configuration Parameters
    $eTitle = $arg['gateway']['title'];
    $eStoreId = $arg['gateway']['STORE_ID'];
    $hashKey = $arg['gateway']['HASH_KEY'];
    $eOrderPrefix = $arg['gateway']['ORDER_PREFIX'];
    $ePaymentMethod = $arg['gateway']['PAYMENT_METHOD'];
    $eLive = $arg['gateway']['LIVE'];
    $gateway_id = $arg['gateway']['id'];
    $eAutoRedirect = 1;


    // Invoice Parameters
    $invoiceid = $arg['invoice']['invoiceid'];
    $description = $arg['invoice']['description'];
    $amount = $arg['invoice']['amount'];
    $currencyCode = $arg['invoice']['currency'];


    //Client Parameters
    $firstname = $arg['client']['firstname'];
    $lastname = $arg['client']['lastname'];
    $custEmail = $arg['client']['email'];
    $address1 = $arg['client']['address1'];
    $address2 = $arg['client']['address2'];
    $city = $arg['client']['city'];
    $state = $arg['client']['state'];
    $postcode = $arg['client']['postcode'];
    $country = $arg['client']['country'];
    $custCell = $arg['client']['phonenumber'];


    // System Parameters
    $companyName = $arg['system']['companyname'];
    $systemUrl = $arg['system']['systemurl'];            // System URL
    $siteurl = $arg['system']['siteurl'];            // System URL




    $callback = $systemUrl.'/client/paytoinvoice/'.$gateway_id.'/'.$invoiceid;


    $currentDate = new DateTime();
    $currentDate->modify('+ 10 day');
    $expiryDate = $currentDate->format('Ymd His'); // Conversion for expiry date&time



    $easypayIndexPage = '';
	if ($eLive == 'no') {
		$easypayIndexPage = 'https://easypaystg.easypaisa.com.pk/easypay/Index.jsf';
	} else {
		$easypayIndexPage = 'https://easypay.easypaisa.com.pk/easypay/Index.jsf';
	}
	
	

    $currentDate = new DateTime();
    $currentDate->modify('+ 10 day');
    $expiryDate = $currentDate->format('Ymd His');
	

	$hashRequest = '';
	if(strlen($hashKey) > 0 && (strlen($hashKey) == 16 || strlen($hashKey) == 24 || strlen($hashKey) == 32 )) {
		// Create Parameter map
		$paramMap = array();
		$paramMap['amount']  = $amount ;
		$paramMap['autoRedirect']  = $eAutoRedirect;
		if($custEmail != null && $custEmail != '') $paramMap['emailAddr']  = $custEmail;
		if($expiryDate != null && $expiryDate != '') $paramMap['expiryDate'] = $expiryDate;
		if($custCell != null && $custCell != '') $paramMap['mobileNum'] = $custCell;
		$paramMap['orderRefNum']  = $invoiceid;
		if($ePaymentMethod != null && $ePaymentMethod != '') $paramMap['paymentMethod']  = $ePaymentMethod;
		$paramMap['postBackURL'] = $merchantConfirmPage;
		$paramMap['storeId']  = $storeId ;
		//Creating string to be encoded
		$mapString = '';
		foreach ($paramMap as $key => $val) {
			$mapString .=  $key.'='.$val.'&';
		}
		$mapString  = substr($mapString , 0, -1);
		
		// Encrypting mapString
		function pkcs5_pad($text, $blocksize) {
			
			$pad = $blocksize - (strlen($text) % $blocksize);
			return $text . str_repeat(chr($pad), $pad);
			
		}

		$alg = MCRYPT_RIJNDAEL_128; // AES
		$mode = MCRYPT_MODE_ECB; // ECB

		$iv_size = mcrypt_get_iv_size($alg, $mode);
		$block_size = mcrypt_get_block_size($alg, $mode);
		$iv = mcrypt_create_iv($iv_size, MCRYPT_DEV_URANDOM);	

		$mapString = pkcs5_pad($mapString, $block_size);
		$crypttext = mcrypt_encrypt($alg, $hashKey, $mapString, $mode, $iv);
		$hashRequest = base64_encode($crypttext);
	}
	

$htmlOutput = '';
    $amount = number_format($amount,1,'.', ''); //Converting amount to one-decimal point


$htmlOutput .= '<form name="easypayform" action="'.$easypayIndexPage.'" method="POST">
<! -- Store Id Provided by Easypay-->
<input name="storeId" value="'.$eStoreId.'" type="hidden"/>
<! -- Amount of Transaction from merchant’s website -->
<input name="amount" value="'.$amount.'" type="hidden"/>
<! – Post back URL from merchant’s website -- >
<input name="postBackURL" value="'.$callback.'" type="hidden"/>
<! – Order Reference Number from merchant’s website -- >
<input name="orderRefNum" value="'.$invoiceid.'" type="hidden"/>
<! – Expiry Date from merchant’s website (Optional) -- >
<input type ="hidden" name="expiryDate" value="'.$expiryDate .'">
<! – Merchant Hash Value (Optional) -- >
<input type ="hidden" name="merchantHashedReq" value="'.$hashRequest .'">
<! – If Merchant wants to redirect to Merchant website after payment completion (Optional) -- >
<input type ="hidden" name="autoRedirect" value="'.$eAutoRedirect.'">
<! – If merchant wants to post specific Payment Method (Optional) -- >
<input type ="hidden" name="paymentMethod" value="'.$ePaymentMethod.'">
<! – If merchant wants to post specific Payment Method (Optional) -- >
<input type ="hidden" name="emailAddr" value="'.$custEmail.'">
<! – If merchant wants to post specific Payment Method (Optionl) -- >
<input type ="hidden" name="mobileNum" value="'.$custCell.'">

<br><button type="submit" class="btn btn-success"><i class="glyphicon glyphicon-credit-card"></i> Pay Now By '.$eTitle.'</button>
</form>';
    return $htmlOutput;
}



    function easypaisa_callback($arg)
    {
        // Gateway Configuration Parameters
        $eStoreId = $arg['gateway']['STORE_ID'];
        $gateway_id = $arg['gateway']['id'];
        $invoiceid = $arg['invoice']['invoiceid'];
        $eLive = $arg['gateway']['LIVE'];
    
            $easypayConfirmPage = '';
            if ($eLive == 'no') {
            	$easypayConfirmPage = 'https://easypaystg.easypaisa.com.pk/easypay/Confirm.jsf';
            } else {
            	$easypayConfirmPage = 'https://easypay.easypaisa.com.pk/easypay/Confirm.jsf';
            }
    
    
            $htmlOutput = '';
            $gateway_variables =  GetPaymentGatewayVariables($gateway_id);
            
            $isValid =  DoInvoiceValidate($invoiceid);
            if(!$isValid)
            {
            $htmlOutput .='<div class="alert alert-danger">Invoice Id is Invailed.....</div>'; 
            }
            else
            {
    
                // Invoice Parameters
                $currency = $arg['invoice']['currency'];
                $currencyCode = $arg['invoice']['currencyCode'];
                $payment_method = $arg['invoice']['payment_method'];
                $user_id = $arg['client']['id'];
    
    
                $auth_token = isset($_GET['auth_token']) ? $_GET['auth_token']:'';
                $callback = $systemUrl.'/client/paytoinvoice/'.$gateway_id.'/'.$invoiceid;
                
                if(!empty($auth_token))
                {
                    $htmlOutput .= '<form name="easypayconfirmform" action="'.$easypayConfirmPage.'" method="POST">
                        <input name="auth_token" value="'.$auth_token.'" type="hidden"/>
                        <input name="postBackURL" value="'.$callback.'" type="hidden"/>	
                    </form>
                    <script data-cfasync="false" type="text/javascript">
                        document.easypayconfirmform.submit();
                    </script>';
                }
                elseif (isset($_GET['paymentToken']))
                {
                    $invoiceId = $_GET['orderRefNumber'];
                    $paymentToken = $_GET['paymentToken'];
                    $htmlOutput .= '<script language="javascript"> setTimeout( function() { window.location.replace("'.$systemUrl.'/client/view-invoice/'.$invoiceId.'"); }, 5000);</script>
                    <div class="content">
                      <h2>Token Issued ... </h2>
                    </div>
                    <div class="content">
                      <h1>Token # '.$paymentToken.'</h1>
                      <h2>EasyPaisa</h2>
                      <p>Payments</p>
                    </div>';
                }
                elseif(isset($_GET['orderRefNumber']))
                {
                    $invoiceId = $_GET['orderRefNumber'];
                    if($_GET['success']=="true"){
                        $status='paymentsuccess=1';
                    }else{
                        $status='paymentfailed=1';
                    }
                    header("Location: /client/view-invoice/'.$invoiceId.'&".$status); 
                }
                else{
        
                }
    
            if (isset($_GET['url']))
            {
                // Matching if the URL == easypay.easypaisa.com.pk
                $web = '/^https:\/\/easypay\.easypaisa\.com\.pk\//i';
                $url = $_GET['url'];
                if (preg_match($web,$url))
                {
                    // If the URL was easypay's original url then we should cURL it to extract Transaction Result.
                    // Using cURL to extract EasyPay IPN JSON data sent to the Callback.
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_URL,$url); // using cURL @ $url to get transaction data.
                    $result=curl_exec($ch);
                    curl_close($ch);
                    //Decoding JSON results into PHP object
                    $obj = json_decode($result);
                    $status = $obj->transaction_status;          // Payment Status
                    $storeId = $obj->store_id;                   // Store ID
                    $invoiceId = $obj->order_id;                 // Invoice ID
                    $transactionId = $obj->transaction_id;       // Transaction ID
                    $paymentAmount = $obj->transaction_amount;   // Transaction Amount
                    $paymentMethod = $obj->payment_method;       // Payment Method
                
                    // switching to see what payment method was used.
                    switch($paymentMethod){
                        case "OTC":
                            $paymentMethod = "EasyPaisa Shop";
                            break;
                        case "CC":
                            $paymentMethod = "Credit Card";
                            break;
                        case "MA":
                            $paymentMethod = "EasyPaisa Mobile Account";
                            break;
                        default:
                            $paymentMethod ;
                    }
                 }
                    /**
                     * Validate Callback Invoice ID.
                     *
                     * Checks invoice ID is a valid invoice number. Note it will count an
                     * invoice in any status as valid.
                     *
                     * Performs a die upon encountering an invalid Invoice ID.
                     *
                     * Returns a normalised invoice ID.
                     */
    
                 $isValid =  DoInvoiceValidate($invoiceId);
                if(!$isValid)
                {
                   $htmlOutput .='<div class="alert alert-danger">Invoice Id is Invailed.....</div>'; 
                }
                else
                {
                    /**
                     * Check Callback Transaction ID.
                     *
                     * Performs a check for any existing transactions with the same given
                     * transaction number.
                     *
                     * Performs a die upon encountering a duplicate.
                     */
                   $isValid =  DoTransactionValidate($invoiceId,$transactionId);
                    if($isValid)
                    {
                        $htmlOutput .='<div class="alert alert-danger">Transaction No already exist.....</div>';
                    }
                    else
                    {
                        /**
                         * Log Transaction.
                         *
                         * Add an entry to the Gateway Log for debugging purposes.
                         *
                         * The debug data can be a string or an array. In the case of an
                         * array it will be
                         *
                         * @param string $gatewayName        Display label
                         * @param string|array $debugData    Data to log
                         * @param string $transactionStatus  Status
                         */
                        AddGatewayLogs(array('invoice_id'=>$invoiceId,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$result));
                        
                        if ($status=="PAID" && $storeId == $eStoreId)
                        {
                            /**
                             * Add Invoice Payment.
                             *
                             * Applies a payment transaction entry to the given invoice ID.
                             *
                             * @param int $invoiceId         Invoice ID
                             * @param string $transactionId  Transaction ID
                             * @param float $paymentAmount   Amount paid (defaults to full balance)
                             * @param float $paymentFee      Payment fee (optional)
                             * @param string $gatewayModule  Gateway module name
                             */
                             
                            $fee = 0;
                            $payment_added = AddInvoicePayment(array('invoice_id'=>$invoiceId,'currency'=>$currency,'user_id'=>$user_id,'payment_method'=>$payment_method,'amount'=>$paymentAmount,'trno'=>$transactionId,'fee'=>$fee,'notification'=>'YES'));
                            if($payment_added['status'] == 'success')
                            {
                            header("Location: /client/view-invoice/".$invoiceid);                   
                            }else{
                            $htmlOutput .='<div class="alert alert-danger">Payment Confirmed but Payment entry error...'.$payment_added['message'].'</div>';
                            }
                        }
                    }
                }
            }
        }  
        return $htmlOutput;
    }




?>
