<?php
if (!defined("WHMSOLS")) exit("This file cannot be accessed directly");
use \Stripe\Stripe;
use \Stripe\Charge;
use \Stripe\Exception\CardException;
use \Stripe\Exception\RateLimitException;
use \Stripe\Exception\InvalidRequestException;
use \Stripe\Exception\AuthenticationException;
use \Stripe\Exception\ApiConnectionException;
use \Stripe\Exception\ApiErrorException;
function stripe_info()
{
    return array(
        'DisplayName' => 'Stripe', // display name for client
        'APIVersion' => '1.2', 
        'DisableLocalCredtCardInput' => false,
        'TokenisedStorage' => false,
    );
}
function stripe_config()
{
    return [
        "Name" => ["Type" => "System", "Value" => "Stripe"],
        "Publishablekey" => ["Name" => "Publishable API Key", "Type" => "text", "Size" => "500","Description"=>"Your publishable API key identifies your website to Stripe during communications. This can be obtained from <a href=\"https://dashboard.stripe.com/account/apikeys\" target=\"_blank\">here</a>"],
        "Secretkey" => ["Name" => "Secret API Key", "Type" => "text", "Size" => "500","Description"=>"Your secret API Key ensures only communications from Stripe are validated. This can be obtained from <a href=\"https://dashboard.stripe.com/account/apikeys\" target=\"_blank\">here</a>"],
        "WebHookEndpointSecret" => ["Name" => "WebHook Endpoint Secret", "Type" => "text", "Size" => "500","Description"=>"Automatically generated web-hook secret key for Live web-hooks"],
        "WebHookEndpointSecretTest" => ["Name" => "WebHook Endpoint Secret (Test/Sandbox)", "Type" => "text", "Size" => "500","Description"=>"Automatically generated web-hook secret key for Sandbox web-hooks"],
        "noAccount" => ["Type" => "system", "Description" => "<div class=\"alert alert-info\" style=\"margin-bottom: 0;\">New to Skrill? <a href=\"https://whmsols.com/go/skrill-signup\" class=\"autoLinked alert-link\" target=\"_blank\">Create an account</a></div>"]
        ];
}
function stripe_link(array $params)
{
    $Publishablekey = $params["Publishablekey"];
    $Title = $params["Title"];
    $Gateway_id = $params["gateway"]["id"];
    $invoiceid = $params["invoiceid"];
    $userid = $params["userid"];
    $amount = $params["amount"]*100;
    if($amount<=0) return "<div class=\"alert alert-danger\">The payable amount must be greater than or equal to the minimum payable amount 0.1.<br><b>Reasons:</b><br>Maybe Invoice already paid or Invoice Status not updated.</div>";
     $code = '<form id="frmStripePayment" method="post" action="" onsubmit="ws.gateway_actions(this,false); return false;">
                <input type="hidden" name="act" value="CardForm" />
                <input type="hidden" name="id" value="'.$invoiceid.'" />
                <input type="hidden" name="user_id" value="'.$userid.'" />
                <input type="hidden" name="gateway_id" value="'.$Gateway_id.'" />
                <button type="submit" class="btn btn-success w-100" />'.$Title.'</button>
            </form>';
    return $code;
}
function stripe_CardForm(array $params)
{
    $Publishablekey = $params["Publishablekey"];
    $full_name = strtoupper($params["clientdetails"]["fullName"]);
    $Card = _s("card",$params,[]);
    $year = date('m/y');
    $time = time();
    $js = '<script>var pk_key = "'.$Publishablekey.'";</script>
    <script src="/modules/gateways/stripe/assets/all.js?t='.time().'"></script>';
    $code = [
        "stripeToken"=>["Name"=>"Stripe Token","Id"=>"ccc-stripeToken","Class"=>"","Type"=>"hidden","Placeholder"=>"","Value"=>"","Cols"=>"12"],
        "cardnumber"=>["Name"=>"Card Number","Title"=>"Credit or debit card number","Id"=>"cc-cardnumber","Class"=>"form-control","Type"=>"Div","Placeholder"=>"","Value"=>"","Cols"=>"12"],
        "exp_date"=>["Name"=>"Expiration (mm/yy)","Title"=>"Enter expiry date in MM/YY format","Id"=>"cc-exp-date","Class"=>"form-control","Type"=>"Div","Placeholder"=>"","Value"=>"","Cols"=>"6"],
        "cvc_cvv"=>["Name"=>"CVV/CVC2","Title"=>"Credit or debit card CVC/CVV","Id"=>"cc-cvc","Class"=>"form-control","Type"=>"Div","Placeholder"=>"","Value"=>"","Cols"=>"6"],
        "ccdescription"=>["Name"=>"Card Description","Title"=>"Enter a name for this card (Optional)","Id"=>"cc-description","Class"=>"form-control","Type"=>"Text","Placeholder"=>"","Value"=>$full_name,"Cols"=>"12"],
        "js"=>["Type"=>"Content","Value"=>$js],
        "custom_action"=>["Type"=>"Content","Value"=>"post_checkout"],
        ];
    return ["status"=>"success","message"=>"","content"=>$code];
}
function _stripe_formatValue($value)
{
    return $value !== "" ? $value : NULL;
}
function _stripe_formatAmount($amount, $currencyCode)
{
    $currenciesWithoutDecimals = ["BIF", "CLP", "DJF", "GNF", "JPY", "KMF", "KRW", "MGA", "PYG", "RWF", "VND", "VUV", "XAF", "XOF", "XPF"];
    $currencyCode = strtoupper($currencyCode);
    $isNoDecimalCurrency = in_array($currencyCode, $currenciesWithoutDecimals);
    $amount = str_replace([",","."], "", $amount);
    if ($isNoDecimalCurrency) $amount = round($amount / 100);
    return $amount;
}
function stripe_objToArray($arg=[])
{
    return _jd(_je($arg),true);
}
function stripe_get_intents($params=[],$token="")
{
    $SecretKey = $params["Secretkey"];
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        $intent = $stripe->paymentIntents->retrieve($token, []);
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($intent)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_create_intent($params=[],$customer_id="",$method_id="",$additional=[])
{
    if(empty($customer_id)) return ["status"=>"error","message"=>"Stripe Customer ID is empty"];
    if(empty($method_id)) return ["status"=>"error","message"=>"Stripe Payment Method ID is empty"];
    $SecretKey = $params["Secretkey"];
    $invoice_no = $params["invoice_no"];
    $description = "Invoice No:".$invoice_no;  
    $amount = $params["amount"];
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        $data = [
                    "amount" => $amount*100,
                    "currency" => strtolower($params["currency"]),
                    "customer" =>$customer_id,
                    "payment_method_types" =>["card"],
                    "payment_method" =>$method_id,
                    "description" =>$description,
                    "metadata" =>["id"=>$params["invoiceid"],"invoiceNumber"=>$invoice_no],
                    "statement_descriptor_suffix"=>$description
                ];
        if (!empty($additional)) $data = array_merge($data, $additional);
        $res = $stripe->paymentIntents->create($data);
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($res)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_create_setupintent($params=[],$customer_id="",$method_id="",$additional=[])
{
    if(empty($customer_id)) return ["status"=>"error","message"=>"Stripe Customer ID is empty"];
    if(empty($method_id)) return ["status"=>"error","message"=>"Stripe Payment Method ID is empty"];
    $SecretKey = $params["Secretkey"];

    $invoiceid = isset($params["invoiceid"]) ? $params["invoiceid"]:0;
    if($invoiceid == 0)
    {
        $invoiceid = $invoice_no = isset($params["userid"]) ? $params["userid"]:0;
        $description = "Customer Id:".$invoice_no;  
    }else{
        $invoice_no = isset($params["invoice_no"]) ? $params["invoice_no"]:0;
        $description = "Invoice No:".$invoice_no;  
    }
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        $data = [
                    "customer" =>$customer_id,
                    "payment_method_types" =>["card"],
                    "payment_method" =>$method_id,
                    "description" =>$description,
                    "metadata" =>["id"=>$invoiceid,"invoiceNumber"=>$invoice_no],
                ];
        if (!empty($additional)) $data = array_merge($data, $additional);
        $res = $stripe->setupIntents->create($data);
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($res)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_get_setupIntents($params=[],$token="")
{
    $SecretKey = $params["Secretkey"];
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        $intent = $stripe->setupIntents->retrieve($token, []);
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($intent)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_detach_payment_method($params=[],$token="")
{
    if(substr($token, 0, 2) != "pm") return ["status"=>"error","message"=>"Stripe Payment method ID is not valid. ID is :".$token];
    $stripeSecretKey = $params["Secretkey"];
    $message = "";
    try
    {
        $stripe = new \Stripe\StripeClient($stripeSecretKey);
        $res = $stripe->paymentMethods->detach($token, []);
        return ["status"=>"success","message"=>""];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}

function stripe_get_payment_method($params=[],$token="")
{
    if (substr($token, 0, 2) != "pm") return ["status"=>"error","message"=>"Stripe Payment method is not valid. ID is :".$token];
    $SecretKey = $params["Secretkey"];
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        $intent = $stripe->paymentMethods->retrieve($token, []);
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($intent)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_getCustomer($params=[],$customer_id="")
{
    if(empty($customer_id)) return ["status"=>"error","message"=>"Stripe Customer ID is empty"];
    $stripeSecretKey = $params["Secretkey"];
    $message = "";
    try
    {
        $stripe = new \Stripe\StripeClient($stripeSecretKey);
        $res = $stripe->customers->retrieve($customer_id, []);    
        return ["status"=>"success","message"=>"","data"=>stripe_objToArray($res)];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_addCustomer($params=[],$useToken=true,$method_id="") 
{
    $stripeToken = isset($params["custom"]["stripeToken"]) ? $params["custom"]["stripeToken"]:"";
    $stripeSecretKey = $params["Secretkey"];
    $client = $params["clientdetails"];
    $message = "";
    try
    {
        $fullname = $client["fullName"];
        $email = $client["email"];
        $stripe = new \Stripe\StripeClient($stripeSecretKey);
        $data = [
          'description'=>"Customer for ".$fullname." (".$email.")",
          'name' => $fullname,
          'email' => $email,
          'phone' => $client["phonenumber"],
          'address' =>[
              "city"=>$client["city"],
              "country"=>$client["country"],
              "line1"=>$client["address1"],
              "line2"=>"",
              "postal_code"=>$client["postcode"],
              "state"=>$client["state"]
              ],
           'metadata' =>["id" => $client["userid"], "fullName" =>$fullname,"email"=>$client["email"]]
        ];
        
        if(!empty($stripeToken) && $useToken)
        {
            $data["source"] = $stripeToken;
        }else{
            if(!empty($method_id)) $data["payment_method"] = $method_id;
        }
        $res = $stripe->customers->create($data);
        return ["status"=>"success","id"=>$res->id];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_payment_method_create($params=[],$token="")
{
    $SecretKey = $params["Secretkey"];
    $message="";
    $firstname = $params["clientdetails"]["firstname"];
    $lastname = $params["clientdetails"]["lastname"];
    $email = $params["clientdetails"]["email"];
    $phonenumber = $params["clientdetails"]["phonenumber"];
    $city = $params["clientdetails"]["city"];
    $country = $params["clientdetails"]["country"];
    $address1 = $params["clientdetails"]["address1"];
    $address2 = $params["clientdetails"]["address2"];
    $postcode = $params["clientdetails"]["postcode"];
    $state = $params["clientdetails"]["state"];
    $full_name = strtoupper($firstname." ".$lastname);
    
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        if (substr($token, 0, 3) == "tok")
        {
            $intent = $stripe->paymentMethods->create([
              'type' => 'card',
              'card' => [
                'token' =>$token,
              ],
              'billing_details' => [
                'name' =>$full_name,
                'email' =>$email,
                'phone' =>$phonenumber,
                'address' =>[
                    "city"=>$city,
                    "country"=>$country,
                    "line1"=>$address1,
                    "line2"=>$address2,
                    "postal_code"=>$postcode,
                    "state"=>$state,
                ],
              ],
            ]); 
            return ["status"=>"success","message"=>"","data"=>stripe_objToArray($intent)];
        }
        return ["status"=>"error","message"=>"Stripe Token is not valid. Token is :".$token];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_setRemoteData($params=[],$StripeData=[],$customer_id="",$payment_method_id="",$charge_id="")
{
    $invoiceid = $params['invoice']['invoiceid'];
    invoiceSetPayMethodRemoteToken($invoiceid,$StripeData,"StripeTokenId");
    $cres = stripe_get_payment_method($params,$payment_method_id); 
    if($cres["status"] == "success")
    {
        $card = $cres["data"]["card"];
        $array = [
            "card_type"=>ucfirst($card["brand"]),
            "expiry_date"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT)."/".substr($card["exp_year"], 2),
            "exp_month"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT),
            "exp_year"=>substr($card["exp_year"], 2),
            "cardnumber"=>$card["last4"],
            "last_four"=>$card["last4"],
            "customer_id"=>$customer_id,
            "method_id"=>$payment_method_id,
            "payment_intent"=>$StripeData["payment_intent"],
            "charge_id"=>$charge_id,
        ];
        invoiceSaveRemoteCard($params,$array);    
    }
}
function stripe_callback($params=[],$StripeData=[],$redirect=true)
{
    $eTitle = $params['gateway']['title'];
    $gateway_id = $params['gateway']['id'];
    $stripeSecretKey = $params["Secretkey"];
    // Invoice Parameters
    $invoiceid = $params['invoice']['invoiceid'];
    $invoice_no = $params["invoice_no"];
    $description = $params['invoice']["description"];
    $amount = $params['invoice']['amount'];
    $currency = $currencyCode = $params['invoice']['currency'];
    $payment_method = $params['invoice']['payment_method'];
    $user_id = $params['client']['id'];
    $fee = 0;
    $request = $_REQUEST;
    if(!empty($StripeData)) $request = $StripeData;
    $token = _s("setup_intent",$request,"");
    $source_type = _s("source_type",$request,"");
    $returnData = array();
    $StripeIntent = stripe_get_setupIntents($params,$token);
    if($StripeIntent["status"] == "error")
    {
        $message = "Transaction failed by Stripe, ".$StripeIntent["message"];
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$request,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    $StripeData = $StripeIntent["data"];
    $StripeData["payment_intent"] = $StripeData["id"];
    $payment_customer = $StripeData["customer"];
    $trans_payment_method_id = $StripeData["payment_method"];
    $PaymentIntentRes = stripe_create_intent($params,$payment_customer,$trans_payment_method_id,['confirm' =>true,"off_session"=>true]);
    if($PaymentIntentRes["status"] == "error")
    {
        $message = "Transaction failed by Stripe, ".$PaymentIntentRes["message"];
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$request,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    $trans_status = $StripeData["status"];
    $last_setup_error = $StripeData["last_setup_error"];
    if($trans_status != "succeeded")
    {
        $message = "Transaction failed by Stripe, ".$last_setup_error;
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    $StripeData = $PaymentIntentRes["data"];
    $UnsetKeys = ["payment_method_types","payment_method_options","next_action","on_behalf_of","livemode","metadata","application","application_fee_amount","automatic_payment_methods","processing","receipt_email","review","setup_future_usage","shipping","transfer_data","transfer_group"];
    foreach($UnsetKeys as $uk)
    {
        if(isset($StripeData[$uk])) unset($StripeData[$uk]);
    }
    $transid = $StripeData["latest_charge"];
    $trans_currency = $StripeData["currency"];
    $trans_payment_method = $transid;
    $trans_status = $StripeData["status"];
    $trans_last_payment_error = $StripeData["last_payment_error"];
    $trans_statement_descriptor_suffix = $StripeData["statement_descriptor_suffix"];
    $trans_amount = $StripeData["amount"]/100;
    $trans_amount_total = $StripeData["amount_received"]/100;
    $statement_descriptor_suffix = "Invoice No:".$invoice_no;
    if($trans_status != "succeeded")
    {
        $message = "Transaction failed by Stripe, ".$trans_last_payment_error;
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    if($trans_amount_total <> $trans_amount)
    {
        $message = "Transaction failed by Stripe, due to Received Amount is not valid";
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    if($statement_descriptor_suffix != $trans_statement_descriptor_suffix)
    {
        $message = "Transaction validation failed, Trans Suffix not matched with stripe suffix";
        AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    $isValid =  DoInvoiceValidate($invoiceid);
    if(!$isValid) return ["status"=>"error","message"=>"Invoice Id is Invailed"]; 
    $isValid =  DoTransactionValidate($invoiceid,$transid);
    if($isValid) return ["status"=>"error","message"=>"Transaction No already exist....."];
    $payment_added = AddInvoicePayment(['invoice_id'=>$invoiceid,'currency'=>$currency,'user_id'=>$user_id,'payment_method'=>$payment_method,'amount'=>$trans_amount_total,'trno'=>$transid,'fee'=>$fee,'notification'=>'YES']);
    $message = "Payment Confirmed By Stripe Gateway ";
    if($payment_added['status'] != 'success') $message .=" but ".$payment_added['message'];
    $request["TransStatus"] = "Success";
    AddGatewayLogs(array('invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$request,"message"=>$message));
    if($payment_added['status'] = 'success')
    {
        stripe_setRemoteData($params,$StripeData,$payment_customer,$trans_payment_method_id,$trans_payment_method);
        if($redirect)
        {
            header("Location: /client/view-invoice/".$invoiceid);
            die();
        }
        return ["status"=>"success","message"=>$message,"transid"=>$transid];
    }
    return ["status"=>"error","message"=>$message];
}
function stripe_post_checkout(array $params)
{
    $token = $params["custom"]["stripeToken"];
    $invoice_no = $params["invoice_no"];
    $cus_response = stripe_addCustomer($params,false);
    if($cus_response["status"] == "error") return $cus_response;
    $params["customerId"] = $customerId = $cus_response["id"];
    $response = stripe_payment_method_create($params,$token);
    if($response["status"] == "error") return $response;
    $params["method_id"] = $method_id = $response["data"]["id"];
    unset($params["custom"]);
    $params["description"] = "Invoice #:".$invoice_no;    
    $pend_url = $ret_url = $params["returnurl"];
    $message = "";
    $res = stripe_create_setupintent($params,$customerId,$method_id,['return_url' =>$pend_url,'confirm' =>true]);
    if($res["status"] == "error") return $res;
    $StripeData = $res = $res["data"];
    $next_action = _s("next_action",$res,[]);
    if(!empty($next_action)) return ["status"=>"success","message"=>"Stripe Redirected to Autorization","url"=>$next_action[$next_action["type"]]["url"]];
    $StripeData["payment_intent"] = $res["id"];
    $StripeData["source_type"] = "card"; 
    return stripe_callback($params,$StripeData,false);
}
function stripe_Scaauth($params=[],$StripeData=[])
{
    $stripeSecretKey = $params["Secretkey"];
    $gateway_id = $params['gateway']['id'];
    $user_id = $params['client']['id'];
    $request = $_REQUEST;
    if(!empty($StripeData)) $request = $StripeData;
    $token = _s("setup_intent",$request,"");
    $source_type = _s("source_type",$request,"");
    $returnData = array();
    if(empty($token)) return ["status"=>"error","message"=>"Stripe (SCA) Authorization Token empty"];
    $StripeIntent = stripe_get_setupIntents($params,$token);
    if($StripeIntent["status"] == "error")
    {
        $message = "3dSecure (SCA) Authorization failed by Stripe, ".$StripeIntent["message"];
        AddGatewayLogs(['invoice_id'=>0,'user_id'=>$user_id,'payment_method'=>$gateway_id,'response'=>$request,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }
    $StripeData = $StripeIntent["data"];
    $StripeData["payment_intent"] = $StripeData["id"];
    $customerId = $StripeData["customer"];
    $params["method_id"] = $method_id = $StripeData["payment_method"];
    $StripeData["source_type"] = "card"; 

    $trans_status = $StripeData["status"];
    $last_setup_error = $StripeData["last_setup_error"];
    if($trans_status != "succeeded")
    {
        $message = "Stripe Error: ".$last_setup_error;
        AddGatewayLogs(['invoice_id'=>0,'user_id'=>$user_id,'payment_method'=>$gateway_id,'response'=>$request,"message"=>$message]);
        return ["status"=>"error","message"=>$message];
    }

    $cres = stripe_get_payment_method($params,$method_id);
    $carddata = [];
    if($cres["status"] == "success")
    {
        $card = $cres["data"]["card"];
        $carddata = [
            "card_type"=>ucfirst($card["brand"]),
            "expiry_date"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT)."/".substr($card["exp_year"], 2),
            "exp_month"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT),
            "exp_year"=>substr($card["exp_year"], 2),
            "cardnumber"=>$card["last4"],
            "last_four"=>$card["last4"],
            "customer_id"=>$customerId,
            "method_id"=>$method_id,
            "payment_intent"=>$StripeData["payment_intent"],
        ];
        return ["status" => "success","message"=>"","carddata"=>$carddata,"rawdata"=>[]];        
    }
    return ["status"=>"error","message"=>"Stripe Remote Data Storing error","carddata"=>$carddata,"rawdata"=>$StripeData];
}

function stripe_storeremote(array $params)
{
    $stripeSecretKey = $params["Secretkey"];
    $card = _s("card",$params,[]);
    $customerId = _s("customer_id",$card,"");
    $method_id = _s("method_id",$card,"");
    $userid = _s("userid",$params,"");
    $custom = _s("custom",$params,[]);
    $token = _s("stripeToken",$custom,"");
    $action = _s("action",$custom,"");
    if($action == "delete") return stripe_detach_payment_method($params,$method_id);

    $intent = NULL;
    $method = NULL;
    $setupIntent = NULL;
    if(empty($customerId))
    {
        $Cards = _s("Cards",$params,[]);
        if(!empty($Cards)) $customerId = isset($Cards[0]["card_data"]["customer_id"]) ? $Cards[0]["card_data"]["customer_id"]:0;
    }
    if(empty($customerId))
    {
        $cus_response = stripe_addCustomer($params,false);
        if($cus_response["status"] == "error") return $cus_response;
        $params["customerId"] = $customerId = $cus_response["id"];
    }
    $response = stripe_payment_method_create($params,$token);
    if($response["status"] == "error") return $response;
    $params["method_id"] = $method_id = $response["data"]["id"];
    unset($params["custom"]);
    $params["description"] = "Customer ID:".$userid;    
    $pend_url = $ret_url = $params["returnurl"];
    $message = "";
    $res = stripe_create_setupintent($params,$customerId,$method_id,['return_url'=>$pend_url,'confirm' =>true]);
    if($res["status"] == "error") return $res;
    $StripeData = $res = $res["data"];
    $next_action = _s("next_action",$res,[]);
    if(!empty($next_action)) return ["status"=>"success","message"=>"Stripe Redirected to Autorization","url"=>$next_action[$next_action["type"]]["url"]];
    $StripeData["payment_intent"] = $res["id"];
    $StripeData["source_type"] = "card"; 
    $trans_status = $StripeData["status"];
    $last_setup_error = $StripeData["last_setup_error"];
    if($trans_status != "succeeded")
    {
        $message = "Stripe Error: ".$last_setup_error;
        return ["status"=>"error","message"=>$message];
    }
    $cres = stripe_get_payment_method($params,$method_id);
    $carddata = [];
    if($cres["status"] == "success")
    {
        $card = $cres["data"]["card"];
        $carddata = [
            "card_type"=>ucfirst($card["brand"]),
            "expiry_date"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT)."/".substr($card["exp_year"], 2),
            "exp_month"=>str_pad($card["exp_month"], 2, "0", STR_PAD_LEFT),
            "exp_year"=>substr($card["exp_year"], 2),
            "cardnumber"=>$card["last4"],
            "last_four"=>$card["last4"],
            "customer_id"=>$customerId,
            "method_id"=>$method_id,
            "payment_intent"=>$StripeData["payment_intent"],
        ];
        return ["status" => "success","message"=>"","carddata"=>$carddata,"rawdata"=>[]];        
    }
    return ["status"=>"error","message"=>"Stripe Remote Data Storing error","carddata"=>$carddata,"rawdata"=>$StripeData];
}
function stripe_Capture($params=[])
{
    $card = isset($params["card"]) ? $params["card"]:"";
    if(empty($card)) return array("status" =>"error", "rawdata" =>[],"message" =>"Card Data is empty");  
    $customer_id = $card["customer_id"];
    $method_id = $card["method_id"];
    $stripeSecretKey = $params["Secretkey"];
    $currency = $params["currency"];
    $amount = $params["amount"];
    try {
        $cus_response = stripe_getCustomer($params,$customer_id);
        if($cus_response["status"] == "error") return $cus_response;
        $meth_response = stripe_get_payment_method($params,$method_id);
        if($meth_response["status"] == "error") return $meth_response;
        $pay_response = stripe_create_intent($params,$customer_id,$method_id,['confirm' =>true,'off_session' =>true]);
        if($pay_response["status"] == "error") return $pay_response;
        $StripeData = $pay_response["data"];
        $StripeData["payment_intent"] = $StripeData["id"];
        $StripeData["source_type"] = "card";
        $gateway_id = $params['gateway']['id'];
        // Invoice Parameters
        $invoiceid = $params['invoice']['invoiceid'];
        $payment_method = $params['invoice']['payment_method'];
        $user_id = $params['client']['id'];
        $fee = 0;
        $request = $StripeData;
        $token = _s("payment_intent",$request,"");
        $source_type = _s("source_type",$request,"");
        $returnData = array();
        if($pay_response["status"] == "error")
        {
            $message = "Transaction failed by Stripe, ".$StripeIntent["message"];
            AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$request,"message"=>$message]);
            return ["status"=>"error","message"=>$message];
        }
        $UnsetKeys = ["payment_method_types","payment_method_options","next_action","on_behalf_of","livemode","metadata","application","application_fee_amount","automatic_payment_methods","processing","receipt_email","review","setup_future_usage","shipping","transfer_data","transfer_group"];
        foreach($UnsetKeys as $uk)
        {
            if(isset($StripeData[$uk])) unset($StripeData[$uk]);
        }
        $transid = $StripeData["latest_charge"];
        $payment_customer = $StripeData["customer"];
        $trans_currency = $StripeData["currency"];
        $trans_payment_method = isset($StripeData["payment_method_configuration_details"]["id"]) ? $StripeData["payment_method_configuration_details"]["id"]:$transid;
        $trans_card_id = !empty($StripeData["source"]) ? $StripeData["source"]:$StripeData["payment_method"];
        $trans_status = $StripeData["status"];
        $trans_last_payment_error = $StripeData["last_payment_error"];
        $trans_amount = $StripeData["amount"]/100;
        $trans_amount_total = $StripeData["amount_received"]/100;
        if($trans_status != "succeeded")
        {
            $message = "Transaction failed by Stripe, ".$trans_last_payment_error;
            AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
            return ["status"=>"error","message"=>$message];
        }
        if($trans_amount_total <> $trans_amount)
        {
            $message = "Transaction failed by Stripe, due to Received Amount is not valid";
            AddGatewayLogs(['invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$StripeData,"message"=>$message]);
            return ["status"=>"error","message"=>$message];
        }
        $payment_added = AddInvoicePayment(['invoice_id'=>$invoiceid,'currency'=>$currency,'user_id'=>$user_id,'payment_method'=>$payment_method,'amount'=>$trans_amount_total,'trno'=>$transid,'fee'=>$fee,'notification'=>'YES']);
        $message = "Payment Confirmed By Stripe Gateway ";
        if($payment_added['status'] != 'success') $message .=" but ".$payment_added['message'];
        $request["TransStatus"] = "Success";
        AddGatewayLogs(array('invoice_id'=>$invoiceid,'user_id'=>$user_id,'payment_method'=>$payment_method,'response'=>$request,"message"=>$message));
        if($payment_added['status'] = 'success') return ["status"=>"success","message"=>$message,"transid"=>$transid,"rawdata" =>$request,"fee"=>0];
        return ["status" =>"error", "rawdata" =>$StripeData,"message"=>$response["message"]];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return array("status" =>"error", "rawdata" =>[],"message" =>$message);        
}
function stripe_refund($params=[])
{
    $SecretKey = isset($params["gateway"]["Secretkey"]) ? $params["gateway"]["Secretkey"]:"";
    $transaction_no = $params["transaction_no"];
    if(empty($transaction_no)) return ["status"=>"error","message"=>"Transaction No empty"];
    try {
        $stripe = new \Stripe\StripeClient($SecretKey);
        $res = $stripe->refunds->create(['charge' => $transaction_no]);
        $res = stripe_objToArray($res);
        
        $trans_id = $res["id"];
        $trans_status = $res["status"];
        $trans_reason = $res["reason"];
        unset($res["destination_details"]);
        unset($res["metadata"]);
        if($trans_status == "succeeded") return ["status"=>"success","message"=>"Gateway Refunded Successfully","transid"=>$trans_id,"fees"=>0,"rawdata"=>$res];
        return ["status"=>"error","message"=>$trans_reason];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e)
    {
      $message = $e->getMessage();
    } 
    return ["status"=>"error","message"=>$message];
}
function stripe_TransactionInformation($params=[])
{
    $SecretKey = $params["Secretkey"];
    $transaction_no = $params["transaction_no"];
    $transactionData = [];
    $message="";
    try{
        $stripe = new \Stripe\StripeClient($SecretKey);
        if (substr($transaction_no, 0, 2) == "ch")
        {
            $intent = $stripe->charges->retrieve($transaction_no, []);
            $res = stripe_objToArray($intent);
            $information = 
                [
                    "TransactionId"=>$res['id'],
                    "Amount"=>$res['amount_captured']/100,
                    "Currency"=>$res['currency'],
                    "Type"=>$res["payment_method_details"]['type'],
                    "AvailableOn"=>date('Y-m-d h:i:s',$res['created']),//Datetime as Y-m-d h:i:s
                    "Created"=>date('Y-m-d h:i:s',$res['created']),//Datetime as Y-m-d h:i:s
                    "Description"=>$res['statement_descriptor_suffix'],
                    "Fee"=>$res['application_fee'],
                    "Status"=>$res['status'],
                    "AdditionalDatum"=>
                    [
                      "Card No"=>"**** **** **** ".$res['payment_method_details']["card"]["last4"],
                      "Card Expiry"=>$res['payment_method_details']["card"]["exp_month"]."/".$res['payment_method_details']["card"]["exp_year"],
                      "Card Network"=>$res['payment_method_details']["card"]["network"],
                      "Card Country"=>$res['payment_method_details']["card"]["country"],
                    ],
                ];
        }
        if (substr($transaction_no, 0, 2) == "re")
        {
            $intent = $stripe->refunds->retrieve($transaction_no, []);
            $res = stripe_objToArray($intent);
            $charge_intent = $stripe->charges->retrieve($res["charge"], []);
            $charge_res = stripe_objToArray($charge_intent);
            $information = 
                [
                    "TransactionId"=>$res['id'],
                    "Amount"=>$res['amount']/100,
                    "Currency"=>$res['currency'],
                    "Type"=>$res["destination_details"]['type'],
                    "AvailableOn"=>date('Y-m-d h:i:s',$res['created']),//Datetime as Y-m-d h:i:s
                    "Created"=>date('Y-m-d h:i:s',$res['created']),//Datetime as Y-m-d h:i:s
                    "Description"=>"Transaction Refunded",
                    "Fee"=>0,
                    "Status"=>$res['status'],
                    "AdditionalDatum"=>
                    [
                      "Card No"=>"**** **** **** ".$charge_res['payment_method_details']["card"]["last4"],
                      "Card Expiry"=>$charge_res['payment_method_details']["card"]["exp_month"]."/".$charge_res['payment_method_details']["card"]["exp_year"],
                      "Card Network"=>$charge_res['payment_method_details']["card"]["network"],
                      "Card Country"=>$charge_res['payment_method_details']["card"]["country"],
                    ],
                ];
        }
        return ["status"=>"success","message"=>"Your Gateway message here if exist","response"=>$information];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message,"response"=>$information];
}
function stripe_Balance($params=[])
{
    $SecretKey = isset($params["gateway"]["Secretkey"]) ? $params["gateway"]["Secretkey"]:"";
    $balances = [];
    if(empty($SecretKey)) return ["status"=>"error","message"=>"Stripe Secret Key empty","data"=>$balances];
    try {
        $stripe = new \Stripe\StripeClient($SecretKey);
        $res = $stripe->balance->retrieve([]);    
        $res = _jd(_je($res),true);
        $availables = $res["available"];
        foreach($availables as $available)
        {
            $currency = $available["currency"];
            $amount = $available["amount"];
            $balances["available"][$currency] = $amount;
        }
        $pendings = $res["pending"];
        foreach($pendings as $pending)
        {
            $currency = $pending["currency"];
            $amount = $pending["amount"];
            $balances["pending"][$currency] = $amount;
        }
        return ["status"=>"success","message"=>"","data"=>$balances];
    }catch(CardException|RateLimitException|InvalidRequestException|AuthenticationException|ApiConnectionException|ApiErrorException $e)
    {
      $message = $e->getError()->message;
    } catch (Exception $e) {
      $message = $e->getMessage();
    }
    return ["status"=>"error","message"=>$message,"data"=>$balances];
}