<?php
use Carbon\Carbon;
function cosmotown_getConfig()
{
    return array(
        "name"=>"Cosmotown",
        "description"=>"This module can be used for any TLDs that have no integrated registrar",
        "version"=>"1.0",
        "author"=>"whasols",
        "author_url"=>"https://docs.whasols.com/Cosmotown",
        "configs"=>[
        'APIKey' => [
            'Name' => 'API Key',
            'Type' => 'password',
            'Size' => '25',
            'Default' => '',
            'Description' => 'Enter your API key here. To get your api key, go to My Account section in cosmotown.com, then click Reseller API link on the left hand side. C/p the key here. DON\'T include your password.',
        ],
        'PluginMode' => [
            'Name' => 'Plugin Mode',
            'Type' => 'radio',
            'Options' =>["Live Mode"=>"Live Mode","Sandbox"=>"Sandbox"],
            'Description' => 'Choose plugin behavior',
        ],
    ]);
}
function cosmotown_api(string $action, array $params = [], array $payload = [], string $method = 'POST')
{
    $apiKey = _s('APIKey',$params,"");
    if(empty($apiKey)) return ["status"=>"error","message"=>"API Key Required"];
    if($params['PluginMode'] == "Live Mode")
    {
        $apiUrl='https://cosmotown.com/v1/reseller/';            
    }else{
        $apiUrl='https://sandbox.cosmotown7.com/v1/reseller/';
    }
    $payload = json_encode($payload);
    try {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $apiUrl . $action);
    $headers = [
        'X-API-TOKEN: ' . $apiKey,
        'Content-Type: application/json'
    ];
    if ($method == 'POST') {
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        $headers[] = 'Content-Length: ' . strlen($payload);
    }
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_TIMEOUT, 100);
    $response = curl_exec($ch);
    if (curl_errno($ch))
    {
        return ["status"=>"error","message"=>'Connection Error: ' . curl_errno($ch) . ' - ' . curl_error($ch)];
    }
    curl_close($ch);
    $results = [];
    if($response)
    {
        $results = json_decode($response, true);
        if ($results === null && json_last_error() !== JSON_ERROR_NONE) 
            return ["status"=>"error","message"=>"Bad response received from API"];
    }
    logModuleCall('CosmotownRegistrar',$apiUrl.$action,$payload,$response,$results);
    } catch (\Exception $e)
    {
        return ["status"=>"error","message"=>$e->getMessage()];
    }
    return ["status"=>"success","message"=>"","data"=>$results];
}

function cosmotown_call(string $action, array $params = [], array $payload = [], string $method = 'POST'){
    return cosmotown_api($action,$params,$payload,$method);
}
function cosmotown_RegisterDomain($params)
{
    $payload = array(
        "coupon_id" => $params['PromotionCode'],
        "items" => [
            [
                "name" => $params["domainname"],
                "years" => (int)$params["regperiod"]
            ]
        ]
    );
    return cosmotown_call("registerdomains",$params,$payload);
}
function cosmotown_RenewDomain($params)
{
    $payload = array(
        "coupon_id" => $params['PromotionCode'],
        "items" => [
            [
                "name" => $params["domainname"],
                "years" => (int)$params["regperiod"]
            ]
        ]
    );
    return cosmotown_call('renewdomains',$params,$payload);
}
function cosmotown_TransferDomain($params)
{
    $payload = ["items" => [["name" => $params["domainname"],"authCode" => base64_encode($params["transfersecret"])]]];
    return cosmotown_call('transferdomains',$params,$payload);
}
function cosmotown_Sync($params)
{
    $results = cosmotown_call('listdomains?domain=' . $params["domainname"],$params,[], 'GET');
    if($results["status"] == "error") return $results;
    $result = $results["data"];
    if (isset($result['domains'][0]['domain']))
    {
        $expDate = Carbon::createFromFormat('Y-m-d H:i:s', $result['domains'][0]['expiration_date']);
        $nowDate = Carbon::now();
        return [
            'active' => ($expDate->format('Ymd') > $nowDate->format('Ymd')),
            'cancelled' => false,
            'transferredAway' => false,
            'expirydate' => $expDate->format('Y-m-d'),
            'expired' => ($expDate->format('Ymd') < $nowDate->format('Ymd')),
        ];
    }
    return ["status"=>"error","message"=>'Domain ' . $params["domainname"] . ' not found in this registrar.'];
}
function cosmotown_TransferSync($params)
{
    $values = ['completed' => false,'reason' => 'In process'];
    $results = cosmotown_call('domainstatus',$params,["domains" => [$params["domainname"]]]);
    if($results["status"] == "error") return $results;
    $results = $results["data"];
    foreach ($results as $result)
    {
        if ($result['domain'] == $params["domainname"])
        {
            if ($result['registration_status'] == 'COMPLETE') {
                $values['completed'] = true;
                $values['failed'] = false;
            } else {
                $values['completed'] = false;
                $values['failed'] = true;
                $values['reason'] = $result['message'];
            }
        }
    }
    return $values;
}
function cosmotown_GetDomainInformation($params)
{
    $domain = $params["domainname"];
    $defaultNameservers = ['ndns1.cosmotown.com', 'ndns2.cosmotown.com'];
    $results = cosmotown_call('domaininfo?domain=' . $domain,$params,[],'GET');
    if($results["status"] == "error") return $results;
    $result = $results["data"];
    if (!isset($result['domain'])) return ["status"=>"error","message"=>"Domain Information not found"];
    $nameservers = [];
    foreach ($result['nameservers'] as $i => $ns)
    {
        $nameservers['ns'.($i + 1)] = $ns;
    }
    if (empty($nameservers)) $nameservers = $defaultNameservers;
    $expDate = date('Y-m-d H:i:s',strtotime($result['expiration_date']));
    return [
        'status'=>'success',
        'response'=>[
            'domain'=>$domain,
            'regdate'=>"",
            'expdate'=>$expDate,
            'lockstatus'=>(bool)$result['locked'],
            'nameservers'=>$nameservers
        ]
    ];
}
function cosmotown_GetRegistrarLock($params)
{
    $domain = $params["domainname"];
    $results = cosmotown_call('domaininfo?domain=' . $domain,$params,[],'GET');
    if($results["status"] == "error") return $results;
    $result = $results["data"];
    $lockstatus = $result['locked'] ? "locked" : "unlocked";
    return ['status'=>'success','response'=>$lockstatus];
}
function cosmotown_SaveRegistrarLock($params)
{
    $domain = $params["domainname"];
    $results = cosmotown_call('domaininfo?domain=' . $domain,$params,[],'GET');
    if($results["status"] == "error") return $results;
    $result = $results["data"];
    $payload = array(
        "domain" => $domain,
        "options" => [
            "enable_private_whois" => $result['whois_privacy'],
            "lock_domain" => (("locked" == $params['lockenabled']) ? true : false),
            "enable_auto_billing" => $result['auto_billing']
        ]
    );
    $results = cosmotown_call('domaininfo',$params,$payload);
    if($results["status"] == "error") return $results;
    return ["status"=>"success"];
}
function cosmotown_GetNameservers($params)
{
    $domain = $params["domainname"];
    $results = cosmotown_call('domaininfo?domain=' . $domain,$params,[],'GET');
    if($results["status"] == "error") return $results;
    $result = $results["data"];
    if (!isset($result['nameservers'])) return ["status"=>"error","message"=>"Domain Name Servers not found"];
    $nameservers = [];
    foreach ($result['nameservers'] as $i => $ns)
    {
        $nameservers['ns'.($i + 1)] = $ns;
    }
    if (empty($nameservers)) $nameservers = $defaultNameservers;
    return [
        "status"=>"success",
        'ns1' => _s('ns1',$nameservers,""),
        'ns2' => _s('ns2',$nameservers,""),
        'ns3' => _s('ns3',$nameservers,""),
        'ns4' => _s('ns4',$nameservers,""),
        'ns5' => _s('ns5',$nameservers,""),
        'ns6' => _s('ns6',$nameservers,""),
    ];
}
function cosmotown_SaveNameservers($params)
{
    $domain = $params["domainname"];
    $nameservers = [];
    $nameservers[] = $params['ns1'];
    $nameservers[] = $params['ns2'];
    $nameservers[] = $params['ns3'];
    $nameservers[] = $params['ns4'];
    $nameservers[] = $params['ns5'];
    $payload = ["domain" => $domain,"nameservers" => $nameservers];
    $results = cosmotown_call('savedomainnameservers',$params,$payload);
    if($results["status"] == "error") return $results;
    return ["status"=>"success"];
}