<?php
class ApiClient
{
    protected $results = array();
    protected $config;
    public function __construct($config)
    {
        $this->config = $config;
    }
    /**
     * Make external API call to registrar API.
     *
     * @param string $action
     * @param array $postfields
     *
     * @return array
     * @throws \Exception Bad API response
     *
     * @throws \Exception Connection error
     */
    public function request($action, $postfields)
    {
        $timestamp = time();
        $params = [
            'user' => $this->config['user'],
            'timestamp' => $timestamp,
            'token' => md5($this->config['user'] . md5($this->config['pass']) . $timestamp . $action),
            'apikey' => $this->config['apikey'],
            'from' => 'WHASOLS'
        ];
        $params = array_merge($params, $postfields);
        $ch = curl_init();
        $url = 'https://' . $this->config['server'] . '/api4/domain/index.php?command=' . $action;
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->config['timeout']);
        $response = curl_exec($ch);
        if (curl_errno($ch)) return ["status"=>"error","message"=>'Connection Error: ' . curl_errno($ch) . ' - ' . curl_error($ch)];
        curl_close($ch);
        $this->results = $this->processResponse($response);
        logModuleCall(
            'OnlineNICAPI4',
            $action,
            $postfields,
            $response,
            $this->results,
            array(
                $this->config['user'],
                $this->config['pass'],
            )
        );
        if ($this->results === null && json_last_error() !== JSON_ERROR_NONE) {
            throw new \Exception('Bad response received from API');
        }
        if ($this->results['code'] > 1000 && $this->results['code'] < 2000) {
            throw new \Exception($this->results['msg']);
        }
        return $this->results;
    }
    /**
     * Process API response.
     *
     * @param string $response
     *
     * @return array
     */
    public function processResponse($response)
    {
        return json_decode($response, true);
    }
    /**
     * Get from response results.
     *
     * @param string $key
     *
     * @return string
     */
    public function getFromResponse($key)
    {
        return isset($this->results[$key]) ? $this->results[$key] : '';
    }
}