<?php
use Carbon\Carbon;
/************************************************************************
*                                                                       *
* apnscp Provisioning Module for WHMSOLS (v1.0).                        *
*                                                                       *
* Contributed by: WHMSOLS                                               *
* Website: https://whmsols.com                                          *
*                                                                       *
*************************************************************************/



require_once('apnscp_class.php');

/**
 * Define module related meta data.
 *
 * Values returned here are used to determine module related abilities and
 * settings.
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return array
 */


function apnscp_config()
{
    //$plans = apnscp_getPlans();
    $plans = [];
    return array(
        "name" => "Apnscp Control Panel",
        "description" => "CWP Hosting control panel",
        "version" => "1.0",
        "author" => "contributed by Hostarina",
        "author_url"=>"",
        "RequiresServer"=>true,//true or false
        "DefaultNonSSLPort"=>'2082',
        "DefaultSSLPort"=>'2083',
        "ServiceSingleSignOnLabel"=>'Login to Panel',
        "AdminSingleSignOnLabel"=>'Login to Panel as Admin',
        "PackagesLoader"=>'PackagesLoader',
        "configs" => array('apnscpPlan' => array(
            'Type'        => 'dropdown',
            'Options'     => $plans,
            'Default'     => 'basic',
            'Description' => 'Choose a plan (auto populated)',
        ))
    );
    
    return $configarray;
}



function apnscp_TestConnection($arg){
    try
    { 
        $server = $arg['server'];
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $time = $client->common_get_uptime();
        return array('status'=>($time) ? 'success':'error','message'=>'');
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $url,
            $e->getMessage(),
            $e->getTraceAsString()
        );
      return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

/**
 * PackagesLoader function that will populate the field in ConfigOptions
 * @return array The list of package names
 */
function apnscp_PackagesLoader($arg) {
 try {    
        $server = $arg['server'];
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $packageNames = $client->admin_list_plans();
        $list = [];
        foreach ($packageNames as $opKey => $opValue) {
            $list[$opValue] = ucfirst($opValue);
        }
        return $list;
    } catch (Exception $e) {
        logModule(
            'apnscp',
            __FUNCTION__,
            $url,
            $e->getMessage(),
            $e->getTraceAsString()
        );
       return array('basic' => 'basic (api call failed)');
    }
    return array('status'=>'success','message'=>'');
}



/**
 * Provision a new instance of a product/service.
 *
 * Attempt to provision a new instance of a given product/service. This is
 * called any time provisioning is requested inside of WHMSOLS. Depending upon the
 * configuration, this can be any of:
 * * When a new order is placed
 * * When an invoice for a new order is paid
 * * Upon manual request by an admin user
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_CreateAccount($arg) {

    try
    {

      $server = $arg['server'];
      $client = $arg['client'];
      $product = $arg['product'];
      
      $domain = strtolower($product['domain']);
      $package = $product['package'];
      $username = $product['username'];
      $password = $product['password'];
      $producttype = $product['producttype'];
      $email = $client['email'];
      $phonenumber = $client['phonenumber'];
      $firstname = $client['firstname'];
      $lastname = $client['lastname'];

        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];

        $_options = array('domain'=>$domain,'username'=>$username,'email'=>$email,'package'=>$package);
        $opts = ApnsCpHelper::generateOptions($_options);
        $cliCommand = ApnsCpHelper::generateCommand($opts, 'AddDomain');

        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->admin_add_site($domain, $username, $opts);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Account Created Successfully....':'Account Already Exist or error...'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $_options,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }

    return array('status'=>'success','message'=>'');
}

/**
 * Suspend an instance of a product/service.
 *
 * Called when a suspension is requested. This is invoked automatically by WHMSOLS
 * when a product becomes overdue on payment or can be called manually by admin
 * user.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_SuspendAccount($arg)
{
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];      
      $domain = strtolower($product['domain']);
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->admin_deactivate_site($domain);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Account Suspended Successfully....':'Account Suspention failed.......'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domain,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}


/**
 * Un-suspend instance of a product/service.
 *
 * Called when an un-suspension is requested. This is invoked
 * automatically upon payment of an overdue invoice for a product, or
 * can be called manually by admin user.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_UnsuspendAccount($arg) {
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];      
      $domain = strtolower($product['domain']);
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->admin_activate_site($domain);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Account UnSuspended Successfully....':'Account UnSuspention failed.......'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domain,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

/**
 * Terminate instance of a product/service.
 *
 * Called when a termination is requested. This can be invoked automatically for
 * overdue products if enabled, or requested manually by an admin user.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_TerminateAccount($arg) {
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];      
      $domain = strtolower($product['domain']);
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
    $opts = [];
    $opts['force'] = 'true';

        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->admin_delete_site($domain, $opts);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Account Terminated Successfully....':'Account Termination failed.......'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domain,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}


/**
 * Change the password for an instance of a product/service.
 *
 * Called when a password change is requested. This can occur either due to a
 * client requesting it via the client area or an admin requesting it from the
 * admin side.
 *
 * This option is only available to client end users when the product is in an
 * active status.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_ChangePassword($arg) {
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];            
      $domain = strtolower($product['domain']);
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];


        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->auth_change_password($password,$username,$domain);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Account Password Changed Successfully....':'Account Password Changes failed.......'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domain,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}



/**
 * Upgrade or downgrade an instance of a product/service.
 *
 * Called to apply any change in product assignment or parameters. It
 * is called to provision upgrade or downgrade orders, as well as being
 * able to be invoked manually by an admin user.
 *
 * This same function is called for upgrades and downgrades of both
 * products and configurable options.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return string "success" or an error message
 */
function apnscp_ChangePackage($arg) {
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $package = $product['package'];            
      $domain = strtolower($product['domain']);
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];


    $opts['siteinfo.plan'] = $package;
    $extra['reset'] = 'true';

    $cliCommand = ApnsCpHelper::generateCommand($opts, 'EditDomain');

        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $result = $client->admin_edit_site($domain, $opts, $extra);
        return array('status'=>($result==1) ? 'success':'error','message'=>(($result==1) ? 'Package Changed Successfully':'Package Changes failed.......'),'response'=>$result);
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domain,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}


/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns a URL to which the user should be redirected.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return array
 */
function apnscp_AdminSingleSignOn($arg) {
    $code = '';
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $package = $product['package'];            
      $domain = strtolower($product['domain']);
      $url = $server['is_secure'].$server['hostname'].':'.$server['port'];

    //$app                = App::get_req_var('app');
    $extra              = [];
    $allowed_apps       = [
        'usermanage',
        'mailboxroutes',
        'vacation',
        'filemanager',
        'domainmanager',
        'bandwidthbd',
        'crontab',
        'subdomains',
        'changemysql',
        'phpmyadmin',
        'webapps',
        'terminal',
    ];




        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $session_id = $client->admin_hijack($domain, $username, 'UI');       
        $extra['esprit_id'] = $session_id;
        $query              = http_build_query($extra);
        $url = $url."/apps/dashboard?".$query;
        if(empty($session_id)) return array('status'=>'error','message'=>"Session Creation failed.");
        return $url;
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $query,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
}

/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns a URL to which the user should be redirected.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return array
 */
function apnscp_ServiceSingleSignOn($arg) {
    $code = '';
    try
    {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $package = $product['package'];            
      $domain = strtolower($product['domain']);
      $url = $server['is_secure'].$server['hostname'].':'.$server['port'];

    //$app                = App::get_req_var('app');
    $extra              = [];
    $allowed_apps       = [
        'usermanage',
        'mailboxroutes',
        'vacation',
        'filemanager',
        'domainmanager',
        'bandwidthbd',
        'crontab',
        'subdomains',
        'changemysql',
        'phpmyadmin',
        'webapps',
        'terminal',
    ];




        $adminId = session_id();
        $client  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
        $session_id = $client->admin_hijack($domain, $username, 'UI');       
        $extra['esprit_id'] = $session_id;
        $query              = http_build_query($extra);
        $url = $url."/apps/dashboard?".$query;
        if(empty($session_id)) return array('status'=>'error','message'=>"Session Creation failed.");
        return $url;
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $query,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
}


/**
 * Client area output logic handling.
 *
 * This function is used to define module specific client area output. It should
 * return an array consisting of a template file and optional additional
 * template variables to make available to that template.
 *
 * @param array $params common module parameters
 *
 * @see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return array
 */
function apnscp_ClientArea(array $arg)
{
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $package = $product['package'];            
      $domain = strtolower($product['domain']);
    return array(
        'templatefile' => 'overview.tpl',
        'templatepath' => 'modules/servers/apnscp/templates/',
        'vars' => array(
            'test1' => 'hello',
            'test2' => 'world',
        ),
    );
}



function apnscp_UsageUpdate($arg) {
    try
    {
        $server = $arg['server'];
        $url = $server['is_secure'].$server['hostname'].':'.$server['port'];
        $stats = [];


        $adminId = session_id();
        $adminClient  = ApnsCpConnector::create_client($server["accesshash"], $url, $adminId);
    $domains     = $adminClient->admin_get_domains();
    foreach ($domains as $domain)
    {
        $session_id = $adminClient->admin_hijack($domain);
        $userClient = ApnsCpConnector::create_client($server["accesshash"], $url, $session_id);
        $bw         = $userClient->site_get_bandwidth_usage();
        $quota      = $userClient->site_get_account_quota();

        $stats[] = ['domain' => $domain, 'bw' => $bw, 'disk' => $quota];
    }
    $count = 0;
    # Now loop through results and update DB
   foreach ($stats AS $values)
    {

           $isUpdated =  update_query("hosting",array(
              "disk_usage"=>$values['disk']['qused'] / 1024,
              "disk_limit"=>$values['disk']['qhard'] / 1024,
              "bandwidth_usage"=>($values['bw']['used'] / 1024) / 1024,
              "bandwidth_limit"=>($values['bw']['total'] / 1024) / 1024,
              "email_accounts_limit"=>'',
              "email_accounts_usage"=>'',
              "domains_limit"=>'',
              "domains_usage"=>'',
              "sub_domains_limit"=>'',
              "sub_domains_usage"=>'',
              "parked_domains_limit"=>'',
              "parked_domains_usage"=>'',
              "databases_limit"=>'',
              "databases_usage"=>'',
              "databases_disk_limit"=>'',
              "databases_disk_usage"=>'',
              "last_update"=>Carbon::now()->format('Y-m-d h:i:s'),
            ),array("server_id"=>$server['id'], "domain_name"=>$values['domain']));
            if($isUpdated) $count++;

    }
    return array('status'=>'success','message'=>$count.' Accounts Stats Updated....');
    }
    catch (Exception $e)
    {
        logModule(
            'apnscp',
            __FUNCTION__,
            $domains,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}
