<?php
/************************************************************************
*                                                                       *
* apnscp Provisioning Module for WHMSOLS (v1.0).                        *
*                                                                       *
* Contributed by: WHMSOLS                                               *
* Website: https://whmsols.com                                          *
*                                                                       *
*************************************************************************/
class ApnsCpConnector extends SoapClient {
    const WSDL_PATH = '/apnscp.wsdl';
    // @var string session cookie identifier
    const COOKIE_NAME = 'esprit_id';

    private $id;

    /**
     * Create new API client
     *
     * @param       $api_key
     * @param       $api_endpoint
     * @param array $ctor additional constructor arguments to SoapClient
     *
     * @return SoapClient
     */
    public static function create_client($api_key, $api_endpoint, ...$ctor): \SoapClient
    {
        $uri                  = $api_endpoint . '/soap';
        $wsdl                 = str_replace('/soap', self::WSDL_PATH, $uri);

        $ip = $ctor[1] ?: $_SERVER['REMOTE_ADDR'];

        $headers = [
            'Abort-On: error',
            'X-Forwarded-For: ' . $ip,
        ];

        $connopts             = $ctor + [
                'connection_timeout' => 30,
                'location'           => $uri,
                'uri'                => 'urn:net.apnscp.soap',
                'trace'              => true,
                'stream_context'     => stream_context_create([
                    'http' => [
                        'header' => implode("\r\n", $headers) . "\r\n",
                    ],
                ]),
            ];

        $connopts['location'] = $uri . '?authkey=' . $api_key;

        return (new static($wsdl, $connopts))->setId($ctor[0] ?? \session_id());
    }

    /**
     * Set session name to pass between requests
     *
     * @param string $name
     *
     * @return Util_API
     */
    public function setId(string $name): self
    {
        $this->id = $name;

        return $this;
    }

    public function __call($function_name, $arguments)
    {
        static $ctr = 0;
        $this->__setCookie(self::COOKIE_NAME, $this->id);
        $ret = parent::__call($function_name, $arguments);
        if ($ret !== null || $ctr >= 5)
        {
            $ctr = 0;

            return $ret;
        }
        // 50 ms sleep
        usleep(50000);
        $ctr++;

        return $this->__call($function_name, $arguments);
    }
}


class ApnsCpHelper {

    /**
     * @param array $params
     *
     * @return array
     */
    public static function generateOptions(array $arg): array
    {
        $opts = [];

        //Site Info
        $opts['siteinfo.enabled']    = '1'; // [0,1] Core account attributes
        $opts['siteinfo.domain']     = $arg['domain']; // <string> Primary domain of the account
        $opts['siteinfo.admin_user'] = $arg['username']; // <string> Administrative user of account
        $opts['siteinfo.email']      = $arg['email']; // [email,[email1,email2...]] Contact address on account
        $opts['siteinfo.plan']       = $arg['package'];

        return $opts;
    }

    /**
     * @param array $opts
     *
     * @return string
     */
    public static function generateCommand(array $opts, $action): string
    {
        $optArray   = [];
        $optArray[] = $action;

        foreach ($opts as $service => $value)
        {
            $service    = str_replace('.', ',', $service);
            $optArray[] = "-c '{$service}'='{$value}'";
        }

        return implode(' ', $optArray);
    }
}