<?php
use Carbon\Carbon;
function cpanel_config()
{
    return array(
        "name" => "Cpanel",
        "description" => "cpanel Hosting control panel",
        "version" => "1.0",
        "author" => "contributed by Hostarina",
        "author_url"=>"",
        "RequiresServer"=>true,//true or false
        "DefaultNonSSLPort"=>'',
        "DefaultSSLPort"=>'2087',
        "WebmailSSLPort"=>'2096',
        "WebmailNonSSLPort"=>'2095',
        "ServiceSingleSignOnLabel"=>'',
        "AdminSingleSignOnLabel"=>'',
        "PackagesLoader"=>'PackagesLoader',
        "configs" => array(
        "SimpleMode" => array(
            'Name' => 'Simple Mode',
            'Type' => 'text',
            'Size' => '25',
            'SimpleMode' => true,
        ),
        "AdvancedMode"  => array(
            'Name' => 'Advance Mode',
            'Type' => 'text',
            'Size' => '25',
        ),
        'comments' => array(
                            'Name' => 'Notes',
                            "Type" => "textarea", # Textarea
                            "Rows" => "3", # Number of Rows
                            "Cols" => "50", # Number of Columns
                            "Description" => "Description goes here",
                            "Default" => "Enter notes here",
            )
        )
    );
}


/**
 * PackagesLoader function that will populate the field in ConfigOptions
 * @return array The list of package names
 */
function cpanel_PackagesLoader($arg) {
 try {      
            $server = $arg['server'];
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->listpkgs());
            $packageNames = $res->package;
            $list = [];
            foreach ($packageNames as $option) {
                $list[$option->name] = ucfirst($option->name);
            }
            return $list;
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}


function cpanel_TestConnection($arg){    
 try {      
        $server = $arg['server'];
        require_once 'cpanel_class.php';
        $xmlapi = new cpanel_xml($server['ipaddress']);
        //$xmlapi->password_auth($svr_info->username,$svr_info->password);
        $xmlapi->hash_auth($server['username'],$server['accesshash']);
        $xmlapi->set_port($server['port']);
        $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
        $xmlapi->set_output("json");
        //$xmlapi->set_auth_type('hash');
        $xmlapi->set_debug(1);  
        $result = json_decode($xmlapi->loadavg(),true);
        if (isset($result["cpanelresult"]["error"])) return ['status'=>'error','message'=>$result["cpanelresult"]["error"]];
    } catch (Exception $e) {
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

function cpanel_createEmailAcct($arg)
{
 try {    
            $server = $arg['server'];
            $product = $arg['product'];
            $username = $product['username'];
            $password = $product['password'];  
            $domain = $product['domain'];  
            $custom = $arg['custom'];

            $email_quota = _s('email_quota',$custom,0);
            $email_quota = $email_quota > 0 ? $email_quota:500;
            $email_prefix = $custom['email_prefix'];
            $email_pw = $custom['email_pw'];
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  

            $decoded_response = json_decode($xmlapi->addpop($username,array('password'=>$email_pw,'quota'=>$email_quota,'email'=>$email_prefix,'domain'=>$domain)));

            $reason = $decoded_response->cpanelresult->data->reason;
            $result = $decoded_response->cpanelresult->data->result;
            if($result == 1)  return ['status'=>'success'];
            return ['status'=>'error','message'=>$reason];
           // $result = $res['result'];

  } catch (Exception $e) {
     return array('status'=>'error','message'=> $e->getMessage());
  }
}





function cpanel_CreateAccount($arg){

 try {      
            $server = $arg['server'];
            $client = $arg['client'];
            $product = $arg['product'];
            
            $domain = $product['domain'];
            $package = $product['package'];
            $username = $product['username'];
            $password = $product['password'];
            $email = $client['email'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  

            $acct = array('username'=> $username,'password' =>$password, 'domain'=>$domain,'plan'=>$package,'contactemail'=>$email,'useregns'=>0,'reseller'=>0);
            $res = json_decode($xmlapi->createacct($acct));
            $result = $res->result;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;

        if($statusmsg != 'Account Creation Ok'){
            logModule(
                'cpanel',
                __FUNCTION__,
                $arg,
                $res,
                $statusmsg
            );
        }


            return array('status'=>(($statusmsg == 'Account Creation Ok') ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
function cpanel_SuspendAccount($arg){
 try {      
            $server = $arg['server'];
            $client = $arg['client'];
            $product = $arg['product'];
            $reason = $arg['reason'];
            
            $domain = $product['domain'];
            $package = $product['package'];
            $username = $product['username'];
            $password = $product['password'];
            $email = $client['email'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->suspendacct($username,$reason));
            $result = $res->result;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;
            if($statusmsg == null) $statusmsg = $res->statusmsg;
            if($status == 1) 
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>"Account Suspended Successfully",'response'=>$res);
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
function cpanel_UnsuspendAccount($arg){

 try {      
            $server = $arg['server'];
            $client = $arg['client'];
            $product = $arg['product'];
            $reason = $arg['reason'];
            
            $domain = $product['domain'];
            $package = $product['package'];
            $username = $product['username'];
            $password = $product['password'];
            $email = $client['email'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->unsuspendacct($username));
            $result = $res->result;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;
            if($statusmsg == null) $statusmsg = $res->statusmsg;
            if($status == 1)
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>"Account UnSuspended Successfully",'response'=>$res);
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');

}
function cpanel_TerminateAccount($arg){

 try {      
            $server = $arg['server'];
            $product = $arg['product'];
            $keepdns = $arg['keepdns'];
            $username = $product['username'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->removeacct($username,(($keepdns == 'Yes') ? true:false)));
            $result = $res->result;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;
            if($statusmsg == null) $statusmsg = $res->statusmsg;
            if($status == 1) return array('status'=>(($status == 1) ? 'success':'error'),'message'=>"Account Terminated Successfully",'response'=>$res);
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');

}
function cpanel_Renew($arg){
    return ['status'=>'success','message'=>'Plan Renew Successfull'];
}
function cpanel_ChangePassword($arg){
 try {      

            $server = $arg['server'];
            $product = $arg['product'];         
            $username = $product['username'];
            $password = $product['password'];
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->passwd($username,$password));
            $result = $res->passwd;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;
            if($statusmsg == null) $statusmsg = $res->statusmsg;
            if($status == 1) return array('status'=>(($status == 1) ? 'success':'error'),'message'=>"Account Password Changed Successfully",'response'=>$res);
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');

}
function cpanel_ChangePackage($arg){

 try {      
            $server = $arg['server'];
            $client = $arg['client'];
            $product = $arg['product'];
            $reason = $arg['reason'];
            
            $domain = $product['domain'];
            $package = $product['package'];
            $username = $product['username'];
            $password = $product['password'];
            $email = $client['email'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $res = json_decode($xmlapi->changepackage($username,$package));
            $result = $res->result;
            $status = $result[0]->status;
            $statusmsg = $result[0]->statusmsg;
            if($statusmsg == null) $statusmsg = $res->statusmsg;
            if($status == 1) return array('status'=>(($status == 1) ? 'success':'error'),'message'=>"Account Plan Changed Successfully",'response'=>$res);
            return array('status'=>(($status == 1) ? 'success':'error'),'message'=>$statusmsg,'response'=>$res);
    } catch (Exception $e) {
        
        logModule(
            'cpanel',
            __FUNCTION__,
            $arg,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}




/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns an URL to which the user should be redirected.
 *
 * @param array $arg common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function cpanel_ServiceSingleSignOn($arg)
{
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];  

      if(empty($product)){
          $username = $server['username'];
          $password = '';
          $service = 'whostmgrd';
      }else{
        $producttype = $product['producttype'];
        if($producttype == 'reseller')
        {
            $service = 'whostmgrd';
        }else{
           $service = 'cpaneld';            
        }
      }
        
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $decoded_response = json_decode($xmlapi->singleSignOn($username,$service,$arg['app']),true);
            $reason = $decoded_response['metadata']['reason'];
            if(empty($decoded_response['data']))  return ["status"=>"error","message"=>$reason];
            $linkautologin = $decoded_response['data']['url'];
            if(empty($linkautologin)) return ["status"=>"error","message"=>$reason];
             $code = $linkautologin;
             $domain = "cpanel.".$product["domain"];
             $linkautologin = str_replace($server['ipaddress'],$domain,$linkautologin);
             $linkautologin = preg_replace('/(https?:\/\/)[^:]+/', '$1' .$domain, $linkautologin);
            return ["status"=>"success","message"=>"Login Session Created","url"=>$linkautologin];
  } catch (Exception $e) {
     return array('status'=>'error','message'=> $e->getMessage());
  }
}




/**
 * Perform single sign-on for a server.
 *
 * Called when single sign-on is requested for a server assigned to the module.
 *
 * This differs from ServiceSingleSignOn in that it relates to a server
 * instance within the admin area, as opposed to a single client instance of a
 * product/service.
 *
 * When successful, returns an URL to which the user should be redirected to.
 *
 * @param array $arg common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function cpanel_AdminSingleSignOn($arg) {
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      
      if(empty($product)){
          $username = $server['username'];
          $password = '';
          $service = 'whostmgrd';
      }else{
        $username = $product['username'];
        $password = $product['password'];
        $producttype = $product['producttype'];
        if($producttype == 'reseller'){
            $service = 'whostmgrd';
        }else{
           $service = 'cpaneld';            
        }
      }
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $decoded_response = json_decode($xmlapi->singleSignOn($username,$service),true);
            $reason = $decoded_response['metadata']['reason'];
            if(empty($decoded_response['data']))  return ["status"=>"error","message"=>$reason];
            $linkautologin = $decoded_response['data']['url'];
            if(empty($linkautologin)) return ["status"=>"error","message"=>$reason];
            if(!empty($product))
            {
             $code = $linkautologin;
            }else{
             $code = $linkautologin;
            }
            $code = str_replace($server['ipaddress'],$server["hostname"],$code);
            //$codeParts = explode(":",$code);
            //$code = "https://".$server["hostname"].$codeParts[1];
            return ["status"=>"success","message"=>"Login Session Created","url"=>$code];
  } catch (Exception $e) {
     return array('status'=>'error','message'=> $e->getMessage());
  }
}

    function cpanel_ListAccounts($arg)
    {
    try {
            $server = $arg['server'];
            $product = $arg['product'];
            $username = $product['username'];
            $password = $product['password'];

            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $result = json_decode($xmlapi->listaccts(),true);
            $accts = $result['acct'];
            
            $count = 0;
            $accounts = [];
            foreach ($accts as $user=>$values)
            {
                if ($values["owner"] != $server["username"] && $values["owner"] != $values["user"]) continue;
                $status = "Active";
                if ($values["suspended"]) $status = "Suspended";
                $plan = $values["plan"];

                $account[] = [
                    "name" => $values["user"],
                    "email" => $values["email"],
                    "username" => $values["user"],
                    "domain" => $values["domain"],
                    "uniqueIdentifier" => $values["domain"],
                    "product" => $plan,
                    "primaryip" => $values["ip"],
                    "created" =>date('Y-m-d',strtotime($values["unix_startdate"])),
                    "status" => $status
                ];
            }
            return ["status"=>"success","accounts" => $accounts];
        } 
        catch (Exception $e) 
        {
            return ["status"=>"error","message"=>$e->getMessage()];
        }
    }

    function cpanel_GetUserCount($arg)
    {
    try {
            $server = $arg['server'];
            $product = $arg['product'];
            $username = $product['username'];
            $password = $product['password'];

            //$postVars = array("want" => "user,owner");
            
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $result = json_decode($xmlapi->listaccts(),true);
            $accts = $result['acct'];
            $totalCount = 0;
            $ownedAccounts = 0;
            if(!empty($accts))
            {
                $totalCount = count($accts);                
                foreach ($accts as $user=>$values)
                {
                    if ($values["owner"] == $server["username"] || $values["owner"] == $values["user"]) $ownedAccounts++;
                }
            }            
            return ["status"=>"success","totalAccounts" => $totalCount, "ownedAccounts" => $ownedAccounts];
        } 
        catch (Exception $e) 
        {
            return ["status"=>"error","message"=>$e->getMessage()];
        }
    }

    function cpanel_GetRemoteMetaData($arg)
    {
    try {
            $server = $arg['server'];
            $product = $arg['product'];
            $username = $product['username'];
            $password = $product['password'];

            //$postVars = array("want" => "user,owner");

            $version = "-";
            $loads = ["fifteen" => "0", "five" => "0", "one" => "0"];
            $maxUsers = 0;

            
            require_once 'cpanel_class.php';
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $loadavg = json_decode($xmlapi->loadavg(),true);
            if(!empty($loadavg))
            {
                $loads['fifteen'] = $loadavg['fifteen'];
                $loads['five'] = $loadavg['five'];
                $loads['one'] = $loadavg['one'];
            }            

            $UserInfo = json_decode($xmlapi->maximumUsers(),true);
            $metadata = $UserInfo["metadata"];
            if(!empty($metadata) && $metadata["reason"] == "OK"){
               $maxUsers = $UserInfo["data"]["maximum_users"]; 
            }
            $versionInfo = json_decode($xmlapi->version(),true);
            $metadata = $versionInfo["metadata"];
            if(!empty($versionInfo) && $metadata["reason"] == "OK"){
               $version = $versionInfo["data"]["version"]; 
            }
            return ["status"=>"success","version" => $version, "load" => $loads, "max_accounts" => $maxUsers];
        } 
        catch (Exception $e) 
        {
            return ["status"=>"error","message"=>$e->getMessage()];
        }
    }

    function cpanel_RenderRemoteMetaData($arg)
    {
        $metaData = $arg["remoteData"];
        if ($metaData) {
            $version = "Unknown";
            $loadOne = $loadFive = $loadFifteen = 0;
            $maxAccounts = "Unlimited";
            if (array_key_exists("version", $metaData)) {
                $version = $metaData["version"];
            }
            if (array_key_exists("load", $metaData)) {
                $loadOne = $metaData["load"]["one"];
                $loadFive = $metaData["load"]["five"];
                $loadFifteen = $metaData["load"]["fifteen"];
            }
            if (array_key_exists("max_accounts", $metaData) && 0 < $metaData["max_accounts"]) {
                $maxAccounts = $metaData["max_accounts"];
            }
            return "cPanel Version: " . $version . "<br>\nLoad Averages: " . $loadOne . " " . $loadFive . " " . $loadFifteen . "<br>\nLicense Max # of Accounts: " . $maxAccounts;
        }
        return "";
    }


function cpanel_UsageUpdate($arg) {
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];
        require_once 'cpanel_class.php';
        $xmlapi = new cpanel_xml($server['ipaddress']);
        //$xmlapi->password_auth($svr_info->username,$svr_info->password);
        $xmlapi->hash_auth($server['username'],$server['accesshash']);
        $xmlapi->set_port($server['port']);
        $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
        $xmlapi->set_output("json");
        //$xmlapi->set_auth_type('hash');
        $xmlapi->set_debug(1);  
        $result = json_decode($xmlapi->listaccts(),true);
        $accts = $result['acct'];
  $count = 0;
    foreach ($accts AS $user=>$values) {
       $isUpdated =  update_query("hosting",array(
          "disk_usage"=>Ws_SizeConvertToMb($values['diskused']),
          "disk_limit"=>Ws_SizeConvertToMb($values['disklimit']),
          "email_accounts_limit"=>$values['maxpop'],
          "domains_limit"=>$values['maxaddons'],
          "sub_domains_limit"=>$values['maxsub'],
          "parked_domains_limit"=>$values['maxparked'],
          "databases_limit"=>$values['maxsql'],
          "databases_disk_limit"=>'',
          "databases_disk_usage"=>'',
          "databases_usage"=>'',
          "parked_domains_usage"=>'',
          "sub_domains_usage"=>'',
          "domains_usage"=>'',
          "email_accounts_usage"=>'',
          "last_update"=>Carbon::now()->format('Y-m-d h:i:s'),
        ),array("server_id"=>$server['id'], "user_name"=>$values['user']));
        if($isUpdated) $count++;
    }
            $xmlapi = new cpanel_xml($server['ipaddress']);
            //$xmlapi->password_auth($svr_info->username,$svr_info->password);
            $xmlapi->hash_auth($server['username'],$server['accesshash']);
            $xmlapi->set_port($server['port']);
            $xmlapi->set_protocol(str_replace('://','',$server['is_secure']));
            $xmlapi->set_output("json");
            //$xmlapi->set_auth_type('hash');
            $xmlapi->set_debug(1);  
            $result = json_decode($xmlapi->showbw(),true);
            $accts = $result['bandwidth'][0]['acct'];
    foreach ($accts AS $user=>$values) {
        $limits = $values['limit'];
        if(!is_numeric($limits)) $limits=-1;
       $isUpdated =  update_query("hosting",array(
          "bandwidth_limit"=>$limits/(1024*1024),
          "bandwidth_usage"=>$values['totalbytes']/(1024*1024),
          "last_update"=>Carbon::now()->format('Y-m-d h:i:s'),
        ),array("server_id"=>$server['id'], "user_name"=>$values['user']));
    }
    if($server['id'] > 0) 
    update_query('hosting_servers',['remote_acct_usage'=>$count,'up_status'=>'Up','last_check_date'=>Carbon::now()->format('Y-m-d h:i:s')],['id'=>$server['id']]);
     return array('status'=>'success','message'=>$count.' Accounts Stats Updated....');

  } catch (Exception $e) {
    update_query('hosting_servers',['up_status'=>'Down','last_check_date'=>Carbon::now()->format('Y-m-d h:i:s')],['id'=>$server['id']]);
         return $e->getMessage();
  }


/*

          "bandwidth_usage"=>$values['U_BANDWIDTH'],
          "bandwidth_limit"=>$values['BANDWIDTH'],
*/

}



function cpanel_ClientArea($arg){
    return array(
        'templatefile' => 'overview.tpl',
        'templatepath' => 'modules/servers/cpanel/templates/',
        'vars' => array(
            'test1' => 'hello',
            'test2' => 'world',
        ),
    );    
}


function cpanel_ClientAreaCustomButtonArray() {
//    ['Button Title'=>'button Custom Function name']
    return [];
}

function cpanel_AdminCustomButtonArray() {
//    ['Button Title'=>'button Custom Function name']
    return [];
}


?>