<?php
/***************************************************************************
// *                                                                       *
// * Centos Web Panel (CWP)  Provisioning (v1.7).                          *
// * This Provisioning module enables you integrate CWP with your WHMSOLS  *
//   and leverage its powerful features.                                   *
// * For assistance on how to use and setup CWP, visit                     *
//   @see https://dev.whmsols.com/provisioning-modules/parameters/         *
// *                                                                       *
// *************************************************************************
// *                                                                       *
// * Contributed by: WHMSOLS                                               *
// * Website: https://whmsols.com                                          *
// *                                                                       *
// *************************************************************************/

/******************************Change Logs***********************************
 * Service Single Signon Added                                              *
 * Admin Single Signon Added                                                *
 * Client Area Template Added                                               *
 * **************************************************************************/

use Carbon\Carbon;

function cwp_config()
{
    return array(
        "name" => "Centos Web Panel",
        "description" => "CWP Hosting control panel",
        "version" => "1.0",
        "author" => "contributed by WHMSOLS",
        "author_url"=>"",
        "RequiresServer"=>true,//true or false
        "DefaultNonSSLPort"=>'2304',
        "DefaultSSLPort"=>'2304',
        "ServiceSingleSignOnLabel"=>'',
        "AdminSingleSignOnLabel"=>'',
        "PackagesLoader"=>'PackagesLoader',
        "configs" => array(
        "SimpleMode" => array(
            'Name' => 'Simple Mode',
            'Type' => 'text',
            'Size' => '25',
            'SimpleMode' => true,
        ),
        "AdvancedMode"  => array(
            'Name' => 'Advance Mode',
            'Type' => 'text',
            'Size' => '25',
        ),
        "PACKAGE-NUMBER" => array( "Type" => "text", "Description" => "Package ID", "Default" => "1"),
        "inode" => array( "Type" => "text" , "Description" => "Max of inode", "Default" => "0",),
        "nofile" => array( "Type" => "text", "Description" => "Max of nofile", "Default" => "100", ),
        "nproc" => array( "Type" => "text" , "Description" => "Nproc limit - 40 suggested", "Default" => "40",),
        "autossl" => array( "Type" => "text" , "Description" => "Auto SSL installation 1=Yes,0=Not", "Default" => "1",),
        'comments' => array(
                            'Name' => 'Notes',
                            "Type" => "textarea", # Textarea
                            "Rows" => "3", # Number of Rows
                            "Cols" => "50", # Number of Columns
                            "Description" => "Description goes here",
                            "Default" => "Enter notes here",
            )
        )
    );
    
    return $configarray;
}

function cwp_curl_query($host,$port,$vars,$uri){
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://'.$host.':'.$port.'/v1/'.$uri);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt ($ch, CURLOPT_POSTFIELDS, http_build_query($vars));
    curl_setopt ($ch, CURLOPT_POST, 1);
    $response = curl_exec($ch);
    curl_close($ch);
    return json_decode($response, true);
}

function cwp_TestConnection($arg){
    $server = $arg['server'];
    $vars = array('key' => $server["accesshash"],'action' => 'list');
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'packages');
      return array('status'=>($result['status'] == 'OK') ? 'success':'error','message'=>'');
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

/**
 * PackagesLoader function that will populate the field in ConfigOptions
 * @return array The list of package names
 */
function cwp_PackagesLoader($arg) {
    $server = $arg['server'];
    $vars = array('key' => $server["accesshash"],'action' => 'list');
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'packages');
      $packageNames = $result['msj'];
        $list = [];
        foreach ($packageNames as $opArray) {
            $list[$opArray['id']] = ucfirst($opArray['package_name']);
        }
        return $list;
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}




function cwp_CreateAccount($arg) {
      $server = $arg['server'];
      $client = $arg['client'];
      $product = $arg['product'];
      
      $domain = $product['domain'];
      $package = $product['package'];
      $username = $product['username'];
      $password = $product['password'];
      $producttype = $product['producttype'];
      $email = $client['email'];
      $phonenumber = $client['phonenumber'];
      $firstname = $client['firstname'];
      $lastname = $client['lastname'];
    $module_configs = cwp_config();
    $module_configs = $module_configs['configs'];
    $autossl = $module_configs['autossl']['Default'];
    $vars = array(
            'package' => $package,
            'domain' => $domain,
            'key' => $server["accesshash"],
            'action' => 'add',
            'username' => $username,
            'user' => $username,
            'pass' => $password,
            'email' => $email,
            'inode' => $module_configs['inode']['Default'],
            'nofile' => $module_configs['nofile']['Default'],
            'nproc' => $module_configs["nproc"]['Default'],
            'autossl' => $autossl,//Autossl (0 = Not / 1 = Yes)
            'reseller' =>($producttype == 'reseller') ? 1:'', //(1 = To resell, Account Reseller for a Reseller's Package, Empty for Standard Package)
            'server_ips'=>$server["ipaddress"]
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'account');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Account Created Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}
function cwp_TerminateAccount($arg) {
      $server = $arg['server'];
      $client = $arg['client'];
      $product = $arg['product'];      
      $username = $product['username'];
      $email = $client['email'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'del',
            'user' => $username,
            'email' => $email
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'account');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Account Terminated Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}
function cwp_SuspendAccount($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $product['username'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'susp',
            'user' => $username
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'account');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Account Suspended Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}
function cwp_UnsuspendAccount($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $product['username'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'unsp',
            'user' => $username
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'account');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Account UnSuspended Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

function cwp_ChangePassword($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $product['username'];
      $password = $product['password'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'udp',
            'user' => $username,
            'pass' => $password
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'changepass');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Account Password Changed Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}
function cwp_ChangePackage($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $product['username'];
      $package = $product['package'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'udp',
            'user' => $username,
            'package' => $package
        );

 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'changepack');
        return array('status'=>($result["status"]=="OK") ? 'success':'error','message'=>(($result["status"]=="OK") ? 'Package Changed Successfully....':$result['msj']),'response'=>$result);
    } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}

function cwp_ClientArea($params)
{
    return array(
        'templatefile' => 'overview.tpl',
        'templatepath' => 'modules/servers/cwp/templates/',
        'vars' => array(
            'test1' => 'hello',
            'test2' => 'world',
        ),
    );
}


/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns an URL to which the user should be redirected.
 *
 * @param array $params common module parameters
 *
 * see https://dev.whmsols.com/provisioning-modules/parameters/
 *
 * @return array
 */
function cwp_ServiceSingleSignOn($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $product['username'];
      $package = $product['package'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'list',
            'user' => $username,
            'timer'=>5
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'user_session');
        $link=$result['msj']['details'];
        $linkautologin=$link[0]['url'];
        if(empty($linkautologin)) return array('status'=>'error','message'=>"Session Link not created By Server");
    return $linkautologin;

  } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
         return $e->getMessage();
  }
}


/**
 * function cwp_AdminSingleSignOn
 *
 * Generate a form to allow for an admin from WHMSOLS to access the admin account on CWP
 *
 * @param array required params An array of paramaters to pass to the Key,action,user method on CWP.
 */ 
function cwp_AdminSingleSignOn($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $server['username'];
      $package = $product['package'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'list',
            'user' =>$username,
            'timer'=>5
        );
 try {    
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'user_session');
        $link=$result['msj']['details'];
        $linkautologin=$link[0]['url'];
        if(empty($linkautologin)) return array('status'=>'error','message'=>"Server Session Link not created By CWP Server: ".$server['hostname']);
    return $linkautologin;

  } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
         return $e->getMessage();
  }
}



function cwp_UsageUpdate($arg) {
      $server = $arg['server'];
      $product = $arg['product'];      
      $username = $server['username'];
      $package = $product['package'];
    $vars = array(
            'key' => $server["accesshash"],
            'action' => 'list',
            'user' => $username,
            'timer'=>5
        );
 try {    
       
      $result = cwp_curl_query($server['hostname'],$server['port'],$vars,'account');
   if($result['status']=='OK'){
    $count = 0;
        $results=$result['msj'];
        foreach ($results AS $user=>$values) {
           $isUpdated =  update_query("hosting",array(
              "disk_usage"=>$values['diskusage'],
              "disk_limit"=>$values['disklimit'],
              "bandwidth_usage"=>$values['bandwidth'],
              "bandwidth_limit"=>$values['bwlimit'],
              "email_accounts_limit"=>'',
              "email_accounts_usage"=>'',
              "domains_limit"=>'',
              "domains_usage"=>'',
              "sub_domains_limit"=>'',
              "sub_domains_usage"=>'',
              "parked_domains_limit"=>'',
              "parked_domains_usage"=>'',
              "databases_limit"=>'',
              "databases_usage"=>'',
              "databases_disk_limit"=>'',
              "databases_disk_usage"=>'',
              "last_update"=>Carbon::now()->format('Y-m-d h:i:s'),
            ),array("server_id"=>$server['id'], "user_name"=>$values['username']));
            if($isUpdated) $count++;
        }   
        return array('status'=>'success','message'=>$count.' Accounts Stats Updated....');
    }      
    return array('status'=>'error','message'=>'Unable to Update Usage:'.$result["msj"]);
  } catch (Exception $e) {
        logModule('cwp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
         return array('status'=>'error','message'=>$e->getMessage());
  }
 return array('status'=>'error','message'=>'');
}

?>