<?php
/**
 *
 * CyberPanel whmsols module
 * @whmsols
 *
 */
class CyberApi
{   
    public function call($params,$post = array())
    {
        $serveraccesshash = $params["serveraccesshash"];
        $serverhostname = $params["serverhostname"];
        $serverport = $params["serverport"];
        if(empty($serverhostname)) return ["status"=>"error","message"=>"API Hostname Missing"];
        if(empty($serverport)) return ["status"=>"error","message"=>"API Port Missing"];
        if(empty($serveraccesshash)) return ["status"=>"error","message"=>"API Token Missing"];
        $sUrl = "https://".$params["serverhostname"].":". $params['serverport'] ."/cloudAPI/";
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => $sUrl,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => '',
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 0,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_SSL_VERIFYHOST => 0,
          CURLOPT_SSL_VERIFYPEER => 0,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => 'POST',
          CURLOPT_POSTFIELDS =>json_encode($post),
          CURLOPT_HTTPHEADER => array(
            'Authorization: '.$serveraccesshash,
            'Content-Type: application/json',
          ),
        ));
        $response = curl_exec($curl);
        if (curl_errno($curl)) return ["status"=>"error","message"=>curl_error($curl)];
        curl_close($curl);
        $response = json_decode($response,true);
        if($response["status"] == 1) return ["status"=>"success","data"=>@$response["data"]];
        if($response["websiteStatus"] == 1) return ["status"=>"success","data"=>@$response["data"]];
        if($response["createEmailStatus"] == 1) return ["status"=>"success","data"=>@$response["data"]];
        return ["status"=>"error","message"=>$response["error_message"]];
    }    
}

function cyberpanel_config()
{
    return array(
        'name' => 'CyberPanel',
        'description' => 'you can sell or automate basic functions of CyberPanel for your customers',
        'version' => '1.0',
        "author" => "Developed by WHMSOLS",
        "author_url"=>"",
        'RequiresServer' => true,
        'DefaultNonSSLPort' => '8090',
        'DefaultSSLPort' => '8090',
        'ServiceSingleSignOnLabel' => 'Login as User',
        'AdminSingleSignOnLabel' => 'Login as Admin',
        "PackagesLoader"=>'PackagesLoader',
        "configs" =>[
        'configoption1' => array(
            'Name' => 'Package Name',
            'Type' => 'text',
            'Default' => 'Default',
            'Description' => 'Enter package name for this product',
        ),
        'configoption2' => array(
            'Name' => 'ACL',
            'Type' => 'text',
            'Default' => 'user',
            'Description' => 'ACL to be assigned to the new user',
        )
    ]);
}

/**
 * PackagesLoader function that will populate the field in ConfigOptions
 * @return array The list of package names
 */
function cyberpanel_PackagesLoader($params,$type=true)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "fetchPackages",
        ];
        $response =  $api->call($params,$postParams);
        logModule(
            'cyberpanel',
            __FUNCTION__,
            $params,
            "Cyberpanel Connection Test started",
            ""
        );
        // Checking for errors
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule(
            'cyberpanel',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $list = [];
    $packages = json_decode($response["data"],true);
    foreach ($packages as $option)
    {
        if($type)
        {
            $list[$option["packageName"]] = ucfirst($option["packageName"]);
        }
        else
        {
            $list[$option["packageName"]] = $option;
        }
    }
    return $list;
}
function cyberpanel_GenerateRandomString($pwd_length=10){
   $letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
   $numbers = '0123456789';
  $chars = $letters.$numbers;

  $str = '';
  $max = strlen($chars) - 1;
  for ($i=0; $i < $pwd_length; $i++)
    $str .= $chars[random_int(0, $max)];
  return $str;
}
function cyberpanel_UserExist($params=[],$user="")
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "fetchUsers",
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel User Checking started","");
        // Checking for errors
        if($response["status"] == "error") return $response;
    } catch (Exception $e)
    {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $users = json_decode($response["data"],true);
    foreach ($users as $row)
    {
        if($row["userName"] == $user) return ["status"=>"exist"];
    }
    return ["status"=>"Not exist"];
}
function cyberpanel_GetWebsite($params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "fetchWebsites",
                "websiteName" => $params["domain"],
                "page" =>1,
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Fetching Websites started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $websites = json_decode($response["data"],true);
    if(!empty($websites)) return $websites;
    return ["status"=>"error","message"=>"Website not found after creation"];
}
function cyberpanel_GetDatabases($params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "fetchDatabases",
                "databaseWebsite" => $params["domain"],
                "page" =>1,
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Fetching Websites started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $websites = json_decode($response["data"],true);
    if(!empty($websites)) return $websites;
    return [];
}
function cyberpanel_GetEmails($params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "getEmailsForDomain",
                "domain" => $params["domain"]
        ];
        $response =  $api->call($params,$postParams);
        if($response["status"] == "error") return [];
    } catch (Exception $e) {
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $websites = json_decode($response["data"],true);
    if(!empty($websites)) return $websites;
    return [];
}
function cyberpanel_createEmailAcct($params)
{
    $custom = $params['custom'];
    $email_quota = 500;
    $email_prefix = $custom['email_prefix'];
    $email_pw = $custom['email_pw'];
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "submitEmailCreation",
                "domain" => $params["domain"],
                "username" =>$email_prefix,
                "passwordByPass" =>$email_pw
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Domain Email Creation started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"success","message"=>"Email Account Created Succesfully"];
}
function cyberpanel_deleteEmailAcct($params)
{
    $custom = $params['custom'];
    $email_quota = 500;
    $email_prefix = $custom['email_prefix'];
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "submitEmailDeletion",
                "email" =>$email_prefix,
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Domain Email Deletion started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"success","message"=>"Email Account Deleted Succesfully"];
}

function cyberpanel_CheckWebsite($params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "fetchWebsites",
                "websiteName" => $params["domain"],
                "page" =>1,
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Fetching Websites started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    $websites = json_decode($response["data"],true);
    if(!empty($websites)) return ["status"=>"success"];
    return ["status"=>"error","message"=>"Website not found after creation"];
}
function cyberpanel_CreateAccount(array $params)
{
    $packages = cyberpanel_PackagesLoader($params,false);
    $product = $params["product"];
    $package = $product["package"];
    $client = $params["client"];
    $allowedDomains = $packages[$package]["allowedDomains"];
    $password = cyberpanel_GenerateRandomString(10);
    $isExist = cyberpanel_UserExist($params,$params["username"]);
    if($isExist["status"] == "error") return $isExist;
    $userCreation = false;
    if($isExist["status"] != "exist") $userCreation=true;
    if($userCreation)
    {
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel User Creation started","");
    try {
            $api = new CyberApi();
            $postParams =[
                    "serverUserName" => $params["serverusername"],
                    "controller" => "submitUserCreation",
                    "firstName" =>$client["firstname"],
                    "lastName" =>$client["lastname"],
                    "email" =>$client["email"],
                    "userName" =>$params["username"],
                    "password" =>$password,
                    "websitesLimit" => $allowedDomains,
                    "selectedACL" => "user",
            ];
            $response =  $api->call($params,$postParams);
            $status = $response["status"];
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel User Creation Failed",$response);
            if($status == "error") return $response;
            update_query("hosting",["password"=>$password],["id"=>$product["serviceid"]]);
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel User Creation Completed","");
        } catch (Exception $e) {
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel User Creation Failed",$e->getMessage());
            return ['status'=>'error','message'=>$e->getMessage()];
        }
    }
    
    $isWebsiteCreated = cyberpanel_CheckWebsite($params);
    $isWebExist = false;
    if($isWebsiteCreated["status"] == "success") $isWebExist=true;
    if(!$isWebExist)
    {
        try {
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Website Creation started","");
            $api = new CyberApi();
            $postParams =[
                    "serverUserName" => $params["serverusername"],
                    "controller" => "submitWebsiteCreation",
                    "domainName" =>$params["domain"],
                    "package" =>$package,
                    "adminEmail" =>$client["email"],
                    "phpSelection" =>"PHP 7.4",
                    "websiteOwner" =>$params["username"],
                    "ssl" =>0,
                    "dkimCheck" => 0,
                    "openBasedir" => 0,
            ];
            $response =  $api->call($params,$postParams);
            $status = $response["status"];
            if($status == "success")
            {
                logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Website Creation Completed","");
                return array('status'=>'success','message'=>"Account and Web Created Successfully",'response'=>$response); 
            }
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Website Creation Failed",$response);
            return $response;
        } catch (Exception $e) {
            logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Website Creation Failed",$e->getMessage());
            return ['status'=>'error','message'=>$e->getMessage()];
        }
    }
    if($isWebExist) return array('status'=>'success','message'=>"Account and Web Created Successfully",'response'=>$response);
}
function cyberpanel_SuspendAccount(array $params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "submitWebsiteStatus",
                "websiteName" => $params["domain"],
                "state"=>"Suspend"
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Website Suspension started","");
        if($response["status"] == "success")
        {
            logModule('cyberpanel',__FUNCTION__,$params,"Website Suspended Successfully",$response);
            return ["status"=>"success","message"=>"Website Suspended Successfully","response"=>$response];
        }
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"error","message"=>"Website Suspension failed","response"=>$response];
}
function cyberpanel_UnsuspendAccount(array $params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "submitWebsiteStatus",
                "websiteName" => $params["domain"],
                "state"=>"Unsuspend"
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Website UnSuspension started","");
        if($response["status"] == "success")
        {
            logModule('cyberpanel',__FUNCTION__,$params,"Website UnSuspended Successfully",$response);
            return ["status"=>"success","message"=>"Website UnSuspended Successfully","response"=>$response];
        }
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"error","message"=>"Website UnSuspension failed","response"=>$response];
}
function cyberpanel_TerminateAccount(array $params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "submitWebsiteDeletion",
                "websiteName" => $params["domain"]
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Website Deletion started","");
        if($response["status"] == "success")
        {
            logModule('cyberpanel',__FUNCTION__,$params,"Website Deleted Successfully",$response);
            return ["status"=>"success","message"=>"Website Deleted Successfully","response"=>$response];
        }
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"error","message"=>"Website Deletion failed","response"=>$response];
}
function cyberpanel_ChangePassword(array $params)
{
    $product = $params["product"];
    $password = $params["password"];
    //$password = cyberpanel_GenerateRandomString(10);
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "ChangeLinuxUserPassword",
                "domain" => $params["domain"],
                "password" =>$password
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Account Password Changes started","");
        if($response["status"] == "success")
        {
            update_query("hosting",["password"=>$password],["id"=>$product["serviceid"]]);
            logModule('cyberpanel',__FUNCTION__,$params,"Account Password Changed Successfully",$response);
            return ["status"=>"success","message"=>"Account Password Changed Successfully","response"=>$response];
        }
    } catch (Exception $e) {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ["status"=>"error","message"=>"Account Password Changes failed","response"=>$response];
}
/*
function cyberpanel_ChangePackage(array $params)
{
    return ['status'=>'error','message'=>'Cyberpanel api have not method to change account package so plz, Update manually'];
}
*/
function cyberpanel_TestConnection(array $params)
{
    try {
        $api = new CyberApi();
        $postParams =[
                "serverUserName" => $params["serverusername"],
                "controller" => "verifyLogin",
        ];
        $response =  $api->call($params,$postParams);
        logModule('cyberpanel',__FUNCTION__,$params,"Cyberpanel Connection Test started","");
        if($response["status"] == "error") return $response;
    } catch (Exception $e)
    {
        logModule('cyberpanel',__FUNCTION__,$params,$e->getMessage(),$e->getTraceAsString());        
        return ['status'=>'error','message'=>$e->getMessage()];
    }
    return ['status'=>'success','message'=>''];
}
function cyberpanel_ClientAreaCustomButtonArray($params) {
//    ['Button Title'=>'button Custom Function name']
    $domain = $params["domain"];
    $url = $params["serversecure"].$domain.":".$params["serverport"];
    return ["Login To Cyberpanel"=>$url];
}
function cyberpanel_ClientArea($params)
{
    $password = $params["password"];
    $username = $params["username"];
    $domain = $params["domain"];
    $Website = cyberpanel_GetWebsite($params);
    $Databases = cyberpanel_GetDatabases($params);
    $Emails = cyberpanel_GetEmails($params);
    return array(
        'templatefile' => 'overview.tpl',
        'templatepath' => 'modules/servers/cyberpanel/templates/',
        'vars' => array(
            'serversecure' => $params["serversecure"],
            'serverhostname' => $params["serverhostname"],
            'serverport' => $params["serverport"],
            'domain' => $domain,
            'username' => $username,
            'password' => $password,
            'cpstate' => $Website[0],
            'cpdatabases' =>count($Databases),
            'cpemails' =>count($Emails),
            'cpemaillist' =>$Emails,
        ),
    );    
}
function cyberpanel_AdminSingleSignOn($params)
{
    $server = $arg['server'];
    $product = $arg['product'];
    return ["status"=>"success","message"=>"Login Session Created","url"=>$params["serversecure"].$params["serverhostname"].":".$params['serverport']."/api/loginAPI"];
}