<?php
use Carbon\Carbon;
function directadmin_info()
{
    return ["name" => "DirectAdmin", "version" => "1.1", "DefaultNonSSLPort" => "2222", "DefaultSSLPort" => "2222", "ListAccountsUniqueIdentifierDisplayName" => "Domain", "ListAccountsUniqueIdentifierField" => "domain", "ListAccountsProductField" => "configoption1"];
}
function directadmin_config($params=[])
{
    $resellerSimpleMode = _s("type",$params,"reseller");
    $configs =  ["configoption1" => ["Name"=>"Package Name","Type" => "text", "Size" => "25", "Loader" => function ($params) {
        $return = [];
        if ($resellerSimpleMode) {
            $command = "CMD_API_PACKAGES_RESELLER";
        } else {
            $command = "CMD_API_PACKAGES_USER";
        }
        $result = directadmin_req($command, [], $params);
        if ($result["error"] && $result["details"]) {
            throw new Exception($result["details"]);
        }
        if (isset($result["list"])) {
            foreach ($result["list"] as $package) {
                $return[$package] = ucwords(str_replace("_", " ", $package));
            }
        }
        return $return;
    }, "SimpleMode" => true],
    "configoption2" => ["Name"=>"Reseller IP","Type" => "dropdown", "Options" => ",shared,sharedreseller,assign", "SimpleMode" => $resellerSimpleMode],
    "configoption3" => ["Name"=>"Dedicated IP","Type" => "yesno", "Description" => "Tick to Auto-Assign Dedicated IP"],
    "configoption4" => ["Name"=>"Suspend at Limit","Type" => "yesno", "Description" => "Tick to Auto Suspend Users when reaching Bandwidth Limit"]];
    
    $info = directadmin_info();
    $info["configs"]=$configs;
    return $info;
}
function directadmin_ClientArea($params)
{
    $host = $params["serverhostname"] ? $params["serverhostname"] : $params["serverip"];
    $form = sprintf("<form action=\"%s://%s:%s/CMD_LOGIN\" method=\"post\" target=\"_blank\"><input type=\"hidden\" name=\"username\" value=\"%s\" /><input type=\"hidden\" name=\"password\" value=\"%s\" /><input type=\"submit\" value=\"%s\" class=\"button\" /></form>", $params["serverhttpprefix"],$host,$params["serverport"],$params["username"],$params["password"],"Panel Login");
    return $form;
}
function directadmin_AdminLink($params)
{
    $host = $params["serverhostname"] ? $params["serverhostname"] : $params["serverip"];
    $form = sprintf("<form action=\"%s://%s:%s/CMD_LOGIN\" method=\"post\" target=\"_blank\"><input type=\"hidden\" name=\"username\" value=\"%s\" /><input type=\"hidden\" name=\"password\" value=\"%s\" /><input type=\"submit\" value=\"%s\" /></form>", $params["serverhttpprefix"],$host,$params["serverport"],$params["serverusername"],$params["serverpassword"], "DirectAdmin");
    return $form;
}
function directadmin_CreateAccount($params){
    $fields = [];
    $ip = $params["serverip"];
    if ($params["configoption3"] == "yes")
    {
        $command = "CMD_API_SHOW_RESELLER_IPS";
        $params["getip"] = true;
        $fields["action"] = "all";
        $results = directadmin_req($command, $fields, $params);
        foreach ($results as $ipaddress => $details) {
            if ($details["status"] === "free")
            {
                UpdateServiceInfo(["dedicated_ip" => $ipaddress],["id"=>$params["serviceid"]]);
            }
        }
    }
    $params["getip"] = "";
    if ($params["type"] === "hosting") {
        $fields["action"] = "create";
        $fields["add"] = "Submit";
        $fields["username"] = $params["username"];
        $fields["email"] = $params["clientsdetails"]["email"];
        $fields["passwd"] = $params["password"];
        $fields["passwd2"] = $params["password"];
        $fields["domain"] = $params["domain"];
        $fields["package"] = $params["configoption1"];
        $fields["ip"] = $ip;
        $fields["notify"] = "no";
        $command = "CMD_API_ACCOUNT_USER";
    } else {
        $fields["action"] = "create";
        $fields["add"] = "Submit";
        $fields["username"] = $params["username"];
        $fields["email"] = $params["clientsdetails"]["email"];
        $fields["passwd"] = $params["password"];
        $fields["passwd2"] = $params["password"];
        $fields["domain"] = $params["domain"];
        $fields["package"] = $params["configoption1"];
        if ($params["configoption2"] === "sharedreseller") {
            $fields["ip"] = "sharedreseller";
        } else {
            if ($params["configoption2"] === "assign")
            {
                $fields["ip"] = "assign";
            } else {
                $fields["ip"] = "shared";
            }
        }
        $fields["notify"] = "no";
        $command = "CMD_API_ACCOUNT_RESELLER";
    }
    $results = directadmin_req($command, $fields, $params);
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Account Created Successfully"];
}
function directadmin_TerminateAccount($params){
    $fields = [];
    $fields["confirmed"] = "Confirm";
    $fields["delete"] = "yes";
    $fields["select0"] = $params["username"];
    $results = directadmin_req("CMD_SELECT_USERS", $fields, $params);
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Account Terminated Successfully"];
}
function directadmin_SuspendAccount($params){
    $fields = [];
    $fields["action"] = "create";
    $fields["add"] = "Submit";
    $fields["user"] = $params["username"];
    $results = directadmin_req("CMD_API_SHOW_USER_CONFIG", $fields, $params);
    if ($results["suspended"] === "yes") return ["status"=>"success","message"=>"Account already Suspended"];
    $fields = [];
    $fields["suspend"] = "Suspend/Unsuspend";
    $fields["select0"] = $params["username"];
    $results = directadmin_req("CMD_SELECT_USERS", $fields, $params);
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Account Suspended Successfully"];
}
function directadmin_UnsuspendAccount($params){
    $fields = [];
    $fields["action"] = "create";
    $fields["add"] = "Submit";
    $fields["user"] = $params["username"];
    $results = directadmin_req("CMD_API_SHOW_USER_CONFIG", $fields, $params);
    if ($results["suspended"] === "no") return ["status"=>"success","message"=>"Account already UnSuspended"];
    $fields = [];
    $fields["suspend"] = "Suspend/Unsuspend";
    $fields["select0"] = $params["username"];
    $results = directadmin_req("CMD_SELECT_USERS", $fields, $params);
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Account UnSuspended Successfully"];
}
function directadmin_ChangePassword($params){
    $fields = [];
    $fields["username"] = $params["username"];
    $fields["passwd"] = $params["password"];
    $fields["passwd2"] = $params["password"];
    $results = directadmin_req("CMD_API_USER_PASSWD", $fields, $params, true);
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Account password Changed Successfully"];
}
function directadmin_ChangePackage($params){
    $fields = [];
    $fields["action"] = "package";
    $fields["user"] = $params["username"];
    $fields["package"] = $params["configoption1"];
    if ($params["type"] === "reseller") 
    {
        $results = directadmin_req("CMD_API_MODIFY_RESELLER", $fields, $params);
    }
    else
    {
        $results = directadmin_req("CMD_API_MODIFY_USER", $fields, $params);
    }
    if ($results["error"]) return ["status"=>"error","message"=>$results["details"]];
    return ["status"=>"success","message"=>"Package Changed Successfully"];
}
function directadmin_UsageUpdate($params){
    $count = 0;
    $serverid = $params["serverid"];
    $serverUsername = $params["serverusername"];
    $services = full_query("select a.id,a.user_name,p.account_type as type from hosting a join products_list p on a.package_id=p.id where a.server_id=".$serverid." and a.status in ('Active','Suspended')");
    foreach ($services as $service)
    {
        $username = $service->user_name;
        $fields = ["user" => $username];
        $command1 = "CMD_API_SHOW_USER_USAGE";
        $command2 = "CMD_API_SHOW_USER_CONFIG";
        $fields1 = $fields;
        $fields2 = $fields;
        if ($service->type === "reseller")
        {
            $params["serverusername"] = $serverUsername . "|" . $username;
            $command1 = "CMD_API_RESELLER_STATS";
            $fields1["type"] = "usage";
            $command2 = "CMD_API_RESELLER_STATS";
        }
        $results = directadmin_req($command1, $fields1, $params);
        if (!$results["error"])
        {
            $quota = urldecode($results["quota"]);
            $bandwidth = urldecode($results["bandwidth"]);
            $diskUsed = round($quota);
            $bwUsed = round($bandwidth);
            $results = directadmin_req($command2, $fields2, $params);
            if (!$results["error"])
            {
                $quota = urldecode($results["quota"]);
                $bandwidth = urldecode($results["bandwidth"]);
                $diskLimit = $quota == "unlimited" ? "0" : round($quota);
                $bwLimit = $bandwidth == "unlimited" ? "0" : round($bandwidth);
                $isUpdated =  UpdateServiceInfo([
                "disk_usage" =>$diskUsed,
                "disk_limit" =>$diskLimit,
                "bandwidth_usage" =>$bwUsed,
                "bandwidth_limit" =>$bwLimit,
                "email_accounts_limit"=>0,
                "domains_limit"=>0,
                "sub_domains_limit"=>0,
                "parked_domains_limit"=>0,
                "databases_limit"=>'',
                "databases_disk_limit"=>'',
                "databases_disk_usage"=>'',
                "databases_usage"=>'',
                "parked_domains_usage"=>'',
                "sub_domains_usage"=>'',
                "domains_usage"=>'',
                "email_accounts_usage"=>'',
                "last_update"=>Carbon::now()->format('Y-m-d h:i:s')],["id"=>$service->id]);
                if($isUpdated) $count++;
            }
        }
    }
    if($serverid > 0)
    {
        UpdateServerInfo(['remote_acct_usage'=>$count,'up_status'=>'Up','last_check_date'=>Carbon::now()->format('Y-m-d h:i:s')],['id'=>$serverid]);
    }
    if($count>0) return array('status'=>'success','message'=>$count.' Accounts Stats Updated....');
    return ["status"=>"error","message"=>"Account Stats Updation failed"];
}
function directadmin_req($command, $fields, $params, $post = false){
    $host = $params["serverhostname"] ? $params["serverhostname"] : $params["serverip"];
    $user = $params["serverusername"];
    $pass = $params["serverpassword"];
    $httpprefix = $params["serverhttpprefix"];
    $port = $params["serverport"];
    $resultsarray = [];
    if (!$user || !$pass) {
        $resultsarray = ["error" => "1", "details" => "Login Details Missing"];
        return $resultsarray;
    }
    $fieldstring = "";
    foreach ($fields as $key => $value) {
        $fieldstring .= $key . "=" . urlencode($value) . "&";
    }
    $url = $httpprefix . "://" . $host . ":" . $port . "/" . $command;
    if (!$post) {
        $url .= "?" . $fieldstring;
    }
    $authstr = $user . ":" . $pass;
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_URL, $url);
    if ($post) {
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fieldstring);
    }
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    $curlheaders[0] = "Authorization: Basic " . base64_encode($authstr);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $curlheaders);
    curl_setopt($ch, CURLOPT_TIMEOUT, 300);
    $data = curl_exec($ch);
    if (curl_errno($ch)) {
        $resultsarray["error"] = true;
        $resultsarray["details"] = "Connection Error: " . curl_errno($ch) . " - " . curl_error($ch);
        $data = curl_errno($ch) . " - " . curl_error($ch);
    }
    curl_close($ch);
    $arrayReturnPackages = ["CMD_API_PACKAGES_RESELLER", "CMD_API_PACKAGES_USER", "CMD_API_ADDITIONAL_DOMAINS", "CMD_API_SHOW_ALL_USERS", "CMD_API_SHOW_USERS", "CMD_API_SHOW_RESELLERS"];
    if (!$resultsarray["error"]) {
        if (strpos($data, "DirectAdmin Login") === true || strpos($data, "AUTH_METHOD: \"CMD_LOGIN\"") === true) {
            $resultsarray = ["error" => "1", "details" => "Login Failed"];
        } else {
            if (strpos($data, "Your IP is blacklisted") !== false) {
                $resultsarray = ["error" => "1", "details" => "WHMSOLS Host Server IP is Blacklisted"];
            } else {
                if (strtolower(trim($data)) === "use https") {
                    $resultsarray = ["error" => "1", "details" => "Secure Connection Is Required"];
                } else {
                    if ($params["getip"]) {
                        $data2 = directadmin_unhtmlentities($data);
                        parse_str($data2, $output);
                        foreach ($output as $key => $value) {
                            $key = str_replace("_", ".", urldecode($key));
                            $value = explode("&", urldecode($value));
                            foreach ($value as $temp) {
                                $temp = explode("=", $temp);
                                $resultsarray[urldecode($key)][$temp[0]] = $temp[1];
                            }
                        }
                    } else {
                        if (in_array($command, $arrayReturnPackages)) {
                            $data2 = directadmin_unhtmlentities($data);
                            parse_str($data2, $resultsarray);
                        } else {
                            if ($fields["json"] === "yes") {
                                $data2 = directadmin_unhtmlentities($data);
                                $resultsarray = json_decode($data2, true);
                            } else {
                                $data = explode("&", $data);
                                foreach ($data as $temp) {
                                    $temp = explode("=", $temp);
                                    $temp[0] = urldecode($temp[0]);
                                    $temp[1] = urldecode($temp[1]);
                                    $resultsarray[$temp[0]] = $temp[1];
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    logModuleCall("directadmin", $command, $url, $data, $resultsarray);
    return $resultsarray;
}
function directadmin_unhtmlentities($string){
    return preg_replace_callback("~&#([0-9][0-9])~", function ($match) {
        return chr($match[1]);
    }, $string);
}
function directadmin_TestConnection($params){
    $response = directadmin_req("CMD_API_SHOW_USERS", [], $params);
    if (array_key_exists("error", $response) && $response["error"] == "1") return ["status"=>"error","message"=>$response["details"]];
    return ["status"=>"success"];
}
function directadmin_ListAccounts($params){
    $commands = ["CMD_API_SHOW_USERS", "CMD_API_SHOW_RESELLERS"];
    $accounts = [];
    foreach ($commands as $command)
    {
        $response = directadmin_req($command, [], $params, true);
        if (array_key_exists("error", $response) && $response["error"] === "1") {
            if ($response["text"] !== "You cannot execute that command") return ["status"=>"error","message"=> "Unable to Obtain Account List - " . $response["details"]];
        } else
        {
            if (array_key_exists("list", $response))
            {
                foreach ($response["list"] as $user)
                {
                    $command2 = "CMD_API_SHOW_USER_CONFIG";
                    $response2 = directadmin_req($command2, ["user" => $user], $params);
                    if (array_key_exists("error", $response2) && $response2["error"] === "1") return ["error" => "Unable to Obtain Account List - " . $response2["details"]];
                    $status = "Active";
                    if ($response2["suspended"] !== "no") $status = "Suspended";
                    $accounts[] = [
                        "name" => $response2["username"],
                        "email" => $response2["email"],
                        "username" => $response2["username"],
                        "domain" =>$response2["domain"],
                        "uniqueIdentifier" =>$response2["domain"],
                        "product" =>$response2["package"],
                        "primaryip" =>$response2["ip"],
                        "created" =>date('Y-m-d',strtotime($response2["date_created"])),
                        "status" => $status
                    ];
                }
            }
        }
    }
    return ["status"=>"success", "accounts" => $accounts];
}
function directadmin_GetUserCount($params){
    $commands = ["CMD_API_SHOW_USERS", "CMD_API_SHOW_RESELLERS"];
    $totalCount = 0;
    $ownedAccounts = 0;
    foreach ($commands as $command) {
        $response = directadmin_req($command, [], $params, true);
        if (array_key_exists("error", $response) && $response["error"] === "1") {
            if ($response["text"] !== "You cannot execute that command") {
                return ["error" => "Unable to Obtain Account List - " . $response["details"]];
            }
        } else {
            if (array_key_exists("list", $response)) {
                $totalCount += count($response["list"]);
                $ownedAccounts += count($response["list"]);
                if ($command == "CMD_API_SHOW_RESELLERS") {
                    foreach ($response["list"] as $reseller) {
                        $response2 = directadmin_req("CMD_API_SHOW_USERS", ["reseller" => $reseller], $params, true);
                        if (array_key_exists("list", $response2)) {
                            $totalCount += count($response2["list"]);
                        }
                    }
                }
            }
        }
    }
    return ["success" => true, "totalAccounts" => $totalCount, "ownedAccounts" => $ownedAccounts];
}
function directadmin_GetRemoteMetaData($params){
    $version = "-";
    try {
        $loads = [];
        $maxUsers = 0;
        $response = directadmin_req("CMD_API_SYSTEM_INFO", [], $params, true);
        if (array_key_exists("directadmin", $response)) {
            $version = explode("|", $response["directadmin"]);
            $version = $version[0];
        }
        return ["status"=>"success","version" => $version, "load" => $loads, "max_accounts" => $maxUsers];
    } catch (Exception $e) {
        return ["status" =>"error", "message" => $e->getMessage()];
    }
}
function directadmin_RenderRemoteMetaData($params){
    $metaData = $arg["remoteData"];
    if ($metaData) {
        $version = "Unknown";
        $loadOne = $loadFive = $loadFifteen = 0;
        $maxAccounts = "Unlimited";
        if (array_key_exists("version", $metaData)) {
            $version = $metaData["version"];
        }
        if (array_key_exists("load", $metaData)) {
            $loadOne = $metaData["load"]["one"];
            $loadFive = $metaData["load"]["five"];
            $loadFifteen = $metaData["load"]["fifteen"];
        }
        if (array_key_exists("max_accounts", $metaData) && 0 < $metaData["max_accounts"]) {
            $maxAccounts = $metaData["max_accounts"];
        }
        return "DirectAdmin Version: " . $version . "<br>\nLoad Averages: " . $loadOne . " " . $loadFive . " " . $loadFifteen . "<br>\nLicense Max # of Accounts: " . $maxAccounts;
    }
    return "";
}