<?php
use Carbon\Carbon;
/*******************************************************************************
// *                                                                           *
// * VestaCP  Provisioning (v2.0).                                             *
// * This Provisioning module enables you integrate VestaCP with your WHMSOLS  *
//   and leverage its powerful features.                                       *
// * For assistance on how to use and setup VestaCP, visit                     *
//   @see https://dev.whmsols.com/provisioning-modules/parameters/             *
// *                                                                           *
// *****************************************************************************
// *                                                                           *
// * Contributed by: WHMSOLS                                                 *
// * Website: https://whmsols.com                                            *
// *                                                                           *
// ****************************************************************************/

function vestacp_config() {
  return array(
    "name" => "Vesta Cp",
    "description" => "VestaCp Hosting control panel",
    "version" => "1.0",
    "author" => "contributed by WHMSOLS",
    "author_url"=>"",
    "RequiresServer"=>true,//true or false
    "DefaultNonSSLPort"=>'',
    "DefaultSSLPort"=>'8083',
    "ServiceSingleSignOnLabel"=>'',
    "AdminSingleSignOnLabel"=>'',
    "PackagesLoader"=>'PackagesLoader',
    "configs" => array(
    "SimpleMode" => array(
      'Name' => 'Simple Mode',
            'Type' => 'text',
            'Size' => '25',
            'SimpleMode' => true,
        ),
        "AdvancedMode"  => array(
      'Name' => 'Advance Mode',
            'Type' => 'text',
            'Size' => '25',
        ),
    'comments' => array(
              'Name' => 'Notes',
                    "Type" => "textarea", # Textarea
                    "Rows" => "3", # Number of Rows
                    "Cols" => "50", # Number of Columns
                    "Description" => "Description goes here",
                    "Default" => "Enter notes here",
            )
    )
  );
  
  return $configarray;
}


function vestacp_curl_query($host,$port,$vars){
  $postVars = http_build_query($vars);
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, 'https://'.$host.':'.$port.'/api/');
    curl_setopt($curl, CURLOPT_RETURNTRANSFER,true);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($curl, CURLOPT_POST, true);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $postVars);
    $answer = curl_exec($curl);
    // Decode json data

    return json_decode($answer, true);
}

function vestacp_commands_text($code=0){
  $array = array(
  0=>array('code'=>'OK','message'=>'Command has been successfuly performed'),
  1=>array('code'=>'E_ARGS','message'=>'Not enough arguments provided'),
  2=>array('code'=>'E_INVALID','message'=>'Arguments is not valid'),
  3=>array('code'=>'E_NOTEXIST','message'=>'Account doesn\'t exist'),
  4=>array('code'=>'E_EXISTS','message'=>'Account already exists'),
  5=>array('code'=>'E_SUSPENDED','message'=>'Account is already suspended'),
  6=>array('code'=>'E_UNSUSPENDED','message'=>'Account is already unsuspended'),
  7=>array('code'=>'E_INUSE','message'=>'Account can\'t be deleted because is used by the other object'),
  8=>array('code'=>'E_LIMIT','message'=>'Account cannot be created because of hosting package limits'),
  9=>array('code'=>'E_PASSWORD','message'=>'Wrong password'),
  10=>array('code'=>'E_FORBIDEN','message'=>'Account cannot be accessed be the user'),
  11=>array('code'=>'E_DISABLED','message'=>'Subsystem is disabled'),
  12=>array('code'=>'E_PARSING','message'=>'Configuration is broken'),
  13=>array('code'=>'E_DISK','message'=>'Not enough disk space to complete the action'),
  14=>array('code'=>'E_LA','message'=>'Server is to busy to complete the action'),
  15=>array('code'=>'E_CONNECT','message'=>'Connection failed. Host is unreachable'),
  16=>array('code'=>'E_FTP','message'=>'FTP server is not responding'),
  17=>array('code'=>'E_DB','message'=>'Database server is not responding'),
  18=>array('code'=>'E_RRD','message'=>'RRDtool failed to update the database'),
  19=>array('code'=>'E_UPDATE','message'=>'Update operation failed'),
  20=>array('code'=>'E_RESTART','message'=>'Service restart failed'));
  return $array[$code]['message'];
}
function vestacp_TestConnection($arg){
    $server = $arg['server'];

    // Prepare variables
    $vars = array(
      'user' => $server["username"],
      'password' => $server["password"],
      //'hash' => $server["accesshash"],
      'cmd' => 'v-list-user-packages',
      'arg1' => 'json'
    );

 try {    
      $result = vestacp_curl_query($server['hostname'],$server['port'],$vars);
    } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'success','message'=>'');
}


/**
 * PackagesLoader function that will populate the field in ConfigOptions
 * @return array The list of package names
 */
function vestacp_PackagesLoader($arg) {
    $server = $arg['server'];

    // Prepare variables
    $vars = array(
      'user' => $server["username"],
      'password' => $server["password"],
      //'hash' => $server["accesshash"],
      'cmd' => 'v-list-user-packages',
      'arg1' => 'json'
    );
 try {    
      $packageNames = vestacp_curl_query($server['hostname'],$server['port'],$vars);
        $list = [];
        foreach ($packageNames as $opKey =>$opValue) {
            $list[$opKey] = ucfirst($opKey);
        }
        return $list;
    } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
        return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}


/**
 * function vestacp_CreateAccount
 *
 * Create a new user account on a server managed with VestaCP. 
 * This method also sets up ssh access and domain.
 *
 * @param array required params An array of paramaters to pass to the v-add-user method on VestaCP.
 */ 
function vestacp_CreateAccount($arg) {
      $server = $arg['server'];
      $client = $arg['client'];
      $product = $arg['product'];
      
      $domain = $product['domain'];
      $package = $product['package'];
      $username = $product['username'];
      $password = $product['password'];
      $email = $client['email'];
      $phonenumber = $client['phonenumber'];
      $firstname = $client['firstname'];
      $lastname = $client['lastname'];

    // Execute only if there is assigned server
  

        // Prepare variables
        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-add-user',
          'arg1' => $username,
          'arg2' => $password,
          'arg3' => $email,
          'arg4' => $package,
          'arg5' => $firstname,
          'arg6' => $lastname,
        );
   try {            
      $createAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
        // Add domain
        if(($createAccount == 0) && (!empty($domain))) {
            $vars = array(
              'user' => $server["username"],
              'password' => $server["password"],
              //'hash' => $server["accesshash"],
              'returncode' => 'yes',
              'cmd' => 'v-add-domain',
              'arg1' => $username,
              'arg2' => $domain,
            );
             $createAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
            //logModule('vesta','CreateAccount_AddDomain','https://'.$params["serverhostname"].':8083/api/'.$postdata,$answer);
        }
   return array('status'=>(($createAccount == 0) ? 'success':'error'),'message'=>($createAccount == 0) ? 'User account has been successfuly created' : vestacp_commands_text($createAccount),'response'=>vestacp_commands_text($createAccount));

} catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');

}
/**
 * function vestacp_TerminateAccount
 *
 * Terminate a user account on a server managed with VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-delete-user method on VestaCP.
 */ 
function vestacp_TerminateAccount($arg) {
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];

        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-delete-user',
          'arg1' => $username
        );
        

      $deleteAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
      $message = ($deleteAccount == 0) ? "User account has been successfuly deleted\n":"Query returned error code: " .$deleteAccount. "\n";
  return array('status'=>(($deleteAccount == 0) ? 'success':'error'),'message'=>$message,'response'=>vestacp_commands_text($deleteAccount));
    // Execute only if there is assigned server  
  } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');

}
/**
 * function vestacp_SuspendAccount
 *
 * Suspend a user account on a server managed with VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-suspend-user method on VestaCP.
 */ 
function vestacp_SuspendAccount($arg) {
    $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];

        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-suspend-user',
          'arg1' => $username
        );
        
 try {      
      $suspendAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
      $message = ($suspendAccount == 0) ? "User account has been Suspended successfuly\n":"Query returned error code: " .vestacp_commands_text($suspendAccount). "\n";
      return array('status'=>(($suspendAccount == 0) ? 'success':'error'),'message'=>$message,'response'=>vestacp_commands_text($suspendAccount));
      } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
/**
 * function vestacp_UnsuspendAccount
 *
 * Un-Suspend a user account on a server managed with VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-unsuspend-user method on VestaCP.
 */ 
function vestacp_UnsuspendAccount($arg) {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];

        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-unsuspend-user',
          'arg1' => $username
        );
        
 try {    
      $UnsuspendAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
      $message = ($UnsuspendAccount == 0) ? "User account has been UnSuspended successfuly\n":"Query returned error code: " .vestacp_commands_text($UnsuspendAccount). "\n";
      return array('status'=>(($UnsuspendAccount == 0) ? 'success':'error'),'message'=>$message,'response'=>vestacp_commands_text($UnsuspendAccount));
      } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
/**
 * function vestacp_ChangePassword
 *
 * Change user password on a server managed with VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-change-user-password method on VestaCP.
 */ 
function vestacp_ChangePassword($arg) {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];

        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-change-user-password',
          'arg1' => $username,
          'arg2' => $password
        );
        
 try {    
      $PwdChangedAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
      $message = ($PwdChangedAccount == 0) ? "User account Password Changed successfuly\n":"Query returned error code: " .vestacp_commands_text($PwdChangedAccount). "\n";
      return array('status'=>(($PwdChangedAccount == 0) ? 'success':'error'),'message'=>$message,'response'=>vestacp_commands_text($PwdChangedAccount));

    } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
/**
 * function vestacp_ChangePackage
 *
 * Change user package on a server managed with VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-change-user-package method on VestaCP.
 */ 
function vestacp_ChangePackage($arg) {
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];
       $package = $product['package'];

        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'returncode' => 'yes',
          'cmd' => 'v-change-user-package',
          'arg1' => $username,
          'arg2' => $package
        );
        
 try {    
      $PkgChangedAccount = vestacp_curl_query($server['hostname'],$server['port'],$vars);
      $message = ($PkgChangedAccount == 0) ? "User account Package Changed successfuly\n":"Query returned error code: " .vestacp_commands_text($PkgChangedAccount). "\n";
      return array('status'=>(($PkgChangedAccount == 0) ? 'success':'error'),'message'=>$message,'response'=>vestacp_commands_text($PkgChangedAccount));

      } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
/**
 * function vestacp_ClientArea
 *
 * Generate a form to allow for an admin from WHMSOLS to access the users account on VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-change-user-package method on VestaCP.
 */ 
function vestacp_ClientArea($params) {

    return array(
        'templatefile' => 'overview.tpl',
        'templatepath' => 'modules/servers/vestacp/templates/',
        'vars' => array(
            'serversecure' => $params["serversecure"],
            'serverhostname' => $params["serverhostname"],
            'serverport' => $params["serverport"],
            'domain' => $params["domain"],
            'test1' => 'hello',
            'test2' => 'world',
        ),
    );
}
/**
 * function vestacp_AdminLink
 *
 * Generate a form to allow for an admin from WHMSOLS to access the admin account on VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-change-user-package method on VestaCP.
 */ 
function vestacp_AdminSingleSignOn($arg){
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];
    $code = 'https://'.$server['hostname'].':'.$server['port'].'/login/';
    return $code;
  } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$arg,$e->getMessage(),$e->getTraceAsString());
         return $e->getMessage();
  }
}
/**
 * function vestacp_ServiceSingleSignOn
 *
 * Generate a link to the login page on VestaCP
 *
 * @param array required params An array of paramaters to pass to the v-change-user-package method on VestaCP.
 */ 
function vestacp_ServiceSingleSignOn($arg){
 try {    
      $server = $arg['server'];
      $product = $arg['product'];
      $username = $product['username'];
      $password = $product['password'];
    $code = 'https://'.$server['hostname'].':8083/login/';
    return $code;
  } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$arg,$e->getMessage(),$e->getTraceAsString());
         return $e->getMessage();
  }
}
 
/**
 * function vestacp_UsageUpdate
 *
 * Pull usage statistics from a VestaCP server
 *
 * @param array required params An array of paramaters to pass to the v-change-user-package method on VestaCP.
 */ 
function vestacp_UsageUpdate($arg) {
      $server = $arg['server'];
      $count = 0;
        $vars = array(
          'user' => $server["username"],
          'password' => $server["password"],
          //'hash' => $server["accesshash"],
          'cmd' => 'v-list-users',
          'arg1' => 'json'
        );
try {   
      $results = vestacp_curl_query($server['hostname'],$server['port'],$vars);
    foreach ($results AS $user=>$values) {
       $isUpdated =  update_query("hosting",array(
          "disk_usage"=>$values['U_DISK'],
          "disk_limit"=>$values['DISK_QUOTA'],
          "bandwidth_usage"=>$values['U_BANDWIDTH'],
          "bandwidth_limit"=>$values['BANDWIDTH'],
          "email_accounts_limit"=>$values['MAIL_ACCOUNTS'],
          "email_accounts_usage"=>$values['U_MAIL_ACCOUNTS'],
          "domains_limit"=>$values['WEB_DOMAINS'],
          "domains_usage"=>$values['U_WEB_DOMAINS'],
          "sub_domains_limit"=>$values['WEB_ALIASES'],
          "sub_domains_usage"=>$values['U_WEB_ALIASES'],
          "parked_domains_limit"=>$values['DNS_DOMAINS'],
          "parked_domains_usage"=>$values['U_DNS_DOMAINS'],
          "databases_limit"=>$values['DATABASES'],
          "databases_usage"=>$values['U_DATABASES'],
          "databases_disk_limit"=>$values['DISK_DB'],
          "databases_disk_usage"=>$values['U_DISK_DB'],
          "last_update"=>Carbon::now()->format('Y-m-d h:i:s'),
        ),array("server_id"=>$server['id'], "user_name"=>$user));
        if($isUpdated) $count++;
    }
     return array('status'=>'success','message'=>$count.' Accounts Stats Updated....');
    } catch (Exception $e) {
      logModule('vestacp',__FUNCTION__,$vars,$e->getMessage(),$e->getTraceAsString());
       return array('status'=>'error','message'=>$e->getMessage());
    }
    return array('status'=>'error','message'=>'');
}
?>