<?php
class gretor_com {
	private $timeout = 10; // set to zero for no timeout
	private $userAgent = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)'; 
	private $to	= '';
	private $from = '';
	private $message = '';
	private $url = 'https://sms.gretor.net/api/1.0/advanced/transactional';
	private $credit_url = 'https://sms.gretor.net/api/1.0/advanced/info';
	private $delivery_url = '';
	private $application_id = '';
	private $application_token = '';
	private $sender_id = '';

	function __construct($data=[])
	{
        $configurations = isset($data["configurations"]) ? $data["configurations"]:[];
		$this->to= isset($data['to']) ? $data['to']:'';
		$this->message = isset($data['msg']) ? $data['msg']:'';
		$this->application_token = isset($configurations["application_token"]) ? $configurations["application_token"]:'';
		$this->application_id = isset($configurations["application_id"]) ? $configurations["application_id"]:'';
		$this->sender_id = isset($configurations["sender_id"]) ? $configurations["sender_id"]:'';
		$this->from = isset($configurations["masking"]) ? $configurations["masking"]:'';
		$this->debug = isset($configurations["debug_mode"]) ? $configurations["debug_mode"]:'';
	}

	function configs(){
		return [
			'name'=>'Gretor SMS',
			'description'=>'Gretor SMS can deliver text messages to over 800 operators in over 200 countries and that number is increasing all the time.We use only reliable network providers.',
			'url'=>'https://gretor.com/sms/',
			'country'=>'World Wide',
			'version'=>'',
			"configs"=>[
                "application_token" => [
                    "Name" => "API Token",
                    "Type" => "text",
                    "Size" => "500",
                    "DivColumns"=>"12",
                    "Description" => "Enter API Token", 
                    "Default" => ""
                ],
                "application_id" =>[
                    "Name" => "API Application ID",
                    "Type" => "text",
                    "Size" => "100",
                    "DivColumns"=>"12",
                    "Description" => "Enter API Application ID", 
                    "Default" => ""
                ],
                "sender_id" =>[
                    "Name" => "Sender ID Type",
                    "Type" => "dropdown",
                    "Size" => "100",
                    "Options" => ["gSystem"=>"System number","gOwn"=>"Own number","gText"=>"Sender Text","gShort"=>"Short Code"],
                    "DivColumns"=>"12",
                    "Description" => "Select Sender ID Type", 
                    "Default" => "gSystem"
                ],
                "masking" =>[
                    "Name" => "Sender ID Value",
                    "Type" => "text",
                    "Size" => "100",
                    "DivColumns"=>"12",
                    "Description" => "Enter Sender ID Value For Example: (Own Number, Custom Text, Short Code)", 
                    "Default" => ""
                ],
            ]
		];
	}
	function send()
	{
	    try{
	        $fields = [];
	        $fields["title"] = $this->from;
	        $fields["application_token"] = $this->application_token;
	        $fields["application_id"] = $this->application_id;
	        $fields["number"] = $this->to;
	        $fields["schedule"] = "Now";
	        $fields["text"] = $this->message;
	        if($this->sender_id != "gSystem")
	        {
    	        $fields["sender_id"] = $this->sender_id;
    	        $fields["sender_id_value"] = $this->masking;
	        }
            $ch = curl_init();
            $header = array('Content-Type: application/json');
            curl_setopt($ch, CURLOPT_URL, $this->url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_POST,1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fields));
            //curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer " . $this->api_token]);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            //execute post
            $result = curl_exec($ch);
            $result = json_decode($result,true);
            if(array_key_exists("error",$result)) return ['status'=>false,'msg'=>$result["error"]." ".json_encode($result)];
            if(array_key_exists("data",$result))
            {
                $data = $result["data"];   
                $status = $data["status"];
                if($status == "accepted") return ['status'=>true,'msg'=>"SMS Sent Successfully"];
            }
            return ['status'=>false,'msg'=>"SMS Sending Failed: ".json_encode($result)];
    	} catch(Exception $e)
        {
            return ['valid'=>false,'error'=>"SMS Sending Failed: ".$e->getMessage()];
        }
	}

	function credit()
	{
	    try{
            $fields = array(
               'application_token' => $this->application_token, //destination addresses
               'application_id' => $this->application_id, //destination addresses
            );
            //open connection
            $ch = curl_init();
            $header = array('Content-Type: application/json');
            curl_setopt($ch, CURLOPT_URL, $this->credit_url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_POST,1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fields));
            //curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer " . $this->api_token]);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            //execute post
            $result = curl_exec($ch);
            $result = json_decode($result,true);
            if(array_key_exists("error",$result)) return 0;
            if(array_key_exists("data",$result)){
                $data = $result["data"];   
                $credit = $data["credit"];  
                if($credit > 0) return $credit;
            }
            return 0;
    	} catch(Exception $e)
        {
            return 0;
        }
        return 0;
	}
	function validate()
	{
	    try{
            $fields = array(
               'application_token' => $this->application_token, //destination addresses
               'application_id' => $this->application_id, //destination addresses
            );
            //open connection
            $ch = curl_init();
            $header = array('Content-Type: application/json');
            curl_setopt($ch, CURLOPT_URL, $this->credit_url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_POST,1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fields));
            //curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer " . $this->api_token]);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            //execute post
            $result = curl_exec($ch);
            $result = json_decode($result,true);
            $credit = 0;
            if(array_key_exists("data",$result)){
                $data = $result["data"];   
                $credit = $data["credit"];  
                if($credit > 0) return ['valid'=>"YES",'error'=>"API Validated Successfully and Credit Balance is:".$credit];
                return ['valid'=>"YES",'error'=>"API Validated Successfully and Credit Balance is:0"];
            }
            if(array_key_exists("error",$result)) return ['valid'=>"NO",'error'=>"API Validation failed: ".$result["result_message"]];
    	} catch(Exception $e)
        {
            return ['valid'=>"NO",'error'=>"API Validation failed: ".$e->getMessage()];
        }
        return ['valid'=>"NO",'error'=>"API Validation failed"];
	}
}