<?php 
class smstide{
	private $timeout = 10; // set to zero for no timeout
	private $userAgent = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)'; 
	private $gsmnumber	= '';
	private $from = '';
	private $message = '';
	private $url = 'https://smstide.com';
	private $credit_url = '';
	private $delivery_url = '';
	private $service = '';
	private $apikey = '';
	private $whatsapp = '';
    private $whatsapp_no = '';
	private $device = '';
	private $gateway = '';
	private $sim = '';
	private $debug = '';

	function __construct($data=[])
	{
        $configurations = isset($data["configurations"]) ? $data["configurations"]:[];
		$this->gsmnumber= isset($data['to']) ? $data['to']:'';
		$this->message = isset($data['msg']) ? $data['msg']:'';
		$this->service = isset($configurations["service"]) ? $configurations["service"]:'';
		$this->apikey = isset($configurations["apikey"]) ? $configurations["apikey"]:'';
		$this->whatsapp = isset($configurations["whatsapp"]) ? $configurations["whatsapp"]:'';
        $this->whatsapp_no = isset($configurations["whatsapp_no"]) ? $configurations["whatsapp_no"]:'';
		$this->device = isset($configurations["device"]) ? $configurations["device"]:'';
		$this->gateway = isset($configurations["gateway"]) ? $configurations["gateway"]:'';
		$this->sim = isset($configurations["sim"]) ? $configurations["sim"]:'';
		$this->debug = isset($configurations["debug_mode"]) ? $configurations["debug_mode"]:'';
	}
	
	function configs(){
		return [
			'name'=>'Sms Tide',
			'description'=>'',
			'url'=>'https://smstide.com/dashboard/tools',
			'country'=>'WorldWide',
			'version'=>'',
			"configs"=>array(
                "apikey" => array (
                                    "Name" => "API Key",
                                    "Type" => "text",
                                    "Size" => "100",
                                    "DivColumns"=>"12",
                                    "Description" => "Your SMSTide.com API key (<a href=\"https://smstide.com/dashboard/tools\" target=\"blank\">Create API Key</a>). Please make sure that everything is correct and required permissions are granted: <strong>sms_send</strong>, <strong>wa_send</strong>", 
                                    "Default" => ""
                                    ),
                "service" => array (
                                    "Name" => "Sending Service",
                                    "Type" => "dropdown",
                                    "Size" => "",
                                    "DivColumns"=>"12",
                                    "Description" => "Select the sending service, please make sure that the api key has the following permissions: <strong>sms_send</strong>, <strong>wa_send</strong>, <strong>get_credits</strong>", 
                                    "Options"=>[1=>"SMS",2=>"WhatsApp"],
                                    "Default" => 2
                                    ),
                "whatsapp" => array (
                                    "Name" => "WhatsApp Account ID",
                                    "Type" => "text",
                                    "Size" => "100",
                                    "DivColumns"=>"12",
                                    "Description" => "For WhatsApp service only. WhatsApp account ID you want to use for sending.", 
                                    "Default" => ""
                                    ),
                "whatsapp_no" => array (
                                    "Name" => "WhatsApp Mobile No",
                                    "Type" => "text",
                                    "Size" => "100",
                                    "DivColumns"=>"12",
                                    "Description" => "For Testing WhatsApp service Please enter your whatsapp mobile no with country code as 923011234567", 
                                    "Default" => ""
                                    ),
                "device" => array (
                                    "Name" => "Device Unique ID",
                                    "Type" => "text",
                                    "Size" => "100",
                                    "DivColumns"=>"12",
                                    "Description" => "For SMS service only. Linked device unique ID, please only enter this field if you are sending using one of your devices.", 
                                    "Default" => ""
                                    ),
                "gateway" => array (
                                    "Name" => "Gateway Unique ID",
                                    "Type" => "text",
                                    "Size" => "100",
                                    "DivColumns"=>"12",
                                    "Description" => "For SMS service only. Partner device unique ID or gateway ID, please only enter this field if you are sending using a partner device or third party gateway.", 
                                    "Default" => ""
                                    ),
                "sim" => array (
                                    "Name" => "Sim Slot",
                                    "Type" => "dropdown",
                                    "Size" => "",
                                    "DivColumns"=>"12",
                                    "Description" => "For SMS service only. Select the sim slot you want to use for sending the messages. This is not used for partner devices and third party gateways.", 
                                    "Options"=>[1=>"SIM 1",2=>"SIM 2"],
                                    "Default" => 1
                                    ),
                "debug_mode" => array (
                    "Name" => "Enable Debug Mode?", 
                    "Type" =>  "yesno", 
                    "Size" => "55", 
                    "Description" => "Enable debug mode to log and display errors ", 
                    "Default" => "yes", 
                ),
                            )
		];
	}
    function send()
    {
        $message = $this->message;
        if($this->service == 1)
        {
            $url = $this->url."/api/send/sms";
            $mode = empty($this->device) ? "devices":"credits";
            if($mode == "devices")
            {
                $params = [
                    "secret" => $this->apikey,
                    "mode" => "devices",
                    "device" => $this->device,
                    "phone" => $this->gsmnumber,
                    "message" => $message,
                    "sim" => $this->sim < 2 ? 1 : 2
                ];
            }else{
                    $params = [
                        "secret" => $this->apikey,
                        "mode" => "credits",
                        "gateway" => $this->gateway,
                        "phone" => $this->gsmnumber,
                        "message" => $message
                    ];
            }
        }
        else
        {
            $url = $this->url."/api/send/whatsapp";
            $params = [
                "secret" => $this->apikey,
                "account" => $this->whatsapp,
                "type" => "text",
                "recipient" => $this->gsmnumber,
                "message" => $message
            ];
        }
        $rest_request = curl_init();
        $query_string = '';
        foreach ($params as $parameter_name => $parameter_value) {
            $query_string .= '&'.$parameter_name.'='.urlencode($parameter_value);
        }
        $query_string = substr($query_string, 1);
        curl_setopt($rest_request, CURLOPT_URL, $url . '?' . $query_string);
        curl_setopt($rest_request, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($rest_request, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($rest_request, CURLOPT_SSL_VERIFYHOST, false);
        $rest_response = curl_exec($rest_request);        
        if ($rest_response === false)
        {
            curl_close($rest_request);
            return ['status'=>false,'msg'=>'curl error: ' . curl_error($rest_request)];
        }
        curl_close($rest_request);
        $array = json_decode($rest_response);
        $messageSent = false;
        if($array->status == 200)
        {
            $messageSent = true;
            $error = $array->message;
        }else{
            $error = $array->message;
        }
        return ['status'=>$messageSent,'msg'=>$error];
    }



	function credit()
	{
	    $url = $this->url."/api/get/credits?secret=".$this->apikey;
        $cURL = curl_init();
        curl_setopt($cURL, CURLOPT_URL,$url);
        curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($cURL, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($cURL, CURLOPT_SSL_VERIFYHOST, false);        
        $response = curl_exec($cURL);
        if ($response === false)
        {
            curl_close($response);
            return 0;
        }        
        curl_close($cURL);
        $result = json_decode($response);
        if($result->status == 200)
        {
            $credits = $result->data->credits;
            return $credits>0 ? $credits:0;
        }
        return 0;
	}
	function validate()
	{
	    //if(empty(.$this->apikey))
	    $url = $this->url."/api/get/credits?secret=".$this->apikey;
        $cURL = curl_init();
        curl_setopt($cURL, CURLOPT_URL,$url);
        curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($cURL, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($cURL, CURLOPT_SSL_VERIFYHOST, false);        
        $response = curl_exec($cURL);
        if ($response === false)
        {
            curl_close($response);
            return ['valid'=>'NO','error'=>'curl error: ' . curl_error($response)];
        }        
        curl_close($cURL);
        $result = json_decode($response);
        if($result->status == 200)
        {
            //$credits = $result->data->credits;
                
            $this->message = "Test Message By ".$this->whatsapp_no." During smstide module validation";
            $this->gsmnumber = $this->whatsapp_no;
            $res = $this->send();
            if($res["status"] == false) return ['valid'=>'NO','error'=>$res["msg"]];
            return ['valid'=>'YES','error'=>$result->message];
        }
        return ['valid'=>'NO','error'=>$result->message." (GET_CREDITS)"];
	}
}