<?php
/**
 * @see       https://github.com/r0mdau/jisly for the source repository
 * @copyright Copyright (c) 2018 r0mdau (https://github.com/r0mdau)
 * @license   https://github.com/r0mdau/jisly/blob/master/LICENSE.md Apache License 2.0
 */

declare(strict_types=1);

namespace Jisly;

class JislyCollection
{
    const LOGICAL_AND = "AND";
    const LOGICAL_OR = "OR";

    private $file;
    private $data;
    protected $columns="*";
    protected $where=[];

    public function __construct($dir, $file)
    {
        $this->file = "{$dir}/{$file}";
        $this->data = [];

        if (!file_exists($this->file))
        {
            touch($this->file);
        }
        else
        {
            $handle = $this->openFile('r', LOCK_SH);
            $this->hydrateData($handle);
            flock($handle, LOCK_UN);
        }
    }

    public function delete($_rid): bool
    {
        $success = false;
        $handle = $this->openFile("r+", LOCK_EX);
        $this->hydrateData($handle);

        if (isset($this->data[$_rid])) {
            unset($this->data[$_rid]);
            $success = $this->rewriteFile($handle);
        }
        flock($handle, LOCK_UN);
        return $success;
    }

    public function select($args=[]): array
    {
        $this->columns = (!empty($args["columns"])) ? $args["columns"]:"*";
        $this->where = (!empty($args["where"])) ? $args["where"]:[];
        $result = $this->SelectData();
        return $this->ReturnData('All', $result);
    }
    public function count($args=[]): int
    {
        $this->columns = (!empty($args["columns"])) ? $args["columns"]:"*";
        $this->where = (!empty($args["where"])) ? $args["where"]:[];
        $result = $this->SelectData();
        return $this->ReturnData('Count', $result);
    }
    public function selectOne($args=[]): array
    {
        $this->columns = (!empty($args["columns"])) ? $args["columns"]:"*";
        $this->where = (!empty($args["where"])) ? $args["where"]:[];
        $result = $this->SelectData();
        return $this->ReturnData('One', $result);
    }
    public function find($document = [], $logical = "OR"): array
    {
        $result = $this->search($document, $logical);
        if(!empty($result)) return $result;
        return [];
    }
    public function findOne($document = [], $logical = "OR"): array
    {
        $result = $this->search($document, $logical);
        if(!empty($result)) return $result;
        return [];
    }

    public function insert($document): bool
    {
        $FileData = $this->data;
        end($FileData);
        $rows=key($FileData);
        $rows++;
        if (empty($document["_rid"]))
        {
            $document["_rid"] = $rows;
            //$document["_rid"] = md5(uniqid((string)rand(1, 100), true));
        }
        $document["_rid"] = ($document["_rid"]>0) ? $document["_rid"]:1;
        $handle = $this->openFile('a', LOCK_EX);
        $success = $this->write($handle, $document);
        $this->data[] = (object)$document;
        flock($handle, LOCK_UN);
        return $success;
    }

    public function truncate(): bool
    {
        $handle = $this->openFile('w', LOCK_EX);
        flock($handle, LOCK_UN);
        return true;
    }

    public function update($_rid, $values): bool
    {        
        $success = false;
        $handle = $this->openFile("r+", LOCK_EX);
        $this->hydrateData($handle);
        if (isset($this->data[$_rid]))
        {            
            $record = $this->data[$_rid];
            $newRecord = [];
            foreach($record as $rk=>$rv)
            {
                if($rk<>'_rid') $newRecord[$rk] = $rv;
            }
            foreach($values as $vk=>$vv)
            {
                $newRecord[$vk] = $vv;
            }
            $newRecord["_rid"] = $_rid;
            $this->data[$_rid] = (object)$newRecord;
            $success = $this->rewriteFile($handle);
        }
        flock($handle, LOCK_UN);
        return $success;
    }

    private function hydrateData($handle): void
    {
        $this->data = [];
        while (!feof($handle)) {
            $data = fgets($handle);
            if ($data !== false) {
                $data = json_decode($data);
                if (isset($data->_rid)) {
                    $this->data[$data->_rid] = $data;
                }
            }
        }
    }

    private function openFile($mode, $lock)
    {
        $handle = fopen($this->file, $mode);
        flock($handle, $lock);
        return $handle;
    }

    private function rewriteFile($handle): bool
    {
        $success = false;
        if (ftruncate($handle, 0) === true && rewind($handle) === true) {
            foreach ($this->data as $data) {
                $success = $this->write($handle, (array)$data);
            }
        }
        return empty($this->data) ?: $success;
    }
    
    
    function searchFilter(array $array, array $search)
    {
        return array_filter($array, function ($item) use ($search) {
            return array_intersect_assoc($search, $item) === $search;
        });
    }

    function ObjToArray()
    {
        $data = json_decode(json_encode($this->data),true);
        return $data;
    }
    function validateDate($date="")
    {
        if(empty($date)) return false;
        if(is_numeric($date)) return false;
        if (preg_match("/^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/", $date)) return true;
        return false;
    }    
    function isConditionTrue($data=[])
    {
        if(empty($this->where)) return true;
        $match = 0;
        foreach($this->where as $wk=>$wv)
        {
            $kp = explode(':', $wk);
            $k = $kp[0];
            $c = (!empty($kp[1])) ? $kp[1]:'=';
            if($c == '=')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($wv);
                if($isDate)
                {   
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata == strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($data[$k] == $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }            
            if($c == '<>')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($ddata);
                if($isDate)
                {
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata <> strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($data[$k] <> $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }
            if($c == '>')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($ddata);
                if($isDate)
                {
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata > strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($data[$k] > $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }
            if($c == '>=')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($ddata);
                if($isDate)
                {
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata >= strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($ddata >= $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }
            if($c == '<')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($ddata);
                if($isDate)
                {
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata < strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($data[$k] < $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }
            if($c == '<=')
            {
                $ddata = $data[$k];
                $isDate = $this->validateDate($ddata);
                if($isDate)
                {
                    $ddata = strtotime(date('Y-m-d',strtotime($ddata)));
                    if($ddata <= strtotime($wv))
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
                else
                {
                    if($data[$k] <= $wv)
                    {
                        $match++;
                    }
                    else
                    {
                        $match--;
                    }
                }
            }
        }
        return ($match > 0) ? true:false;
    }
    function ColumnsData($data=[])
    {
        if($this->columns == "*") return $data;
        $columns = explode(', ', $this->columns);
        $n = [];
        foreach($columns as $c)
        {
            $n[$c] = @$data[$c];
        }
        $n["_rid"] = @$data["_rid"];
        return $n;
    }
    function Conditions($data=[])
    {
        $n = [];
        foreach($data as $d)
        {
            $isTrue = $this->isConditionTrue($d);
            if($isTrue)
            {
                $n[] = $this->ColumnsData($d);
            }
        }
        return $n;
    }
    function ReturnData($type="All", $result=[])
    {
        if(empty($result))
        {
            $ds = [];
        }else{
            $d = ($type == "One") ? $result[0]:$result;
            $ds = (!empty($d)) ? $d:[];
        }
        if($type == "Count") return count($ds);
        return $ds;
    }
    function SelectData()
    {   
        return $this->Conditions($this->ObjToArray());
    }
    private function search($document = null, $logical = "OR"): array
    {

            $results = [];
            $object = json_decode(json_encode($this->data),true);
            $results = $this->searchFilter($object, $document);
            if(count($results) == 1)
            {
                foreach($results as $k=>$v)
                {
                    return $v;  
                }
            }
            return $results;
            
        foreach ($this->data as $object)
        {
            $find = 0;
            foreach (array_keys($document) as $documentKey)
            {
                if (isset($object->{$documentKey}) && $object->{$documentKey} == $document[$documentKey])
                {
                    $find++;
                    if ($logical == static::LOGICAL_OR || ($logical == static::LOGICAL_AND && $find == count($document))) $results[$object->_rid] = $object;
                }
            }
        }
        return $results;
    }

    private function write($handle, $document)
    {
        return fwrite($handle, json_encode($document) . "\n") !== false;
    }
}
